@@ -24,7 +24,7 @@ KoSelection::KoSelection(QObject *parent)
     , KoShape()
     , d(new Private)
 {
-    connect(&d->selectionChangedCompressor, SIGNAL(timeout()), SIGNAL(selectionChanged()));
+    connect(d->selectionChangedCompressor, SIGNAL(timeout()), SIGNAL(selectionChanged()));
 }
 
 KoSelection::KoSelection(const KoSelection &rhs)
@@ -104,7 +104,7 @@ void KoSelection::select(KoShape *shape)
         setTransformation(QTransform());
     }
 
-    d->selectionChangedCompressor.start();
+    d->selectionChangedCompressor->start();
 }
 
 void KoSelection::deselect(KoShape *shape)
@@ -119,7 +119,7 @@ void KoSelection::deselect(KoShape *shape)
         setTransformation(d->selectedShapes.first()->absoluteTransformation());
     }
 
-    d->selectionChangedCompressor.start();
+    d->selectionChangedCompressor->start();
 }
 
 void KoSelection::deselectAll()
@@ -136,7 +136,7 @@ void KoSelection::deselectAll()
     setTransformation(QTransform());
 
     d->selectedShapes.clear();
-    d->selectionChangedCompressor.start();
+    d->selectionChangedCompressor->start();
 }
 
 int KoSelection::count() const
