@@ -1,23 +1,10 @@
 /* This file is part of the KDE project
  *
- * Copyright (C) 2006-2007, 2010 Thomas Zander <zander@kde.org>
- * Copyright (C) 2006-2008 Thorsten Zachmann <zachmann@kde.org>
- * Copyright (C) 2011 Jan Hambrecht <jaham@gmx.net>
+ * SPDX-FileCopyrightText: 2006-2007, 2010 Thomas Zander <zander@kde.org>
+ * SPDX-FileCopyrightText: 2006-2008 Thorsten Zachmann <zachmann@kde.org>
+ * SPDX-FileCopyrightText: 2011 Jan Hambrecht <jaham@gmx.net>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public License
- * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
 #include "KoShapeController.h"
@@ -29,12 +16,10 @@
 #include "KoSelection.h"
 #include "commands/KoShapeCreateCommand.h"
 #include "commands/KoShapeDeleteCommand.h"
-#include "commands/KoShapeConnectionChangeCommand.h"
 #include "KoCanvasBase.h"
 #include "KoShapeConfigWidgetBase.h"
 #include "KoShapeFactoryBase.h"
 #include "KoShape.h"
-#include "KoConnectionShape.h"
 #include <KoUnit.h>
 
 #include <QObject>
@@ -117,21 +102,6 @@ public:
 
         return resultCommand;
     }
-
-    void handleAttachedConnections(KoShape *shape, KUndo2Command *parentCmd) {
-        foreach (KoShape *dependee, shape->dependees()) {
-            KoConnectionShape *connection = dynamic_cast<KoConnectionShape*>(dependee);
-            if (connection) {
-                if (shape == connection->firstShape()) {
-                    new KoShapeConnectionChangeCommand(connection, KoConnectionShape::StartHandle,
-                                                       shape, connection->firstConnectionId(), 0, -1, parentCmd);
-                } else if (shape == connection->secondShape()) {
-                    new KoShapeConnectionChangeCommand(connection, KoConnectionShape::EndHandle,
-                                                       shape, connection->secondConnectionId(), 0, -1, parentCmd);
-                }
-            }
-        }
-    }
 };
 
 KoShapeController::KoShapeController(KoCanvasBase *canvas, KoShapeControllerBase *shapeController)
@@ -175,9 +145,6 @@ KUndo2Command* KoShapeController::removeShape(KoShape *shape, KUndo2Command *par
 KUndo2Command* KoShapeController::removeShapes(const QList<KoShape*> &shapes, KUndo2Command *parent)
 {
     KUndo2Command *cmd = new KoShapeDeleteCommand(d->shapeController, shapes, parent);
-    foreach (KoShape *shape, shapes) {
-        d->handleAttachedConnections(shape, cmd);
-    }
     return cmd;
 }
 
