@@ -1,20 +1,7 @@
 /* This file is part of the KDE project
- * Copyright (C) 2008-2009 Jan Hambrecht <jaham@gmx.net>
+ * SPDX-FileCopyrightText: 2008-2009 Jan Hambrecht <jaham@gmx.net>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public License
- * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
 #include "KoSnapGuide.h"
@@ -27,19 +14,16 @@
 #include <KoCanvasBase.h>
 
 #include <QPainter>
+#include <QPainterPath>
 
 #include <math.h>
-
-template <class T>
-inline QSharedPointer<T> toQShared(T* ptr) {
-    return QSharedPointer<T>(ptr);
-}
+#include "kis_pointer_utils.h"
 
 class Q_DECL_HIDDEN KoSnapGuide::Private
 {
 public:
     Private(KoCanvasBase *parentCanvas)
-        : canvas(parentCanvas), editedShape(0), currentStrategy(0),
+        : canvas(parentCanvas), additionalEditedShape(0), currentStrategy(0),
         active(true),
         snapDistance(10)
     {
@@ -51,7 +35,7 @@ public:
     }
 
     KoCanvasBase *canvas;
-    KoShape *editedShape;
+    KoShape *additionalEditedShape;
 
     typedef QSharedPointer<KoSnapStrategy> KoSnapStrategySP;
     typedef QList<KoSnapStrategySP> StrategiesList;
@@ -80,20 +64,18 @@ KoSnapGuide::~KoSnapGuide()
 {
 }
 
-void KoSnapGuide::setEditedShape(KoShape *shape)
+void KoSnapGuide::setAdditionalEditedShape(KoShape *shape)
 {
-    d->editedShape = shape;
+    d->additionalEditedShape = shape;
 }
 
-KoShape *KoSnapGuide::editedShape() const
+KoShape *KoSnapGuide::additionalEditedShape() const
 {
-    return d->editedShape;
+    return d->additionalEditedShape;
 }
 
 void KoSnapGuide::enableSnapStrategy(Strategy type, bool value)
 {
-    if ((d->usedStrategies & type) ==  value) return;
-
     if (value) {
         d->usedStrategies |= type;
     } else {
@@ -189,9 +171,6 @@ QPointF KoSnapGuide::snap(const QPointF &mousePosition, Qt::KeyboardModifiers mo
     foreach (Private::KoSnapStrategySP strategy, d->strategies) {
         if (d->usedStrategies & strategy->type() ||
             strategy->type() == GridSnapping ||
-            strategy->type() == GuideLineSnapping ||
-            strategy->type() == DocumentBoundsSnapping ||
-            strategy->type() == DocumentCenterSnapping ||
             strategy->type() == CustomSnapping) {
 
             if (! strategy->snap(mousePosition, &proxy, maxSnapDistance))
@@ -233,12 +212,12 @@ void KoSnapGuide::paint(QPainter &painter, const KoViewConverter &converter)
 
     painter.setBrush(Qt::NoBrush);
 
-    QPen whitePen(Qt::white);
+    QPen whitePen(Qt::white, 0);
     whitePen.setStyle(Qt::SolidLine);
     painter.setPen(whitePen);
     painter.drawPath(decoration);
 
-    QPen redPen(Qt::red);
+    QPen redPen(Qt::red, 0);
     redPen.setStyle(Qt::DotLine);
     painter.setPen(redPen);
     painter.drawPath(decoration);
@@ -272,7 +251,7 @@ QList<KoShape*> KoSnapGuide::ignoredShapes() const
 void KoSnapGuide::reset()
 {
     d->currentStrategy.clear();
-    d->editedShape = 0;
+    d->additionalEditedShape = 0;
     d->ignoredPoints.clear();
     d->ignoredShapes.clear();
     // remove all custom strategies
