@@ -5,13 +5,14 @@
 */
 #include "TestPointRemoveCommand.h"
 
-#include <QPainterPath>
-#include "KoPathShape.h"
 #include "KoPathPointRemoveCommand.h"
+#include "KoPathShape.h"
 #include "KoShapeController.h"
 #include <MockShapes.h>
-#include <sdk/tests/testflake.h>
+#include <QPainterPath>
 #include <simpletest.h>
+#include <testflake.h>
+
 
 void TestPointRemoveCommand::redoUndoPointRemove()
 {
