@@ -20,14 +20,15 @@ class KoPathTool;
 class KoPathPointMoveStrategy : public KoInteractionStrategy
 {
 public:
-    KoPathPointMoveStrategy(KoPathTool *tool, const QPointF &pos);
+    KoPathPointMoveStrategy(KoPathTool *tool, const QPointF &mousePosition, const QPointF &pointPosition);
     ~KoPathPointMoveStrategy() override;
     void handleMouseMove(const QPointF &mouseLocation, Qt::KeyboardModifiers modifiers) override;
     void finishInteraction(Qt::KeyboardModifiers modifiers) override;
     KUndo2Command *createCommand() override;
 
 private:
-    QPointF m_originalPosition;
+    QPointF m_startMousePosition;
+    QPointF m_startPointPosition;
     /// the accumulated point move amount
     QPointF m_move;
     /// pointer to the path tool
