@@ -1,19 +1,7 @@
 /*
- *  Copyright (c) 2004,2006-2007 Cyrille Berger <cberger@cberger.net>
+ *  SPDX-FileCopyrightText: 2004, 2006-2007 Cyrille Berger <cberger@cberger.net>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "kis_base_processor.h"
@@ -24,6 +12,7 @@
 #include "filter/kis_filter_configuration.h"
 #include "kis_paint_device.h"
 #include "kis_selection.h"
+#include <KisGlobalResourcesInterface.h>
 
 
 class KisBaseProcessorConfigurationFactory : public KisSerializableConfigurationFactory
@@ -31,11 +20,12 @@ class KisBaseProcessorConfigurationFactory : public KisSerializableConfiguration
 public:
     KisBaseProcessorConfigurationFactory(KisBaseProcessor* _generator) : m_generator(_generator) {}
     ~KisBaseProcessorConfigurationFactory() override {}
+
     KisSerializableConfigurationSP createDefault() override {
-        return m_generator->defaultConfiguration();
+        return m_generator->defaultConfiguration(KisGlobalResourcesInterface::instance());
     }
     KisSerializableConfigurationSP create(const QDomElement& e) override {
-        KisSerializableConfigurationSP config = m_generator->factoryConfiguration();
+        KisSerializableConfigurationSP config = m_generator->factoryConfiguration(KisGlobalResourcesInterface::instance());
         config->fromXML(e);
         return config;
     }
@@ -85,14 +75,14 @@ KisBaseProcessor::~KisBaseProcessor()
     delete d;
 }
 
-KisFilterConfigurationSP KisBaseProcessor::factoryConfiguration() const
+KisFilterConfigurationSP KisBaseProcessor::factoryConfiguration(KisResourcesInterfaceSP resourcesInterface) const
 {
-    return new KisFilterConfiguration(id(), 1);
+    return new KisFilterConfiguration(id(), 1, resourcesInterface);
 }
 
-KisFilterConfigurationSP KisBaseProcessor::defaultConfiguration() const
+KisFilterConfigurationSP KisBaseProcessor::defaultConfiguration(KisResourcesInterfaceSP resourcesInterface) const
 {
-    return factoryConfiguration();
+    return factoryConfiguration(resourcesInterface);
 }
 
 KisConfigWidget * KisBaseProcessor::createConfigurationWidget(QWidget *, const KisPaintDeviceSP, bool) const
