@@ -1,19 +1,7 @@
 /*
- *  Copyright (c) 2014 Dmitry Kazakov <dimula73@gmail.com>
+ *  SPDX-FileCopyrightText: 2014 Dmitry Kazakov <dimula73@gmail.com>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #ifndef __KIS_CAGE_TRANSFORM_WORKER_H
@@ -28,7 +16,7 @@ class QImage;
 class KRITAIMAGE_EXPORT KisCageTransformWorker
 {
 public:
-    KisCageTransformWorker(KisPaintDeviceSP dev,
+    KisCageTransformWorker(const QRect &deviceNonDefaultRegion,
                            const QVector<QPointF> &origCage,
                            KoUpdater *progress,
                            int pixelPrecision = 8);
@@ -43,7 +31,7 @@ public:
 
     void prepareTransform();
     void setTransformedCage(const QVector<QPointF> &transformedCage);
-    void run();
+    void run(KisPaintDeviceSP srcDevice, KisPaintDeviceSP dstDevice);
 
     QRect approxChangeRect(const QRect &rc);
     QRect approxNeedRect(const QRect &rc, const QRect &fullBounds);
