@@ -52,8 +52,6 @@
 #include "kis_keyframe_channel.h"
 #include "kis_stroke_strategy.h"
 #include "kis_simple_stroke_strategy.h"
-#include "kis_image_barrier_locker.h"
-
 
 #include "kis_undo_stores.h"
 #include "kis_legacy_undo_adapter.h"
@@ -173,6 +171,30 @@ public:
                         [this] () {
                 undoStore->purgeRedoState();
             });
+
+            scheduler.setPostSyncLod0GUIPlaneRequestForResumeCallback(
+                        [this] () {
+
+                KIS_SAFE_ASSERT_RECOVER_NOOP(q->currentLevelOfDetail() == 0);
+
+                bool addedUIUpdateRequestSuccessfully = false;
+
+                for (auto it = std::make_reverse_iterator(projectionUpdatesFilters.end());
+                     it != std::make_reverse_iterator(projectionUpdatesFilters.begin());
+                     ++it) {
+
+                    KisSuspendProjectionUpdatesStrokeStrategy::SuspendUpdatesFilterInterface *iface =
+                        dynamic_cast<KisSuspendProjectionUpdatesStrokeStrategy::SuspendUpdatesFilterInterface*>(it->data());
+
+                    if (iface) {
+                        iface->addExplicitUIUpdateRect(q->bounds());
+                        addedUIUpdateRequestSuccessfully = true;
+                        break;
+                    }
+                }
+
+                KIS_SAFE_ASSERT_RECOVER_NOOP(addedUIUpdateRequestSuccessfully);
+            });
         }
 
         connect(q, SIGNAL(sigImageModified()), KisMemoryStatisticsServer::instance(), SLOT(notifyImageChanged()));
@@ -192,7 +214,7 @@ public:
          * Firstly we need to disconnect the nodes from the image,
          * because some of the nodes (e.g. KisGroupLayer) may
          * request the image back via defaultBouds() and/or
-         * animationInyterface()
+         * animationInterface()
          */
         if (rootLayer->image() == q) {
             rootLayer->setImage(0);
@@ -235,6 +257,7 @@ public:
     bool isolateGroup;
 
     bool wrapAroundModePermitted = false;
+    WrapAroundAxis wrapAroundModeAxis = WRAPAROUND_BOTH;
 
     QScopedPointer<KisUndoStore> undoStore;
     KisLegacyUndoAdapter legacyUndoAdapter;
@@ -380,7 +403,7 @@ void KisImage::copyFromImageImpl(const KisImage &rhs, int policy)
     KIS_ASSERT_RECOVER_RETURN(bool(policy & REPLACE) != bool(policy & CONSTRUCT));
 
     /**
-     * We should replace the root before amitting any signals, because some of the layers
+     * We should replace the root before emitting any signals, because some of the layers
      * may be subscribed to sigSizeChanged() signal (e.g. KisSelectionBasedLayer). So the
      * old layers should be fully detached before we actually emit this signal.
      *
@@ -575,7 +598,6 @@ void KisImage::aboutToRemoveANode(KisNode *parent, int index)
 void KisImage::nodeChanged(KisNode* node)
 {
     KisNodeGraphListener::nodeChanged(node);
-    requestStrokeEnd();
     m_d->signalRouter.emitNodeChanged(node);
 }
 
@@ -613,7 +635,8 @@ void KisImage::setOverlaySelectionMask(KisSelectionMaskSP mask)
             }
 
             if (oldMask) {
-                m_image->m_d->rootLayer->setDirtyDontResetAnimationCache(oldMask->extent());
+                const QRect oldMaskRect = oldMask->graphListener() ? oldMask->extent() : m_image->bounds();
+                m_image->m_d->rootLayer->setDirtyDontResetAnimationCache(oldMaskRect);
             }
 
             if (newMask) {
@@ -722,7 +745,7 @@ QString KisImage::nextLayerName(const QString &_baseName) const
 
     // special case if there is only root node
     if (numLayers == 1) {
-        return i18n("Background");
+        return i18nc("Name for the bottom-most layer in the layerstack", "Background");
     }
 
     if (baseName.isEmpty()) {
@@ -1552,12 +1575,12 @@ qint32 KisImage::nHiddenLayers() const
 
 qint32 KisImage::nChildLayers() const
 {
-    QStringList list;
-    list << "KisLayer";
+    const QStringList list = {"KisLayer"};
 
     KoProperties koProperties;
     KisCountVisitor visitor(list, koProperties);
-    for (auto childNode : m_d->rootLayer->childNodes(list, koProperties)) {
+    const QList<KisNodeSP> childNodes = m_d->rootLayer->childNodes(list, koProperties);
+    for (KisNodeSP childNode: childNodes) {
         childNode->accept(visitor);
     }
     return visitor.count();
@@ -1641,13 +1664,12 @@ QImage KisImage::convertToQImage(const QSize& scaledImageSize, const KoColorProf
         return convertToQImage(size(), profile).scaled(scaledImageSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
     }
 
-    QPointer<KoUpdater> updater = new KoDummyUpdater();
+    KoDummyUpdaterHolder updaterHolder;
+    QPointer<KoUpdater> updater = updaterHolder.updater();
 
     KisTransformWorker worker(dev, scaleX, scaleY, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, updater, KisFilterStrategyRegistry::instance()->value("Bicubic"));
     worker.run();
 
-    delete updater;
-
     return dev->convertToQImage(profile);
 }
 void KisImage::notifyLayersChanged()
@@ -1717,7 +1739,7 @@ void KisImage::setDefaultProjectionColor(const KoColor &color)
 KoColor KisImage::defaultProjectionColor() const
 {
     KIS_ASSERT_RECOVER(m_d->rootLayer) {
-        return KoColor(Qt::transparent, m_d->colorSpace);
+        return KoColor::createTransparent(m_d->colorSpace);
     }
 
     return m_d->rootLayer->defaultProjectionColor();
@@ -1727,7 +1749,7 @@ void KisImage::setRootLayer(KisGroupLayerSP rootLayer)
 {
     emit sigInternalStopIsolatedModeRequested();
 
-    KoColor defaultProjectionColor(Qt::transparent, m_d->colorSpace);
+    KoColor defaultProjectionColor = KoColor::createTransparent(m_d->colorSpace);
 
     if (m_d->rootLayer) {
         m_d->rootLayer->setGraphListener(0);
@@ -1858,7 +1880,7 @@ bool KisImage::startIsolatedMode(KisNodeSP node, bool isolateLayer, bool isolate
     if ((isolateLayer || isolateGroup) == false) return false;
 
     /**
-     * Isolation of trnsform masks is not possible, so we should
+     * Isolation of transform masks is not possible, so we should
      * not allow that
      */
     if (!node->projection()) return false;
@@ -2347,7 +2369,7 @@ void KisImage::requestProjectionUpdate(KisNode *node, const QVector<QRect> &rect
 
         const QRect boundRect = effectiveLodBounds();
         Q_FOREACH (const QRect &rc, rects) {
-            KisWrappedRect splitRect(rc, boundRect);
+            KisWrappedRect splitRect(rc, boundRect, m_d->wrapAroundModeAxis);
             allSplitRects.append(splitRect);
         }
 
@@ -2378,8 +2400,9 @@ KisNode *KisImage::graphOverlayNode() const
 void KisImage::keyframeChannelHasBeenAdded(KisNode *node, KisKeyframeChannel *channel)
 {
     Q_UNUSED(node);
+
     channel->connect(channel, SIGNAL(sigAddedKeyframe(const KisKeyframeChannel*, int)), m_d->animationInterface, SIGNAL(sigKeyframeAdded(const KisKeyframeChannel*, int)), Qt::UniqueConnection);
-    channel->connect(channel, SIGNAL(sigRemovingKeyframe(const KisKeyframeChannel*,int)), m_d->animationInterface, SIGNAL(sigKeyframeRemoved(const KisKeyframeChannel*, int)), Qt::UniqueConnection);
+    channel->connect(channel, SIGNAL(sigKeyframeHasBeenRemoved(const KisKeyframeChannel*,int)), m_d->animationInterface, SIGNAL(sigKeyframeRemoved(const KisKeyframeChannel*, int)), Qt::UniqueConnection);
 }
 
 void KisImage::keyframeChannelAboutToBeRemoved(KisNode *node, KisKeyframeChannel *channel)
@@ -2387,7 +2410,7 @@ void KisImage::keyframeChannelAboutToBeRemoved(KisNode *node, KisKeyframeChannel
     Q_UNUSED(node);
 
     channel->disconnect(channel, SIGNAL(sigAddedKeyframe(const KisKeyframeChannel*, int)), m_d->animationInterface, SIGNAL(sigKeyframeAdded(const KisKeyframeChannel*, int)));
-    channel->disconnect(channel, SIGNAL(sigRemovingKeyframe(const KisKeyframeChannel*, int)), m_d->animationInterface, SIGNAL(sigKeyframeRemoved(const KisKeyframeChannel*, int)));
+    channel->disconnect(channel, SIGNAL(sigKeyframeHasBeenRemoved(const KisKeyframeChannel*, int)), m_d->animationInterface, SIGNAL(sigKeyframeRemoved(const KisKeyframeChannel*, int)));
 }
 
 QList<KisLayerCompositionSP> KisImage::compositions()
@@ -2470,6 +2493,17 @@ void KisImage::setWrapAroundModePermitted(bool value)
     }
 }
 
+void KisImage::setWrapAroundModeAxis(WrapAroundAxis value)
+{
+    m_d->wrapAroundModeAxis = value;
+}
+
+WrapAroundAxis KisImage::wrapAroundModeAxis() const
+{
+    return m_d->wrapAroundModeAxis;
+}
+
+
 bool KisImage::wrapAroundModePermitted() const
 {
     return m_d->wrapAroundModePermitted;
