@@ -27,6 +27,7 @@
 
 #ifdef Q_OS_MACOS
 #include <errno.h>
+#include "KisMacosSecurityBookmarkManager.h"
 #endif
 
 KisImageConfig::KisImageConfig(bool readOnly)
@@ -237,10 +238,24 @@ QString KisImageConfig::safelyGetWritableTempLocation(const QString &suffix, con
     // nice to the user. having a clearly named swap file in the home folder is
     // much nicer to Krita's users.
 
+    // NOTE: QStandardPaths::AppLocalDataLocation on macos sandboxed envs
+    // does not return writable locations at all times, using QDir static methods
+    // will always return locations inside the sandbox Container
+
     // furthermore, this is just a default and swapDir can always be configured
     // to another location.
 
-    QString swap = QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation) + '/' + suffix;
+    QString swap;
+
+    KisMacosSecurityBookmarkManager *bookmarkmngr = KisMacosSecurityBookmarkManager::instance();
+    if ( bookmarkmngr->isSandboxed() ) {
+        QDir sandboxHome = QDir::home();
+        if (sandboxHome.cd("tmp")) {
+            swap = sandboxHome.path();
+        }
+    } else {
+        swap = QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation) + '/' + suffix;
+    }
 #else
     Q_UNUSED(suffix);
     QString swap = QDir::tempPath();
@@ -255,6 +270,7 @@ QString KisImageConfig::safelyGetWritableTempLocation(const QString &suffix, con
 
     QString chosenLocation;
     QStringList proposedSwapLocations;
+
     proposedSwapLocations << swap;
     proposedSwapLocations << QDir::tempPath();
     proposedSwapLocations << QDir::homePath();
@@ -325,8 +341,10 @@ int KisImageConfig::onionSkinOpacity(int offset) const
 
     if (value < 0) {
         const int num = numberOfOnionSkins();
-        const qreal dx = qreal(qAbs(offset)) / num;
-        value = 0.7 * exp(-pow2(dx) / 0.5) * 255;
+        if (num > 0) {
+            const qreal dx = qreal(qAbs(offset)) / num;
+            value = 0.7 * exp(-pow2(dx) / 0.5) * 255;
+        }
     }
 
     return value;
@@ -508,6 +526,10 @@ KisProofingConfigurationSP KisImageConfig::defaultProofingconfiguration()
 
 void KisImageConfig::setDefaultProofingConfig(const KoColorSpace *proofingSpace, int proofingIntent, bool blackPointCompensation, KoColor warningColor, double adaptationState)
 {
+    if (!proofingSpace || !proofingSpace->profile()) {
+        return;
+    }
+
     m_config.writeEntry("defaultProofingProfileName", proofingSpace->profile()->name());
     m_config.writeEntry("defaultProofingProfileModel", proofingSpace->colorModelId().id());
     m_config.writeEntry("defaultProofingProfileDepth", proofingSpace->colorDepthId().id());
@@ -637,6 +659,16 @@ void KisImageConfig::setAnimationCacheRegionOfInterestMargin(qreal value)
     m_config.writeEntry("animationCacheRegionOfInterestMargin", value);
 }
 
+qreal KisImageConfig::selectionOutlineOpacity(bool defaultValue) const
+{
+    return defaultValue ? 1.0 : m_config.readEntry("selectionOutlineOpacity", 1.0);
+}
+
+void KisImageConfig::setSelectionOutlineOpacity(qreal value)
+{
+    m_config.writeEntry("selectionOutlineOpacity", value);
+}
+
 QColor KisImageConfig::selectionOverlayMaskColor(bool defaultValue) const
 {
     QColor def(255, 0, 0, 128);
@@ -663,6 +695,16 @@ int KisImageConfig::maxMaskingBrushSize() const
     return qMin(15000, 3 * maxBrushSize());
 }
 
+bool KisImageConfig::renameMergedLayers(bool defaultValue) const
+{
+    return defaultValue ? true : m_config.readEntry("renameMergedLayers", true);
+}
+
+void KisImageConfig::setRenameMergedLayers(bool value)
+{
+    m_config.writeEntry("renameMergedLayers", value);
+}
+
 QString KisImageConfig::exportConfigurationXML(const QString &exportConfigId, bool defaultValue) const
 {
     return (defaultValue ? QString() : m_config.readEntry("ExportConfiguration-" + exportConfigId, QString()));
