@@ -57,6 +57,10 @@
 
 namespace KisLayerUtils {
 
+namespace Private {
+    void refreshHiddenAreaAsync(KisImageSP image, KisNodeSP rootNode, const QRect &preparedArea, const QRect &extraUpdateRect);
+}
+
     void fetchSelectionMasks(KisNodeList mergedNodes, QVector<KisSelectionMaskSP> &selectionMasks)
     {
         foreach (KisNodeSP node, mergedNodes) {
@@ -482,26 +486,30 @@ namespace KisLayerUtils {
         bool m_skipIfDstIsGroup;
     };
 
-    struct RefreshHiddenAreas : public KUndo2Command {
+    struct RefreshHiddenAreas : public KisCommandUtils::AggregateCommand {
+        struct refresh_entire_image_t {};
+        static constexpr refresh_entire_image_t refresh_entire_image{};
+
         RefreshHiddenAreas(MergeDownInfoBaseSP info) : m_image(info->image), m_nodes(info->allSrcNodes()) {}
-        RefreshHiddenAreas(KisImageSP image, KisNodeList nodes) : m_image(image), m_nodes(nodes) {}
+        RefreshHiddenAreas(MergeDownInfoBaseSP info, refresh_entire_image_t) : m_image(info->image), m_nodes(info->allSrcNodes()), m_extraUpdateRect(info->image->bounds()) {}
         RefreshHiddenAreas(KisImageSP image, KisNodeSP node) : m_image(image), m_nodes() {
             m_nodes << node;
         }
 
-        void redo() override {
+        void populateChildCommands() override {
             KisImageAnimationInterface *interface = m_image->animationInterface();
             const QRect preparedRect = !interface->externalFrameActive() ?
                 m_image->bounds() : QRect();
 
             foreach (KisNodeSP node, m_nodes) {
-                refreshHiddenAreaAsync(m_image, node, preparedRect);
+                Private::refreshHiddenAreaAsync(m_image, node, preparedRect, m_extraUpdateRect);
             }
         }
 
     private:
         KisImageWSP m_image;
         KisNodeList m_nodes;
+        QRect m_extraUpdateRect;
     };
 
     struct RefreshDelayedUpdateLayers : public KUndo2Command {
@@ -1261,7 +1269,7 @@ namespace KisLayerUtils {
         AddNewFrame(MergeDownInfoBaseSP info, int frame) : m_frame(frame), m_sampledNodes(info->allSrcNodes()), m_mergeInfo(info) {}
 
         void populateChildCommands() override {
-            KUndo2Command *cmd = new KisCommandUtils::SkipFirstRedoWrapper();
+            KUndo2Command *cmd = new KUndo2Command;
             KisNodeSP node = m_node ? m_node : m_mergeInfo->dstNode;
             KisKeyframeChannel *channel = node->getKeyframeChannel(KisKeyframeChannel::Raster.id(), true);
             channel->addKeyframe(m_frame, cmd);
@@ -1412,11 +1420,23 @@ namespace KisLayerUtils {
             applicator.applyCommand(new KUndo2Command(), KisStrokeJobData::BARRIER);
 
             if (info->frames.size() > 0) {
+                /**
+                 * Save the original time before we start switching is with
+                 * asynchronous SwitchFrameCommand.
+                 */
+                const int currentTimeOnStart = info->image->animationInterface()->currentTime();
+
                 foreach (int frame, info->frames) {
                     applicator.applyCommand(new SwitchFrameCommand(info->image, frame, false, info->storage));
 
                     applicator.applyCommand(new AddNewFrame(info, frame));
-                    applicator.applyCommand(new RefreshHiddenAreas(info));
+                    /**
+                     * When switching frames we need to update the entire image, not
+                     * only the **new** extent of the layer, hence we pass `refresh_entire_image`
+                     * to the command to make sure that the entire image bounds rect is added
+                     * to the update rect
+                     */
+                    applicator.applyCommand(new RefreshHiddenAreas(info, RefreshHiddenAreas::refresh_entire_image));
                     applicator.applyCommand(new RefreshDelayedUpdateLayers(info), KisStrokeJobData::BARRIER);
 
                     /**
@@ -1428,7 +1448,7 @@ namespace KisLayerUtils {
                      * See https://bugs.kde.org/show_bug.cgi?id=475550
                      */
                     const bool skipMergingSourceLayer = !layer->isAnimated() &&
-                            frame != info->image->animationInterface()->currentTime();
+                            frame != currentTimeOnStart;
 
                     applicator.applyCommand(new MergeLayers(info, skipMergingSourceLayer), KisStrokeJobData::BARRIER);
 
@@ -1823,7 +1843,13 @@ namespace KisLayerUtils {
                     applicator.applyCommand(new SwitchFrameCommand(info->image, frame, false, info->storage));
 
                     applicator.applyCommand(new AddNewFrame(info, frame));
-                    applicator.applyCommand(new RefreshHiddenAreas(info));
+                    /**
+                     * When switching frames we need to update the entire image, not
+                     * only the **new** extent of the layer, hence we pass `refresh_entire_image`
+                     * to the command to make sure that the entire image bounds rect is added
+                     * to the update rect
+                     */
+                    applicator.applyCommand(new RefreshHiddenAreas(info, RefreshHiddenAreas::refresh_entire_image));
                     applicator.applyCommand(new RefreshDelayedUpdateLayers(info), KisStrokeJobData::BARRIER);
                     applicator.applyCommand(new MergeLayersMultiple(info), KisStrokeJobData::BARRIER);
 
@@ -2121,8 +2147,9 @@ namespace KisLayerUtils {
     }
     }
 
-    void refreshHiddenAreaAsync(KisImageSP image, KisNodeSP rootNode, const QRect &preparedArea) {
-        QRect realNodeRect = Private::realNodeChangeRect(rootNode);
+namespace Private {
+    void refreshHiddenAreaAsync(KisImageSP image, KisNodeSP rootNode, const QRect &preparedArea, const QRect &extraUpdateRect) {
+        QRect realNodeRect = Private::realNodeChangeRect(rootNode) | extraUpdateRect;
         if (!preparedArea.contains(realNodeRect)) {
 
             QRegion dirtyRegion = realNodeRect;
@@ -2135,6 +2162,11 @@ namespace KisLayerUtils {
             }
         }
     }
+} // namespace Private
+
+    void refreshHiddenAreaAsync(KisImageSP image, KisNodeSP rootNode, const QRect &preparedArea) {
+        Private::refreshHiddenAreaAsync(image, rootNode, preparedArea, QRect());
+    }
 
     QRect recursiveTightNodeVisibleBounds(KisNodeSP rootNode)
     {
