@@ -1,21 +1,9 @@
 /*
- *  Copyright (c) 2006 Boudewijn Rempt <boud@valdyas.org>
- *            (c) 2009 Dmitry Kazakov <dimula73@gmail.com>
+ *  SPDX-FileCopyrightText: 2006 Boudewijn Rempt <boud@valdyas.org>
+ *  SPDX-FileCopyrightText: 2009 Dmitry Kazakov <dimula73@gmail.com>
  *
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.b
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 #ifndef _KIS_MASK_
 #define _KIS_MASK_
@@ -42,7 +30,7 @@
  are just paint devices, too.
 
  Masks should show up in the layerbox as sub-layers for the layer they
- are associated with and be ccp'able and draggable to other layers.
+ are associated with and be cut/copy-able and draggable to other layers.
 
  Examples of masks are:
 
@@ -188,6 +176,7 @@ public:
     QRect needRect(const QRect &rect, PositionToFilthy pos = N_FILTHY) const override;
     QRect changeRect(const QRect &rect, PositionToFilthy pos = N_FILTHY) const override;
     QImage createThumbnail(qint32 w, qint32 h, Qt::AspectRatioMode aspectRatioMode = Qt::IgnoreAspectRatio) override;
+    int thumbnailSeqNo() const override;
 
     void testingInitSelection(const QRect &rect, KisLayerSP parentLayer);
 
@@ -220,6 +209,7 @@ protected:
     virtual bool paintsOutsideSelection() const;
 
     KisKeyframeChannel *requestKeyframeChannel(const QString &id) override;
+    bool supportsKeyframeChannel(const QString &id) override;
 
     void baseNodeChangedCallback() override;
 
