@@ -23,12 +23,13 @@ bool KisDropAllProjectionUpdatesFilter::filter(KisImage *image, KisNode *node, c
     return true;
 }
 
-bool KisDropAllProjectionUpdatesFilter::filterRefreshGraph(KisImage *image, KisNode *node, const QVector<QRect> &rects, const QRect &cropRect)
+bool KisDropAllProjectionUpdatesFilter::filterRefreshGraph(KisImage *image, KisNode *node, const QVector<QRect> &rects, const QRect &cropRect, KisUpdatesFacade::UpdateFlags flags)
 {
     Q_UNUSED(image);
     Q_UNUSED(node);
     Q_UNUSED(rects);
     Q_UNUSED(cropRect);
+    Q_UNUSED(flags);
     return true;
 }
 
