@@ -1,19 +1,7 @@
 /*
- *  Copyright (c) 2011 Dmitry Kazakov <dimula73@gmail.com>
+ *  SPDX-FileCopyrightText: 2011 Dmitry Kazakov <dimula73@gmail.com>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #ifndef __KIS_STROKE_JOB_STRATEGY_H
@@ -55,6 +43,9 @@ public:
     bool isCancellable() const;
     void setCancellable(bool value);
 
+    int levelOfDetailOverride() const;
+    void setLevelOfDetailOverride(int value);
+
 protected:
     KisStrokeJobData(const KisStrokeJobData &rhs);
 
@@ -62,6 +53,7 @@ private:
     Sequentiality m_sequentiality;
     Exclusivity m_exclusivity;
     bool m_isCancellable;
+    int m_levelOfDetailOverride;
 };
 
 
