@@ -170,9 +170,22 @@ void KisTransformMask::slotDelayedStaticUpdate()
     if (!parentLayer) return;
 
     KisImageSP image = parentLayer->image();
-    if (image) {
-        image->addSpontaneousJob(new KisRecalculateTransformMaskJob(this));
+
+    if (!image) {
+        return;
     }
+
+    /**
+     * Don't try to start a regeneration stroke while image
+     * is locked. It may happen on loading, when all necessary
+     * conversions are not yet finished.
+     */
+    if (image->locked()) {
+        m_d->updateSignalCompressor.start();
+        return;
+    }
+
+    image->addSpontaneousJob(new KisRecalculateTransformMaskJob(this));
 }
 
 KisPaintDeviceSP KisTransformMask::buildPreviewDevice()
@@ -239,7 +252,7 @@ void KisTransformMask::overrideStaticCacheDevice(KisPaintDeviceSP device)
     m_d->staticCacheIsOverridden = bool(device);
 }
 
-void KisTransformMask::recaclulateStaticImage()
+void KisTransformMask::recalculateStaticImage()
 {
     /**
      * Note: this function must be called from within the scheduler's
@@ -272,6 +285,26 @@ void KisTransformMask::recaclulateStaticImage()
     QRect requestedRect =
         parentLayer->changeRect(parentLayer->original()->exactBounds());
 
+
+    QRect bounds;
+
+    {
+        KisNodeSP parentNode = parent();
+
+        if (parentNode) {
+            bounds = parentNode->original()->defaultBounds()->bounds();
+        } else {
+            bounds = QRect(0,0,777,777);
+            warnKrita << "WARNING: transform mask has no parent (change rect)."
+                      << "Cannot run safe transformations."
+                      << "Will limit bounds to" << ppVar(bounds);
+        }
+    }
+
+    const QRect limitingRect = KisAlgebra2D::blowRect(bounds, m_d->offBoundsReadArea);
+
+    requestedRect &= limitingRect;
+
     // force reset parent layer's projection, because we might have changed
     // our mask parameters and going to write to some other area
     parentLayer->projection()->clear();
@@ -413,7 +446,6 @@ QRect KisTransformMask::needRect(const QRect& rect, PositionToFilthy pos) const
      * on the higher/lower level
      */
     if (rect.isEmpty()) return rect;
-    if (!m_d->params->isAffine()) return rect;
 
     QRect bounds;
     QRect interestRect;
