@@ -1,19 +1,7 @@
 /*
- *  Copyright (c) 2015 Dmitry Kazakov <dimula73@gmail.com>
+ *  SPDX-FileCopyrightText: 2015 Dmitry Kazakov <dimula73@gmail.com>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #ifndef __KIS_LAYER_STYLE_FILTER_PROJECTION_PLANE_H
@@ -25,6 +13,8 @@
 
 #include "kis_types.h"
 
+class KisLayerStyleKnockoutBlower;
+
 
 class KisLayerStyleFilterProjectionPlane : public KisAbstractProjectionPlane
 {
@@ -42,9 +32,24 @@ public:
     QRect changeRect(const QRect &rect, KisLayer::PositionToFilthy pos) const override;
     QRect accessRect(const QRect &rect, KisLayer::PositionToFilthy pos) const override;
     QRect needRectForOriginal(const QRect &rect) const override;
+    QRect tightUserVisibleBounds() const override;
 
     KisPaintDeviceList getLodCapableDevices() const override;
 
+    /**
+     * \returns true if a call to apply() will actually paint anything. Basically,
+     * it is a cached version of isEnabled(), though the state may change after calling
+     * to recalculate().
+     */
+    bool isEmpty() const;
+
+    KisLayerStyleKnockoutBlower *knockoutBlower() const;
+
+protected:
+
+    KisLayerStyleFilter* filter() const;
+    KisPSDLayerStyleSP style() const;
+
 private:
     struct Private;
     const QScopedPointer<Private> m_d;
