@@ -1,19 +1,7 @@
 /*
- *  Copyright (c) 2015 Dmitry Kazakov <dimula73@gmail.com>
+ *  SPDX-FileCopyrightText: 2015 Dmitry Kazakov <dimula73@gmail.com>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #ifndef __KIS_LS_UTILS_H
@@ -23,13 +11,13 @@
 #include "kritaimage_export.h"
 
 #include "kis_lod_transform.h"
-
+#include <KoPattern.h>
 
 struct psd_layer_effects_context;
 class psd_layer_effects_shadow_base;
 struct psd_layer_effects_overlay_base;
 class KisLayerStyleFilterEnvironment;
-class KoPattern;
+
 class KisMultipleProjection;
 class KisCachedSelection;
 
@@ -77,7 +65,7 @@ namespace KisLsUtils
                      const QRect &applyRect,
                      KisLayerStyleFilterEnvironment *env,
                      int scale,
-                     KoPattern *pattern,
+                     KoPatternSP pattern,
                      int horizontalPhase,
                      int verticalPhase,
                      bool alignWithLayer);
@@ -85,6 +73,7 @@ namespace KisLsUtils
     void fillOverlayDevice(KisPaintDeviceSP fillDevice,
                            const QRect &applyRect,
                            const psd_layer_effects_overlay_base *config,
+                           KisResourcesInterfaceSP resourcesInterface,
                            KisLayerStyleFilterEnvironment *env);
 
     void applyFinalSelection(const QString &projectionId,
@@ -94,7 +83,7 @@ namespace KisLsUtils
                              const QRect &srcRect,
                              const QRect &dstRect,
                              const psd_layer_effects_context *context,
-                             const psd_layer_effects_shadow_base *config,
+                             const psd_layer_effects_shadow_base *config, KisResourcesInterfaceSP resourcesInterface,
                              const KisLayerStyleFilterEnvironment *env);
 
     bool checkEffectEnabled(const psd_layer_effects_shadow_base *config, KisMultipleProjection *dst);
