@@ -261,7 +261,7 @@ void KisColorizeMask::setProfile(const KoColorProfile *profile, KUndo2Command *p
     m_d->fakePaintDevice->setProfile(profile, parentCommand);
     m_d->coloringProjection->setProfile(profile, parentCommand);
 
-    for (auto stroke : m_d->keyStrokes) {
+    for (auto & stroke : m_d->keyStrokes) {
         stroke.color.setProfile(profile);
     }
 }
@@ -360,7 +360,7 @@ void KisColorizeMask::slotUpdateRegenerateFilling(bool prefilterOnly)
             const KoColor color =
                 !stroke.isTransparent ?
                 stroke.color :
-                KoColor(Qt::transparent, stroke.color.colorSpace());
+                KoColor::createTransparent(stroke.color.colorSpace());
 
             strategy->addKeyStroke(stroke.dev, color);
         }
@@ -425,6 +425,7 @@ void KisColorizeMask::slotRegenerationFinished(bool prefilterOnly)
 void KisColorizeMask::slotRegenerationCancelled()
 {
     slotRegenerationFinished(true);
+    m_d->setNeedsUpdateImpl(true, false);
 }
 
 KisBaseNode::PropertyList KisColorizeMask::sectionModelProperties() const
@@ -657,14 +658,6 @@ QRect KisColorizeMask::nonDependentExtent() const
     return extent();
 }
 
-KisImageSP KisColorizeMask::fetchImage() const
-{
-    KisLayerSP parentLayer(qobject_cast<KisLayer*>(parent().data()));
-    if (!parentLayer) return KisImageSP();
-
-    return parentLayer->image();
-}
-
 void KisColorizeMask::setImage(KisImageWSP image)
 {
     KisDefaultBoundsSP bounds(new KisDefaultBounds(image));
@@ -691,9 +684,7 @@ void KisColorizeMask::setCurrentColor(const KoColor &_color)
     QList<KeyStroke>::const_iterator it =
         std::find_if(m_d->keyStrokes.constBegin(),
                      m_d->keyStrokes.constEnd(),
-                     [color] (const KeyStroke &s) {
-                         return s.color == color;
-                     });
+                     kismpl::mem_equal_to(&KeyStroke::color, color));
 
     KisPaintDeviceSP activeDevice;
     bool newKeyStroke = false;
@@ -701,7 +692,7 @@ void KisColorizeMask::setCurrentColor(const KoColor &_color)
     if (it == m_d->keyStrokes.constEnd()) {
         activeDevice = new KisPaintDevice(KoColorSpaceRegistry::instance()->alpha8());
         activeDevice->setParentNode(this);
-        activeDevice->setDefaultBounds(KisDefaultBoundsBaseSP(new KisDefaultBounds(fetchImage())));
+        activeDevice->setDefaultBounds(KisDefaultBoundsBaseSP(new KisDefaultBounds(image())));
         newKeyStroke = true;
     } else {
         activeDevice = it->dev;
@@ -796,7 +787,7 @@ void KisColorizeMask::mergeToLayerUnthreaded(KisNodeSP layer, KUndo2Command *par
     /**
      * When merging, we use barrier jobs only for ensuring that the merge jobs
      * are not split by the update jobs. Merge jobs hold the shared lock, so
-     * forcinf them out of CPU will basically cause a deadlock. When running in
+     * forcing them out of CPU will basically cause a deadlock. When running in
      * the fake executor, the jobs cannot be split anyway, so there is no danger
      * in that.
      */
@@ -946,7 +937,7 @@ void KisColorizeMask::setKeyStrokesColors(KeyStrokeColors colors)
         newList[i].isTransparent = colors.transparentIndex == i;
     }
 
-    KisProcessingApplicator applicator(fetchImage(), KisNodeSP(this),
+    KisProcessingApplicator applicator(image(), KisNodeSP(this),
                                        KisProcessingApplicator::NONE,
                                        KisImageSignalVector(),
                                        kundo2_i18n("Change Key Stroke Color"));
@@ -965,15 +956,13 @@ void KisColorizeMask::removeKeyStroke(const KoColor &_color)
     QList<KeyStroke>::iterator it =
         std::find_if(m_d->keyStrokes.begin(),
                      m_d->keyStrokes.end(),
-                     [color] (const KeyStroke &s) {
-                         return s.color == color;
-                     });
+                     kismpl::mem_equal_to(&KeyStroke::color, color));
 
     KIS_SAFE_ASSERT_RECOVER_RETURN(it != m_d->keyStrokes.end());
 
     const int index = it - m_d->keyStrokes.begin();
 
-    KisProcessingApplicator applicator(KisImageWSP(fetchImage()), KisNodeSP(this),
+    KisProcessingApplicator applicator(image(), KisNodeSP(this),
                                        KisProcessingApplicator::NONE,
                                        KisImageSignalVector(),
                                        kundo2_i18n("Remove Key Stroke"));
@@ -1090,7 +1079,7 @@ void KisColorizeMask::testingAddKeyStroke(KisPaintDeviceSP dev, const KoColor &c
     m_d->keyStrokes << KeyStroke(dev, color, isTransparent);
 }
 
-void KisColorizeMask::testingRegenerateMask()
+void KisColorizeMask::forceRegenerateMask()
 {
     slotUpdateRegenerateFilling();
     m_d->updateIsRunning = false;
@@ -1114,9 +1103,8 @@ void KisColorizeMask::setKeyStrokesDirect(const QList<KisLazyFillTools::KeyStrok
         it->dev->setParentNode(this);
     }
 
-    KisImageSP image = fetchImage();
-    KIS_SAFE_ASSERT_RECOVER_RETURN(image);
-    setImage(image);
+    KIS_SAFE_ASSERT_RECOVER(image())
+    {};
 }
 
 qint32 KisColorizeMask::x() const
