@@ -70,8 +70,8 @@ public:
                        PositionToFilthy maskPos) const override;
 
     void setCurrentColor(const KoColor &color) override;
-    void mergeToLayer(KisNodeSP layer, KisPostExecutionUndoAdapter *undoAdapter, const KUndo2MagicString &transactionText,int timedID) override;
-    void writeMergeData(KisPainter *painter, KisPaintDeviceSP src) override;
+    void mergeToLayerThreaded(KisNodeSP layer, KUndo2Command *parentCommand, const KUndo2MagicString &transactionText, int timedID, QVector<KisRunnableStrokeJobData *> *jobs) override;
+    void writeMergeData(KisPainter *painter, KisPaintDeviceSP src, const QRect &rc) override;
     bool supportsNonIndirectPainting() const override;
 
     QRect exactBounds() const override;
@@ -110,9 +110,10 @@ public:
     bool limitToDeviceBounds() const;
 
     void testingAddKeyStroke(KisPaintDeviceSP dev, const KoColor &color, bool isTransparent = false);
-    void testingRegenerateMask();
     KisPaintDeviceSP testingFilteredSource() const;
 
+    void forceRegenerateMask();
+
     QList<KisLazyFillTools::KeyStroke> fetchKeyStrokesDirect() const;
     void setKeyStrokesDirect(const QList<KisLazyFillTools::KeyStroke> &strokes);
 
@@ -149,9 +150,10 @@ private:
     bool showKeyStrokes() const;
     void setShowKeyStrokes(bool value);
 
+    void mergeToLayerUnthreaded(KisNodeSP layer, KUndo2Command *parentCommand, const KUndo2MagicString &transactionText, int timedID);
+
 private:
     void rerenderFakePaintDevice();
-    KisImageSP fetchImage() const;
     void moveAllInternalDevices(const QPoint &diff);
 
     template <class DeviceMetricPolicy>
