@@ -1,23 +1,11 @@
 /*
- *  Copyright (c) 2007 Boudewijn Rempt <boud@valdyas.org>
+ *  SPDX-FileCopyrightText: 2007 Boudewijn Rempt <boud@valdyas.org>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "kis_filter_mask_test.h"
-#include <QTest>
+#include <simpletest.h>
 
 #include <KoColorSpaceRegistry.h>
 
@@ -31,9 +19,10 @@
 #include "kis_paint_layer.h"
 #include "kis_types.h"
 #include "kis_image.h"
+#include <KisGlobalResourcesInterface.h>
 
 
-#include "testutil.h"
+#include <testutil.h>
 
 #define IMAGE_WIDTH 1000
 #define IMAGE_HEIGHT 1000
@@ -51,12 +40,13 @@ void KisFilterMaskTest::testProjectionNotSelected()
 
     KisFilterSP f = KisFilterRegistry::instance()->value("invert");
     Q_ASSERT(f);
-    KisFilterConfigurationSP  kfc = f->defaultConfiguration();
+    KisFilterConfigurationSP  kfc = f->defaultConfiguration(KisGlobalResourcesInterface::instance());
     Q_ASSERT(kfc);
 
     KisFilterMaskSP mask = new KisFilterMask(image, "mask");
     image->addNode(mask, layer);
-    mask->setFilter(kfc);
+
+    mask->setFilter(kfc->cloneWithResourcesSnapshot());
 
     // Check basic apply(). Shouldn't do anything, since nothing is selected yet.
 
@@ -86,13 +76,13 @@ void KisFilterMaskTest::testProjectionSelected()
 
     KisFilterSP f = KisFilterRegistry::instance()->value("invert");
     Q_ASSERT(f);
-    KisFilterConfigurationSP  kfc = f->defaultConfiguration();
+    KisFilterConfigurationSP  kfc = f->defaultConfiguration(KisGlobalResourcesInterface::instance());
     Q_ASSERT(kfc);
 
     KisFilterMaskSP mask = new KisFilterMask(image, "mask");
     image->addNode(mask, layer);
 
-    mask->setFilter(kfc);
+    mask->setFilter(kfc->cloneWithResourcesSnapshot());
     mask->createNodeProgressProxy();
 
     mask->initSelection(layer);
@@ -108,4 +98,4 @@ void KisFilterMaskTest::testProjectionSelected()
 
 }
 
-QTEST_MAIN(KisFilterMaskTest)
+SIMPLE_TEST_MAIN(KisFilterMaskTest)
