@@ -26,6 +26,7 @@
 #include <kis_transform_mask.h>
 #include <kis_transparency_mask.h>
 #include <kis_selection_mask.h>
+#include <lazybrush/kis_colorize_mask.h>
 #include <kis_effect_mask.h>
 #include <kis_paint_layer.h>
 #include <kis_generator_layer.h>
@@ -62,6 +63,7 @@
 #include <QMessageBox>
 
 #include <kis_image_animation_interface.h>
+#include <kis_layer_utils.h>
 
 struct Document::Private {
     Private() {}
@@ -127,6 +129,7 @@ Node *Document::activeNode() const
 
 void Document::setActiveNode(Node* value)
 {
+    if (!value) return;
     if (!value->node()) return;
     KisMainWindow *mainWin = KisPart::instance()->currentMainwindow();
     if (!mainWin) return;
@@ -152,15 +155,19 @@ QList<Node *> Document::topLevelNodes() const
 Node *Document::nodeByName(const QString &name) const
 {
     if (!d->document) return 0;
-    KisNodeSP node = d->document->image()->rootLayer()->findChildByName(name);
+    KisNodeSP node = KisLayerUtils::findNodeByName(d->document->image()->rootLayer(),name);
+
     if (node.isNull()) return 0;
+
     return Node::createNode(d->document->image(), node);
 }
 
 Node *Document::nodeByUniqueID(const QUuid &id) const
 {
     if (!d->document) return 0;
-    KisNodeSP node = d->document->image()->rootLayer()->findChildByUniqueID(id);
+
+    KisNodeSP node = KisLayerUtils::findNodeByUuid(d->document->image()->rootLayer(), id);
+
     if (node.isNull()) return 0;
     return Node::createNode(d->document->image(), node);
 }
@@ -630,6 +637,9 @@ Node* Document::createNode(const QString &name, const QString &nodeType)
     else if (nodeType.toLower()  == "selectionmask") {
         node = new Node(image, new KisSelectionMask(image, name));
     }
+    else if (nodeType.toLower()  == "colorizemask") {
+        node = new Node(image, new KisColorizeMask(image, name));
+    }
 
     return node;
 }
@@ -643,13 +653,13 @@ GroupLayer *Document::createGroupLayer(const QString &name)
     return new GroupLayer(image, name);
 }
 
-FileLayer *Document::createFileLayer(const QString &name, const QString fileName, const QString scalingMethod)
+FileLayer *Document::createFileLayer(const QString &name, const QString fileName, const QString scalingMethod, const QString scalingFilter)
 {
     if (!d->document) return 0;
     if (!d->document->image()) return 0;
     KisImageSP image = d->document->image();
 
-    return new FileLayer(image, name, this->fileName(), fileName, scalingMethod);
+    return new FileLayer(image, name, this->fileName(), fileName, scalingMethod, scalingFilter);
 }
 
 FilterLayer *Document::createFilterLayer(const QString &name, Filter &filter, Selection &selection)
@@ -745,6 +755,15 @@ SelectionMask *Document::createSelectionMask(const QString &name)
     return new SelectionMask(image, name);
 }
 
+TransparencyMask *Document::createTransparencyMask(const QString &name)
+{
+    if (!d->document) return 0;
+    if (!d->document->image()) return 0;
+    KisImageSP image = d->document->image();
+
+    return new TransparencyMask(image, name);
+}
+
 TransformMask *Document::createTransformMask(const QString &name)
 {
     if (!d->document) return 0;
@@ -754,6 +773,15 @@ TransformMask *Document::createTransformMask(const QString &name)
     return new TransformMask(image, name);
 }
 
+ColorizeMask *Document::createColorizeMask(const QString &name)
+{
+    if (!d->document) return 0;
+    if (!d->document->image()) return 0;
+    KisImageSP image = d->document->image();
+
+    return new ColorizeMask(image, name);
+}
+
 QImage Document::projection(int x, int y, int w, int h) const
 {
     if (!d->document || !d->document->image()) return QImage();
@@ -963,7 +991,7 @@ void Document::setFullClipRangeStartTime(int startTime)
     if (!d->document) return;
     if (!d->document->image()) return;
 
-    d->document->image()->animationInterface()->setFullClipRangeStartTime(startTime);
+    d->document->image()->animationInterface()->setDocumentRangeStartFrame(startTime);
 }
 
 
@@ -972,7 +1000,7 @@ int Document::fullClipRangeStartTime()
     if (!d->document) return false;
     if (!d->document->image()) return false;
 
-    return d->document->image()->animationInterface()->fullClipRange().start();
+    return d->document->image()->animationInterface()->documentPlaybackRange().start();
 }
 
 
@@ -981,7 +1009,7 @@ void Document::setFullClipRangeEndTime(int endTime)
     if (!d->document) return;
     if (!d->document->image()) return;
 
-    d->document->image()->animationInterface()->setFullClipRangeEndTime(endTime);
+    d->document->image()->animationInterface()->setDocumentRangeEndFrame(endTime);
 }
 
 
@@ -990,7 +1018,7 @@ int Document::fullClipRangeEndTime()
     if (!d->document) return false;
     if (!d->document->image()) return false;
 
-    return d->document->image()->animationInterface()->fullClipRange().end();
+    return d->document->image()->animationInterface()->documentPlaybackRange().end();
 }
 
 int Document::animationLength()
@@ -1007,7 +1035,7 @@ void Document::setPlayBackRange(int start, int stop)
     if (!d->document->image()) return;
 
     const KisTimeSpan newTimeRange = KisTimeSpan::fromTimeWithDuration(start, (stop-start));
-    d->document->image()->animationInterface()->setPlaybackRange(newTimeRange);
+    d->document->image()->animationInterface()->setActivePlaybackRange(newTimeRange);
 }
 
 int Document::playBackStartTime()
@@ -1015,7 +1043,7 @@ int Document::playBackStartTime()
     if (!d->document) return false;
     if (!d->document->image()) return false;
 
-    return d->document->image()->animationInterface()->playbackRange().start();
+    return d->document->image()->animationInterface()->activePlaybackRange().start();
 }
 
 int Document::playBackEndTime()
@@ -1023,7 +1051,7 @@ int Document::playBackEndTime()
     if (!d->document) return false;
     if (!d->document->image()) return false;
 
-    return d->document->image()->animationInterface()->playbackRange().end();
+    return d->document->image()->animationInterface()->activePlaybackRange().end();
 }
 
 int Document::currentTime()
