@@ -1,19 +1,7 @@
 /*
- *  Copyright (c) 2017 Boudewijn Rempt <boud@valdyas.org>
+ *  SPDX-FileCopyrightText: 2017 Boudewijn Rempt <boud@valdyas.org>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU Lesser General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: LGPL-2.0-or-later
  */
 #ifndef PRESETCHOOSER_H
 #define PRESETCHOOSER_H
@@ -59,17 +47,17 @@ Q_SIGNALS:
     /**
      * Emitted whenever a user selects the given preset.
      */
-    void presetSelected(Resource *resource);
+    void presetSelected(Resource resource);
     
     /**
      * Emitted whenever a user clicks on the given preset.
      */
-    void presetClicked(Resource *resource);
+    void presetClicked(Resource resource);
 
 private Q_SLOTS:
 
-    void slotResourceSelected(KoResource *resource);
-    void slotResourceClicked(KoResource *resource);
+    void slotResourceSelected(KoResourceSP resource);
+    void slotResourceClicked(KoResourceSP resource);
 };
 
 #endif // PRESETCHOOSER_H
