@@ -23,11 +23,14 @@ class KisLocalStrokeResourcesPrivate;
 class KRITARESOURCES_EXPORT KisLocalStrokeResources : public KisResourcesInterface
 {
 public:
+    KisLocalStrokeResources();
     KisLocalStrokeResources(const QList<KoResourceSP> &localResources);
 
     void addResource(KoResourceSP resource);
     void removeResource(KoResourceSP resource);
 
+    KisLocalStrokeResources* clone() const;
+
 protected:
     ResourceSourceAdapter* createSourceImpl(const QString &type) const override;
 
