@@ -18,7 +18,7 @@
 #include <KritaVersionWrapper.h>
 
 #include <klocalizedstring.h>
-#include <kbackup.h>
+#include <KisBackup.h>
 
 #include <kis_debug.h>
 #include <KisUsageLogger.h>
@@ -66,7 +66,10 @@ bool updateSchemaVersion()
     if (f.open(QFile::ReadOnly)) {
         QString sql = f.readAll();
         QSqlQuery q;
-        q.prepare(sql);
+        if (!q.prepare(sql)) {
+            qWarning() << "Could not prepare the schema information query" << q.lastError() << q.boundValues();
+            return false;
+        }
         q.addBindValue(KisResourceCacheDb::databaseVersion);
         q.addBindValue(KritaVersionWrapper::versionString());
         q.addBindValue(QDateTime::currentDateTimeUtc().toSecsSinceEpoch());
@@ -173,7 +176,7 @@ QSqlError createDatabase(const QString &location)
                 qWarning() << "Old schema:" << schemaVersion << "New schema:" << newSchemaVersionNumber;
 
                 schemaIsOutDated = true;
-                KBackup::numberedBackupFile(location + "/" + KisResourceCacheDb::resourceCacheDbFilename);
+                KisBackup::numberedBackupFile(location + "/" + KisResourceCacheDb::resourceCacheDbFilename);
 
                 if (newSchemaVersionNumber == QVersionNumber::fromString("0.0.17")
                         && QVersionNumber::compare(oldSchemaVersionNumber, QVersionNumber::fromString("0.0.14")) > 0
@@ -1374,7 +1377,7 @@ bool KisResourceCacheDb::addTag(const QString &resourceType, const QString stora
             q.bindValue(":comment", comment);
 
             if (!q.exec()) {
-                qWarning() << "Could not execute insert tag_transaltion query" << q.lastError() << q.boundValues();
+                qWarning() << "Could not execute insert tag_translation query" << q.lastError() << q.boundValues();
             }
         }
 
@@ -1636,7 +1639,7 @@ struct ResourceVersion : public boost::less_than_comparable<ResourceVersion>
 
 };
 
-MAYBE_UNUSED
+[[maybe_unused]]
 QDebug operator<<(QDebug dbg, const ResourceVersion &ver)
 {
     dbg.nospace() << "ResourceVersion("
