@@ -70,8 +70,8 @@ KisAllResourcesModel::KisAllResourcesModel(const QString &resourceType, QObject
                                        "AND    resources.storage_id = storages.id\n"
                                        "AND    resource_types.name = :resource_type\n"
                                        "GROUP BY resources.name\n"
-                                       ", resources.filename\n"
-                                       ", resources.md5sum\n"
+                                       ",        resources.filename\n"
+                                       ",        resources.md5sum\n"
                                        "ORDER BY resources.id");
     if (!r) {
         qWarning() << "Could not prepare KisAllResourcesModel query" << d->resourcesQuery.lastError();
@@ -115,12 +115,11 @@ QVariant KisAllResourcesModel::data(const QModelIndex &index, int role) const
 
 QVariant KisAllResourcesModel::headerData(int section, Qt::Orientation orientation, int role) const
 {
-    QVariant v = QVariant();
     if (role != Qt::DisplayRole) {
-        return v;
+        return {};
     }
     if (orientation == Qt::Horizontal) {
-        switch(section) {
+        switch (section) {
         case Id:
             return i18n("Id");
         case StorageId:
@@ -145,11 +144,19 @@ QVariant KisAllResourcesModel::headerData(int section, Qt::Orientation orientati
             return i18n("Storage Active");
         case MD5:
             return i18n("md5sum");
+        case Tags:
+            return i18n("Tags");
+        case LargeThumbnail:
+            return i18n("Large Thumbnail");
+        case Dirty:
+            return i18n("Dirty");
+        case MetaData:
+            return i18n("Metadata");
         default:
             return QString::number(section);
         }
     }
-    return v;
+    return {};
 }
 
 bool KisAllResourcesModel::setData(const QModelIndex &index, const QVariant &value, int role)
@@ -600,13 +607,21 @@ int KisAllResourcesModel::rowCount(const QModelIndex &parent) const
          */
 
         QSqlQuery q;
-        q.prepare("SELECT COUNT(DISTINCT resources.name || resources.filename || resources.md5sum)\n"
+        bool r = q.prepare("SELECT COUNT(DISTINCT resources.name || resources.filename || resources.md5sum)\n"
                   "FROM   resources\n"
                   ",      resource_types\n"
                   "WHERE  resources.resource_type_id = resource_types.id\n"
                   "AND    resource_types.name = :resource_type\n");
+        if (!r) {
+            qWarning() << "Could not prepare all resources rowcount query" << q.lastError();
+            return 0;
+        }
         q.bindValue(":resource_type", d->resourceType);
-        q.exec();
+        r = q.exec();
+        if (!r) {
+            qWarning() << "Could not execute all resources rowcount query" << q.lastError() << q.boundValues();
+            return 0;
+        }
         q.first();
 
         const_cast<KisAllResourcesModel*>(this)->d->cachedRowCount = q.value(0).toInt();
