@@ -75,16 +75,21 @@ int KisAllTagsModel::rowCount(const QModelIndex &parent) const
 
     if (d->cachedRowCount < 0) {
         QSqlQuery q;
-        q.prepare("SELECT count(*)\n"
+        const bool r = q.prepare("SELECT count(*)\n"
                   "FROM   tags\n"
                   ",      resource_types\n"
                   "LEFT JOIN tag_translations ON tag_translations.tag_id = tags.id AND tag_translations.language = :language\n"
                   "WHERE  tags.resource_type_id = resource_types.id\n"
                   "AND    resource_types.name = :resource_type\n");
+        if (!r) {
+            qWarning() << "Could not execute tags rowcount query" << q.lastError();
+            return 0;
+        }
         q.bindValue(":resource_type", d->resourceType);
         q.bindValue(":language", KisTag::currentLocale());
         if (!q.exec()) {
             qWarning() << "Could not execute tags rowcount query" << q.lastError();
+            return 0;
         }
         q.first();
 
@@ -126,7 +131,6 @@ QVariant KisAllTagsModel::data(const QModelIndex &index, int role) const
             }
             case Qt::UserRole + ResourceType:
                 return d->resourceType;
-            case Qt::CheckStateRole:
             case Qt::UserRole + Active:
                 return true;
             case Qt::UserRole + KisTagRole:
@@ -154,7 +158,6 @@ QVariant KisAllTagsModel::data(const QModelIndex &index, int role) const
             }
             case Qt::UserRole + ResourceType:
                 return d->resourceType;
-            case Qt::CheckStateRole:
             case Qt::UserRole + Active:
                 return true;
             case Qt::UserRole + KisTagRole:
@@ -198,7 +201,6 @@ QVariant KisAllTagsModel::data(const QModelIndex &index, int role) const
                 return d->query.value("url");
             case Qt::UserRole + ResourceType:
                 return d->query.value("resource_type");
-            case Qt::CheckStateRole:
             case Qt::UserRole + Active:
                 return d->query.value("active");
             case Qt::UserRole + KisTagRole:
@@ -221,7 +223,7 @@ bool KisAllTagsModel::setData(const QModelIndex &index, const QVariant &value, i
     int id = data(index, Qt::UserRole + Id).toInt();
 
     if (index.isValid() &&
-        (role == Qt::CheckStateRole || role == Active)) {
+        (role == Qt::CheckStateRole || role == Qt::UserRole + Active)) {
 
         QSqlQuery q;
         if (!q.prepare("UPDATE tags\n"
@@ -344,7 +346,7 @@ KisTagSP KisAllTagsModel::addTag(const QString& tagName, const bool allowOverwri
 }
 
 
-bool KisAllTagsModel::addTag(const KisTagSP tag, const bool allowOverwrite, QVector<KoResourceSP> taggedResouces)
+bool KisAllTagsModel::addTag(const KisTagSP tag, const bool allowOverwrite, QVector<KoResourceSP> taggedResources)
 {
     if (!tag) return false;
     if (!tag->valid()) return false;
@@ -376,9 +378,9 @@ bool KisAllTagsModel::addTag(const KisTagSP tag, const bool allowOverwrite, QVec
     tag->setValid(true);
     tag->setActive(data(indexForTag(tag), Qt::UserRole + KisAllTagsModel::Active).toInt());
 
-    if (!taggedResouces.isEmpty()) {
+    if (!taggedResources.isEmpty()) {
         QVector<int> resourceIds;
-        Q_FOREACH(const KoResourceSP resource, taggedResouces) {
+        Q_FOREACH(const KoResourceSP resource, taggedResources) {
 
             if (!resource) continue;
             if (!resource->valid()) continue;
@@ -603,11 +605,11 @@ KisTagSP KisTagModel::tagForUrl(const QString& url) const
 }
 
 
-bool KisTagModel::addTag(const KisTagSP tag, const bool allowOverwrite, QVector<KoResourceSP> taggedResouces)
+bool KisTagModel::addTag(const KisTagSP tag, const bool allowOverwrite, QVector<KoResourceSP> taggedResources)
 {
     KisAbstractTagModel *source = dynamic_cast<KisAbstractTagModel*>(sourceModel());
     if (source) {
-        return source->addTag(tag, allowOverwrite, taggedResouces) ;
+        return source->addTag(tag, allowOverwrite, taggedResources) ;
     }
     return false;
 }
@@ -677,17 +679,22 @@ bool KisTagModel::filterAcceptsRow(int source_row, const QModelIndex &source_par
     {
         if (tagId > 0) {
             QSqlQuery q;
-            q.prepare("SELECT count(*)\n"
+            const bool r = q.prepare("SELECT count(*)\n"
                       "FROM   tags_storages\n"
                       ",      storages\n"
                       "WHERE  tags_storages.tag_id = :tag_id\n"
                       "AND    tags_storages.storage_id = storages.id\n"
                       "AND    storages.active = 1\n");
+            if (!r) {
+                qWarning() << "Could not execute tags in storages query" << q.lastError();
+                return true;
+            }
 
             q.bindValue(":tag_id", tagId);
 
             if (!q.exec()) {
                 qWarning() << "Could not execute tags in storages query" << q.lastError() << q.boundValues();
+                return true;
             }
             else {
                 q.first();
