@@ -1,5 +1,6 @@
 /*
  * SPDX-FileCopyrightText: 2020 Boudewijn Rempt <boud@valdyas.org>
+ * SPDX-FileCopyrightText: 2023 L. E. Segovia <amy@amyspark.me>
  *
  * SPDX-License-Identifier: LGPL-2.0-or-later
  */
@@ -56,7 +57,7 @@ int KisAllTagResourceModel::rowCount(const QModelIndex &parent) const
 
     if (d->cachedRowCount < 0) {
         QSqlQuery q;
-        q.prepare("SELECT COUNT(DISTINCT resource_tags.tag_id || resources.name || resources.filename || resources.md5sum)\n"
+        const bool r = q.prepare("SELECT COUNT(DISTINCT resource_tags.tag_id || resources.name || resources.filename || resources.md5sum)\n"
                   "FROM   resource_tags\n"
                   ",      resources\n"
                   ",      resource_types\n"
@@ -65,10 +66,16 @@ int KisAllTagResourceModel::rowCount(const QModelIndex &parent) const
                   "AND    resource_types.name = :resource_type\n"
                   "AND    resource_tags.active = 1\n");
 
+        if (!r) {
+            qWarning() << "Could not execute resource/tags rowcount query" << q.lastError();
+            return 0;
+        }
+
         q.bindValue(":resource_type", d->resourceType);
 
         if (!q.exec()) {
             qWarning() << "Could not execute resource/tags rowcount query" << q.lastError();
+            return 0;
         }
 
         q.first();
@@ -99,7 +106,11 @@ QVariant KisAllTagResourceModel::data(const QModelIndex &index, int role) const
     if (!pos) {return v;}
 
     if (role < Qt::UserRole + TagId && index.column() < TagId) {
-        v = KisResourceQueryMapper::variantFromResourceQuery(d->query, index.column(), role, true);
+        return KisResourceQueryMapper::variantFromResourceQuery(d->query, index.column(), role, true);
+    }
+
+    if (role == Qt::FontRole) {
+        return QFont();
     }
 
     if (index.column() >= TagId) {
@@ -375,7 +386,7 @@ bool KisAllTagResourceModel::untagResources(const KisTagSP tag, const QVector<in
 
     QSqlQuery allIndices;
     if (!allIndices.prepare(createQuery(true, true))) {
-        qWarning() << "Coult not prepare untagResource-allIndices query " << allIndices.lastError();
+        qWarning() << "Could not prepare untagResource-allIndices query " << allIndices.lastError();
     }
 
     allIndices.bindValue(":resource_type", d->resourceType);
@@ -597,8 +608,81 @@ QString KisAllTagResourceModel::createQuery(bool onlyActive, bool returnADbIndex
                             "                            AND   tag_translations.language   = :language\n");
 
     return query;
+}
 
+// Keep in sync with KisAllResourcesModel::headerData for sections 1-16
+// then KisAllTagResourceModel::data for 16-25
+QVariant KisAllTagResourceModel::headerData(int section, Qt::Orientation orientation, int role) const
+{
+    if (orientation != Qt::Horizontal) {
+        return {};
+    }
+    if (role != Qt::DisplayRole) {
+        return {};
+    }
 
+    switch (section) {
+    case KisAbstractResourceModel::Id:
+        return i18n("Id");
+    case KisAbstractResourceModel::StorageId:
+        return i18n("Storage ID");
+    case KisAbstractResourceModel::Name:
+        return i18n("Name");
+    case KisAbstractResourceModel::Filename:
+        return i18n("File Name");
+    case KisAbstractResourceModel::Tooltip:
+        return i18n("Tooltip");
+    case KisAbstractResourceModel::Thumbnail:
+        return i18n("Image");
+    case KisAbstractResourceModel::Status:
+        return i18n("Status");
+    case KisAbstractResourceModel::Location:
+        return i18n("Location");
+    case KisAbstractResourceModel::ResourceType:
+        return i18n("Resource Type");
+    case KisAbstractResourceModel::ResourceActive:
+        return i18n("Active");
+    case KisAbstractResourceModel::StorageActive:
+        return i18n("Storage Active");
+    case KisAbstractResourceModel::MD5:
+        return i18n("md5sum");
+    case KisAbstractResourceModel::Tags:
+        return i18n("Tags");
+    case KisAbstractResourceModel::LargeThumbnail:
+        return i18n("Large Thumbnail");
+    case KisAbstractResourceModel::Dirty:
+        return i18n("Dirty");
+    case KisAbstractResourceModel::MetaData:
+        return i18n("Metadata");
+    default:
+        role = Qt::UserRole + section;
+        break;
+    }
+
+    switch (role) {
+    case Qt::UserRole + KisAllTagResourceModel::TagId:
+        return i18n("Tag ID");
+    case Qt::UserRole + KisAllTagResourceModel::ResourceId:
+        return i18n("Resource ID");
+    case Qt::UserRole + KisAllTagResourceModel::Tag:
+        return i18n("Tag Url");
+    case Qt::UserRole + KisAllTagResourceModel::Resource:
+        return i18n("Resource");
+    case Qt::UserRole + ResourceActive:
+        return i18n("Resource Active");
+    case Qt::UserRole + TagActive:
+        return i18n("Tag Active");
+    case Qt::UserRole + ResourceStorageActive:
+        return i18n("Storage Active");
+    case Qt::UserRole + ResourceName:
+        return i18n("Resource Name");
+    case Qt::UserRole + TagName:
+        return i18n("Tag File Name");
+    default:
+        break;
+    }
+
+    return {};
 }
 
 bool KisAllTagResourceModel::resetQuery()
@@ -892,3 +976,8 @@ bool KisTagResourceModel::setResourceMetaData(KoResourceSP resource, QMap<QStrin
     KisResourceModel resourceModel(d->resourceType);
     return resourceModel.setResourceMetaData(resource, metadata);
 }
+
+QVariant KisTagResourceModel::headerData(int section, Qt::Orientation orientation, int role) const
+{
+    return d->sourceModel->headerData(section, orientation, role);
+}
