@@ -38,7 +38,18 @@ static QStringList cleanup(const QStringList &pathList)
     QStringList cleanedPathList;
 
     bool getRidOfAppDataLocation = KoResourcePaths::getAppDataLocation() != QStandardPaths::writableLocation(QStandardPaths::AppDataLocation);
-    const QString writableLocation = QStandardPaths::writableLocation(QStandardPaths::AppDataLocation);
+    const QString writableLocation = []() {
+        QString location = QStandardPaths::writableLocation(QStandardPaths::AppDataLocation);
+        // we have to ensure that the location has a trailing separator, because otherwise when we'll do startsWith
+        // check it will skip paths that start have the same path but different directory name. E.g:
+        // ~/.local/share/krita -> AppDataLocation
+        // ~/.local/share/krita3 -> custom location, but this will be skipped in getRidOfAppDataLocation.
+        if (location.back() == '/') {
+            return location;
+        } else {
+            return QString(location + "/");
+        }
+    }();
 
      Q_FOREACH(const QString &path, pathList) {
         QString cleanPath = cleanup(path);
@@ -131,7 +142,8 @@ QString getInstallationPrefix() {
 #elif defined(Q_OS_ANDROID)
     // qApp->applicationDirPath() isn't writable and android system won't allow
     // any files other than libraries
-    return QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/";
+    // NOTE the subscript [1]. It points to the internal location.
+    return QStandardPaths::standardLocations(QStandardPaths::AppDataLocation)[1] + "/";
 #else
     return qApp->applicationDirPath() + "/../";
 #endif
@@ -213,8 +225,22 @@ QString KoResourcePaths::getAppDataLocation()
     KConfigGroup cfg(KSharedConfig::openConfig(), "");
     path = cfg.readEntry(KisResourceLocator::resourceLocationKey, QStandardPaths::writableLocation(QStandardPaths::AppDataLocation));
 
+    QFileInfo fi(path);
+
+    // Check whether an existing location is writable
+    if (fi.exists() && !fi.isWritable()) {
+        path = QStandardPaths::writableLocation(QStandardPaths::AppDataLocation);
+    }
+    else if (!fi.exists()) {
+        // Check whether a non-existing location can be created
+        if (!QDir().mkpath(path)) {
+            path = QStandardPaths::writableLocation(QStandardPaths::AppDataLocation);
+        }
+        QDir().rmpath(path);
+    }
     return path;
 
+
 }
 
 void KoResourcePaths::getAllUserResourceFoldersLocationsForWindowsStore(QString &standardLocation, QString &privateLocation)
@@ -685,9 +711,15 @@ QString KoResourcePaths::locateLocalInternal(const QString &type, const QString
 
 QStringList KoResourcePaths::findExtraResourceDirs() const
 {
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
+    QStringList extraResourceDirs =
+        QString::fromUtf8(qgetenv("EXTRA_RESOURCE_DIRS"))
+            .split(';', Qt::SkipEmptyParts);
+#else
     QStringList extraResourceDirs =
         QString::fromUtf8(qgetenv("EXTRA_RESOURCE_DIRS"))
             .split(';', QString::SkipEmptyParts);
+#endif
 
     const KConfigGroup cfg(KSharedConfig::openConfig(), "");
     const QString customPath =
