@@ -18,6 +18,7 @@
 #include <QAbstractProxyModel>
 #include <QLabel>
 #include <QScrollArea>
+#include <QScrollBar>
 #include <QImage>
 #include <QPixmap>
 #include <QPainter>
@@ -28,6 +29,8 @@
 #include <QComboBox>
 #include <QStandardPaths>
 
+#include "ksharedconfig.h"
+#include "kconfiggroup.h"
 #include <klocalizedstring.h>
 
 #include <KoIcon.h>
@@ -48,9 +51,7 @@
 #include "KisResourceTaggingManager.h"
 #include <KisResourceUserOperations.h>
 
-
 #include "KisStorageChooserWidget.h"
-#include "kis_assert.h"
 
 
 class Q_DECL_HIDDEN KisResourceItemChooser::Private
@@ -62,21 +63,27 @@ public:
 
     QString resourceType;
 
+    // Top/Bottom bar
     KisTagFilterResourceProxyModel *tagFilterProxyModel {0};
-
     KisResourceTaggingManager *tagManager {0};
-    KisResourceItemListView *view {0};
-    QButtonGroup *buttonGroup {0};
     KisPopupButton *viewModeButton {0};
     KisStorageChooserWidget *storagePopupButton {0};
 
+    // Resources view
+    KisResourceItemListView *view {0};
+    QSplitter *resourcesSplitter {0};
+
     QScrollArea *previewScroller {0};
     QLabel *previewLabel {0};
-    QSplitter *splitter {0};
-    QGridLayout *buttonLayout {0};
 
+    // Import and Export buttons
+    QFrame* importExportBtns {0};
     QToolButton *importButton {0};
     QToolButton *deleteButton {0};
+    QButtonGroup *buttonGroup {0};  // does not seem to do much, maybe replace with std::array
+
+    // Custom Buttons
+    QList<QAbstractButton*> customButtons;
 
     bool usePreview {false};
     bool tiledPreview {false};
@@ -84,19 +91,51 @@ public:
     bool synced {false};
     bool updatesBlocked {false};
 
-    QModelIndex savedResourceWhileReset; // Indexes on the proxyModel, not the source resource model
-
-    QList<QAbstractButton*> customButtons;
-
     KoResourceSP currentResource;
+    Layout layout = Layout::NotSet;
+    ListViewMode requestedViewMode = ListViewMode::IconGrid;
+    bool isResponsive = false;
+    bool showViewModeBtn = true;
+    bool showStoragePopupBtn = true;
+
+    // Horizontal Layout Widgets
+    QSplitter* horzSplitter {0};
+    QFrame* left {0};
+    QFrame* right {0};
+
+    QFrame* right2Rows {0};
+    QHBoxLayout* top {0};
+    QHBoxLayout* bot {0};
+
+    QToolButton* scroll_left {0};
+    QToolButton* scroll_right {0};
+
+    bool restoreSplitterState(Layout layout);
 };
 
 KisResourceItemChooser::KisResourceItemChooser(const QString &resourceType, bool usePreview, QWidget *parent)
     : QWidget(parent)
     , d(new Private(resourceType))
 {
-    d->splitter = new QSplitter(this);
+    // Tag manager
+    d->tagFilterProxyModel = new KisTagFilterResourceProxyModel(resourceType, this);
+    d->tagFilterProxyModel->sort(Qt::DisplayRole);
+    d->tagManager = new KisResourceTaggingManager(resourceType, d->tagFilterProxyModel, this);
 
+    // Viewmode button
+    d->viewModeButton = new KisPopupButton(this);
+    d->viewModeButton->setToolTip(i18n("Display settings"));
+    d->viewModeButton->setVisible(false);
+    d->viewModeButton->setArrowVisible(false);
+    d->viewModeButton->setAutoRaise(true);
+
+    // Storage button
+    d->storagePopupButton = new KisStorageChooserWidget(resourceType, this);
+    d->storagePopupButton->setToolTip(i18n("Storage Resources"));
+    d->storagePopupButton->setAutoRaise(true);
+    d->storagePopupButton->setArrowVisible(false);
+
+    // Resource List View
     d->view = new KisResourceItemListView(this);
     d->view->setObjectName("ResourceItemview");
     d->view->setStrictSelectionMode(true);
@@ -107,7 +146,8 @@ KisResourceItemChooser::KisResourceItemChooser(const QString &resourceType, bool
     }
     else if (d->resourceType == ResourceType::PaintOpPresets) {
         d->view->setFixedToolTipThumbnailSize(QSize(128, 128));
-    } else if (d->resourceType == ResourceType::Patterns || d->resourceType == ResourceType::Palettes) {
+    }
+    else if (d->resourceType == ResourceType::Patterns || d->resourceType == ResourceType::Palettes) {
         d->view->setToolTipShouldRenderCheckers(false);
         d->view->setFixedToolTipThumbnailSize(QSize(256, 256));
     }
@@ -115,10 +155,7 @@ KisResourceItemChooser::KisResourceItemChooser(const QString &resourceType, bool
     d->view->setItemDelegate(new KisResourceItemDelegate(this));
     d->view->setSelectionMode(QAbstractItemView::SingleSelection);
     d->view->viewport()->installEventFilter(this);
-
-    d->tagFilterProxyModel = new KisTagFilterResourceProxyModel(resourceType, this);
     d->view->setModel(d->tagFilterProxyModel);
-    d->tagFilterProxyModel->sort(Qt::DisplayRole);
 
     connect(d->tagFilterProxyModel, SIGNAL(afterFilterChanged()), this, SLOT(afterFilterChanged()));
 
@@ -127,8 +164,10 @@ KisResourceItemChooser::KisResourceItemChooser(const QString &resourceType, bool
     connect(d->view, SIGNAL(contextMenuRequested(QPoint)), this, SLOT(contextMenuRequested(QPoint)));
     connect(d->view, SIGNAL(sigSizeChanged()), this, SLOT(updateView()));
 
-    d->splitter->addWidget(d->view);
-    d->splitter->setStretchFactor(0, 2);
+    // Splitter with resource views and preview scroller
+    d->resourcesSplitter = new QSplitter(this);
+    d->resourcesSplitter->addWidget(d->view);
+    d->resourcesSplitter->setStretchFactor(0, 1);
 
     d->usePreview = usePreview;
     if (d->usePreview) {
@@ -139,10 +178,10 @@ KisResourceItemChooser::KisResourceItemChooser(const QString &resourceType, bool
         d->previewScroller->setAlignment(Qt::AlignCenter);
         d->previewLabel = new QLabel(this);
         d->previewScroller->setWidget(d->previewLabel);
-        d->splitter->addWidget(d->previewScroller);
+        d->resourcesSplitter->addWidget(d->previewScroller);
 
-        if (d->splitter->count() == 2) {
-            d->splitter->setSizes(QList<int>() << 280 << 160);
+        if (d->resourcesSplitter->count() == 2) {
+            d->resourcesSplitter->setSizes(QList<int>() << 280 << 160);
         }
 
         QScroller* scroller = KisKineticScroller::createPreconfiguredScroller(d->previewScroller);
@@ -151,61 +190,108 @@ KisResourceItemChooser::KisResourceItemChooser(const QString &resourceType, bool
         }
     }
 
-    d->splitter->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
-    connect(d->splitter, SIGNAL(splitterMoved(int,int)), SIGNAL(splitterMoved()));
-
-    d->buttonGroup = new QButtonGroup(this);
-    d->buttonGroup->setExclusive(false);
-
-    QGridLayout *layout = new QGridLayout(this);
-
-    d->buttonLayout = new QGridLayout();
-
+    // Import and Export buttons
     d->importButton = new QToolButton(this);
     d->importButton->setToolTip(i18nc("@info:tooltip", "Import resource"));
     d->importButton->setAutoRaise(true);
     d->importButton->setEnabled(true);
-    d->buttonGroup->addButton(d->importButton, Button_Import);
-    d->buttonLayout->addWidget(d->importButton, 0, 0);
 
     d->deleteButton = new QToolButton(this);
     d->deleteButton->setToolTip(i18nc("@info:tooltip", "Delete resource"));
     d->deleteButton->setEnabled(false);
     d->deleteButton->setAutoRaise(true);
-    d->buttonGroup->addButton(d->deleteButton, Button_Remove);
-    d->buttonLayout->addWidget(d->deleteButton, 0, 1);
 
+    d->buttonGroup = new QButtonGroup(this);
+    d->buttonGroup->setExclusive(false);
+    d->buttonGroup->addButton(d->importButton, Button_Import);
+    d->buttonGroup->addButton(d->deleteButton, Button_Remove);
     connect(d->buttonGroup, SIGNAL(buttonClicked(int)), this, SLOT(slotButtonClicked(int)));
 
-    d->buttonLayout->setColumnStretch(0, 1);
-    d->buttonLayout->setColumnStretch(1, 1);
-    d->buttonLayout->setColumnStretch(2, 2);
-    d->buttonLayout->setSpacing(0);
-    d->buttonLayout->setMargin(0);
+    d->importExportBtns = new QFrame(this);  
+    QHBoxLayout* importExportLayout = new QHBoxLayout(d->importExportBtns);
+    importExportLayout->setAlignment(Qt::AlignmentFlag::AlignLeft);
+    importExportLayout->setMargin(0);
+    importExportLayout->addWidget(d->importButton);
+    importExportLayout->addWidget(d->deleteButton);
 
-    d->viewModeButton = new KisPopupButton(this);
-    d->viewModeButton->setVisible(false);
-    d->viewModeButton->setArrowVisible(false);
-    d->tagManager = new KisResourceTaggingManager(resourceType, d->tagFilterProxyModel, this);
+    auto removePadding = [](QLayout* layout) {
+        layout->setMargin(0);
+        layout->setSpacing(0);
+    };
 
-    d->storagePopupButton = new KisStorageChooserWidget(resourceType, this);
-    d->storagePopupButton->setToolTip(i18n("Storage Resources"));
-    d->storagePopupButton->setAutoRaise(true);
-    d->storagePopupButton->setArrowVisible(false);
+    // Layout
+    QGridLayout* thisLayout = new QGridLayout(this);
+    thisLayout->setObjectName("ResourceChooser this");
+    removePadding(thisLayout);
 
-    layout->addWidget(d->tagManager->tagChooserWidget(), 0, 0);
-    layout->addWidget(d->viewModeButton, 0, 1);
-    layout->addWidget(d->storagePopupButton, 0, 2);
-    layout->addWidget(d->splitter, 1, 0, 1, 3);
-    layout->addWidget(d->tagManager->tagFilterWidget(), 2, 0, 1, 3);
-    layout->addLayout(d->buttonLayout, 3, 0, 1, 3);
-    layout->setMargin(0);
-    layout->setSpacing(0);
+    // Horizontal Layout
+    {
+        // Splitter
+        d->horzSplitter = new QSplitter(this);
+        d->horzSplitter->setOrientation(Qt::Orientation::Horizontal);
+        connect(d->horzSplitter, SIGNAL(splitterMoved(int, int)), this, SLOT(slotSaveSplitterState()));
 
+        // Horizontal 1 row
+        d->left = new QFrame(this);
+        QSizePolicy newPolicy = d->left->sizePolicy();
+        newPolicy.setHorizontalStretch(2);
+        d->left->setSizePolicy(newPolicy);
+        QHBoxLayout* leftLayout = new QHBoxLayout(d->left);
+        leftLayout->setObjectName("ResourceChooser left");
+
+        d->right = new QFrame(this);
+        newPolicy = d->right->sizePolicy();
+        newPolicy.setHorizontalStretch(0);
+        d->right->setSizePolicy(newPolicy);
+        QHBoxLayout* rightLayout = new QHBoxLayout(d->right);
+        rightLayout->setObjectName("ResourceChooser right");
+
+        // Horizontal 2 rows
+        d->right2Rows = new QFrame(this);
+        newPolicy = d->right2Rows->sizePolicy();
+        newPolicy.setHorizontalStretch(0);
+        d->right2Rows->setSizePolicy(newPolicy);
+        QVBoxLayout* rightLayout2 = new QVBoxLayout(d->right2Rows);
+        rightLayout2->setObjectName("ResourceChooser right 2 rows");
+
+        d->top = new QHBoxLayout();
+        d->bot = new QHBoxLayout();
+        rightLayout2->addLayout(d->top);
+        rightLayout2->addLayout(d->bot);
+
+        // Common style
+        removePadding(leftLayout);
+        removePadding(rightLayout);
+        removePadding(rightLayout2);
+        removePadding(d->top);
+        removePadding(d->bot);
+
+        // Scroll buttons
+        d->scroll_left = new QToolButton(this);
+        d->scroll_left->setIcon(KisIconUtils::loadIcon("draw-arrow-back"));
+        connect(d->scroll_left, &QToolButton::clicked, this, &KisResourceItemChooser::scrollBackwards);
+        d->scroll_right = new QToolButton(this);
+        d->scroll_right->setIcon(KisIconUtils::loadIcon("draw-arrow-forward"));
+        connect(d->scroll_right, &QToolButton::clicked, this, &KisResourceItemChooser::scrollForwards);
+
+        // Hide
+        d->scroll_left->hide();
+        d->scroll_right->hide();
+        d->left->hide();
+        d->right->hide();
+        d->right2Rows->hide();
+        d->horzSplitter->hide();
+    }
+
+    // Other
     updateView();
 
+    // Default Configuration
     updateButtonState();
+    showViewModeBtn(false);
     showTaggingBar(false);
+    showImportExportBtns(true);
+    applyVerticalLayout();
 }
 
 KisResourceItemChooser::~KisResourceItemChooser()
@@ -214,6 +300,31 @@ KisResourceItemChooser::~KisResourceItemChooser()
     delete d;
 }
 
+void KisResourceItemChooser::setResponsiveness(bool isResponsive)
+{
+    if (isResponsive) {
+
+        if (d->isResponsive == false) {
+            d->isResponsive = true;
+            changeLayoutBasedOnSize();
+        }
+        else {
+            d->isResponsive = true;
+        }
+    }
+    else {
+        d->isResponsive = false;
+    }
+}
+
+void KisResourceItemChooser::setListViewMode(ListViewMode newViewMode)
+{
+    d->requestedViewMode = newViewMode;
+    if (d->layout == Layout::NotSet || d->layout == Layout::Vertical) {
+        d->view->setListViewMode(newViewMode);
+    }
+}
+
 KisTagFilterResourceProxyModel *KisResourceItemChooser::tagFilterModel() const
 {
     return d->tagFilterProxyModel;
@@ -263,34 +374,22 @@ void KisResourceItemChooser::slotButtonClicked(int button)
     updateButtonState();
 }
 
-void KisResourceItemChooser::showButtons(bool show)
+void KisResourceItemChooser::showImportExportBtns(bool show)
 {
-    foreach (QAbstractButton * button, d->buttonGroup->buttons()) {
-        show ? button->show() : button->hide();
+    // assert(show == false);
+    if (show) {
+        d->importExportBtns->show();
     }
-
-    Q_FOREACH (QAbstractButton *button, d->customButtons) {
-        show ? button->show() : button->hide();
+    else {
+        d->importExportBtns->hide();
     }
 }
 
-void KisResourceItemChooser::addCustomButton(QAbstractButton *button, int cell)
-{
-    d->buttonLayout->addWidget(button, 0, cell);
-    d->buttonLayout->setColumnStretch(2, 1);
-    d->buttonLayout->setColumnStretch(3, 1);
-}
-
 void KisResourceItemChooser::showTaggingBar(bool show)
 {
     d->tagManager->showTaggingBar(show);
 }
 
-int KisResourceItemChooser::rowCount() const
-{
-    return d->view->model()->rowCount();
-}
-
 void KisResourceItemChooser::setRowHeight(int rowHeight)
 {
     d->view->setItemSize(QSize(d->view->gridSize().width(), rowHeight));
@@ -333,9 +432,36 @@ void KisResourceItemChooser::setCurrentResource(KoResourceSP resource)
     updatePreview(index);
 }
 
+void KisResourceItemChooser::setCurrentResource(QString resourceName)
+{
+    // don't update if the change came from the same chooser
+    if (d->updatesBlocked) {
+        return;
+    }
+
+    for (int row = 0; row < d->tagFilterProxyModel->rowCount(); row++) {
+        for (int col = 0; col < d->tagFilterProxyModel->columnCount(); col++) {
+            QModelIndex index = d->tagFilterProxyModel->index(row, col);
+            KoResourceSP resource = d->tagFilterProxyModel->resourceForIndex(index);
+
+            if (resource->name() == resourceName) {
+                d->view->setCurrentIndex(index);
+
+                // The resource may currently be filtered out, but we want to be able
+                // to select it if the filter changes and includes the resource.
+                // Otherwise, activate() already took care of setting the current resource.
+                if (!index.isValid()) {
+                    d->currentResource = resource;
+                }
+                updatePreview(index);
+            }
+        }
+    }
+}
+
 void KisResourceItemChooser::setPreviewOrientation(Qt::Orientation orientation)
 {
-    d->splitter->setOrientation(orientation);
+    d->resourcesSplitter->setOrientation(orientation);
 }
 
 void KisResourceItemChooser::setPreviewTiled(bool tiled)
@@ -360,6 +486,18 @@ void KisResourceItemChooser::setCurrentItem(int row)
     }
 }
 
+void KisResourceItemChooser::scrollBackwards()
+{
+    QScrollBar* bar = d->view->horizontalScrollBar();
+    bar->setValue(bar->value() - d->view->gridSize().width());
+}
+
+void KisResourceItemChooser::scrollForwards()
+{
+    QScrollBar* bar = d->view->horizontalScrollBar();
+    bar->setValue(bar->value() + d->view->gridSize().width());
+}
+
 void KisResourceItemChooser::activate(const QModelIndex &index)
 {
     if (!index.isValid())
@@ -421,7 +559,7 @@ void KisResourceItemChooser::updatePreview(const QModelIndex &idx)
         image.format() != QImage::Format_ARGB32 &&
         image.format() != QImage::Format_ARGB32_Premultiplied) {
 
-        image = image.convertToFormat(QImage::Format_ARGB32_Premultiplied);
+        image.convertTo(QImage::Format_ARGB32_Premultiplied);
     }
 
     if (d->tiledPreview) {
@@ -477,14 +615,16 @@ void KisResourceItemChooser::contextMenuRequested(const QPoint &pos)
     d->tagManager->contextMenuRequested(currentResource(), pos);
 }
 
-void KisResourceItemChooser::setStoragePopupButtonVisible(bool visible)
+void KisResourceItemChooser::showStorageBtn(bool visible)
 {
     d->storagePopupButton->setVisible(visible);
+    d->showStoragePopupBtn = visible;
 }
 
-void KisResourceItemChooser::setViewModeButtonVisible(bool visible)
+void KisResourceItemChooser::showViewModeBtn(bool visible)
 {
     d->viewModeButton->setVisible(visible);
+    d->showViewModeBtn = visible;
 }
 
 KisPopupButton *KisResourceItemChooser::viewModeButton() const
@@ -547,6 +687,7 @@ bool KisResourceItemChooser::eventFilter(QObject *object, QEvent *event)
 void KisResourceItemChooser::resizeEvent(QResizeEvent *event)
 {
     QWidget::resizeEvent(event);
+    changeLayoutBasedOnSize();
     updateView();
 }
 
@@ -556,13 +697,200 @@ void KisResourceItemChooser::showEvent(QShowEvent *event)
     updateView();
 }
 
-void KisResourceItemChooser::updateView()
+void KisResourceItemChooser::hideEverything()
 {
-    if (d->synced) {
-        KisResourceItemChooserSync *chooserSync = KisResourceItemChooserSync::instance();
-        baseLengthChanged(chooserSync->baseLength());
+    d->horzSplitter->hide();
+    d->left->hide();
+    d->right->hide();
+    d->right2Rows->hide();
+    d->scroll_left->hide();
+    d->scroll_right->hide();
+
+    d->viewModeButton->hide();
+}
+
+void KisResourceItemChooser::applyVerticalLayout()
+{
+    if (d->layout == Layout::Vertical) {
+        return;
     }
 
+    hideEverything();
+
+    d->view->setListViewMode(d->requestedViewMode);
+    emit listViewModeChanged(d->requestedViewMode);
+
+    // The horizontal layouts size the items based widget height not user configured base length
+    // so it needs to be restored when switching back to vertical layout
+    KisResourceItemChooserSync *chooserSync = KisResourceItemChooserSync::instance();
+    d->view->setItemSize(QSize(chooserSync->baseLength(), chooserSync->baseLength()));
+
+    QGridLayout* thisLayout = dynamic_cast<QGridLayout*>(layout());
+    thisLayout->addWidget(d->tagManager->tagChooserWidget(), 0, 0);
+    thisLayout->addWidget(d->viewModeButton, 0, 1);
+    thisLayout->addWidget(d->storagePopupButton, 0, 2);
+    thisLayout->addWidget(d->resourcesSplitter, 1, 0, 1, 3);
+    thisLayout->setRowStretch(1, 1);
+    thisLayout->addWidget(d->tagManager->tagFilterWidget(), 2, 0, 1, 3);
+    thisLayout->addWidget(d->importExportBtns, 3, 0, 1, 3);
+
+    d->viewModeButton->setVisible(d->showViewModeBtn);
+    d->storagePopupButton->setVisible(d->showStoragePopupBtn);
+
+    d->viewModeButton->setVisible(d->showViewModeBtn);
+    d->layout = Layout::Vertical;
+}
+
+void KisResourceItemChooser::changeLayoutBasedOnSize()
+{
+    if (d->isResponsive == false) {
+        return;
+    }
+    // Responsive Layout
+    else {
+        // Vertical
+        if (height() > 100) {
+            applyVerticalLayout();
+        }
+        // Horizontal 2 rows
+        else if (height() > 60) {
+
+            if (d->layout == Layout::Horizontal2Rows) {
+                return;
+            }
+
+            hideEverything();
+
+            d->view->setListViewMode(ListViewMode::IconStripHorizontal);
+            emit listViewModeChanged(ListViewMode::IconStripHorizontal);
+
+            // Left
+            QLayout* leftLayout = d->left->layout();
+            leftLayout->addWidget(d->resourcesSplitter);
+
+            // Right Top
+            d->top->addWidget(d->scroll_left);
+            d->top->addWidget(d->scroll_right);
+            d->top->addWidget(d->tagManager->tagChooserWidget());
+            d->top->addWidget(d->importExportBtns);
+
+            // Right Bot
+            d->bot->addWidget(d->viewModeButton);
+            d->bot->addWidget(d->storagePopupButton);
+            d->bot->addWidget(d->tagManager->tagFilterWidget());
+
+            d->horzSplitter->addWidget(d->left);
+            d->horzSplitter->addWidget(d->right2Rows);
+
+            QGridLayout* thisLayout = dynamic_cast<QGridLayout*>(layout());
+            thisLayout->addWidget(d->horzSplitter, 0, 0);
+            thisLayout->setRowStretch(0, 2);
+            thisLayout->setColumnStretch(0, 2);
+            thisLayout->setRowStretch(1, 0);
+
+            d->viewModeButton->setVisible(false);
+            d->storagePopupButton->setVisible(d->showStoragePopupBtn);
+
+            const bool splitterRestored = d->restoreSplitterState(Layout::Horizontal2Rows);
+
+            d->horzSplitter->show();
+            d->left->show();
+            d->scroll_left->show();
+            d->scroll_right->show();
+            d->right2Rows->show();
+
+            d->layout = Layout::Horizontal2Rows;
+
+            if (!splitterRestored) {
+                slotSaveSplitterState();
+            }
+        }
+        // Horizontal 1 row
+        else {
+            if (d->layout == Layout::Horizontal1Row) {
+                return;
+            }
+
+            hideEverything();
+
+            d->view->setListViewMode(ListViewMode::IconStripHorizontal);
+            emit listViewModeChanged(ListViewMode::IconStripHorizontal);
+
+            QLayout* leftLayout = d->left->layout();
+            leftLayout->addWidget(d->resourcesSplitter);
+            leftLayout->addWidget(d->scroll_left);
+            leftLayout->addWidget(d->scroll_right);
+
+            QLayout* rightLayout = d->right->layout();
+            rightLayout->addWidget(d->tagManager->tagChooserWidget());
+            rightLayout->addWidget(d->viewModeButton);
+            rightLayout->addWidget(d->storagePopupButton);
+            rightLayout->addWidget(d->tagManager->tagFilterWidget());
+            rightLayout->addWidget(d->importExportBtns);
+
+            d->horzSplitter->addWidget(d->left);
+            d->horzSplitter->addWidget(d->right);
+
+            QGridLayout* thisLayout = dynamic_cast<QGridLayout*>(layout());
+            thisLayout->addWidget(d->horzSplitter, 0, 0);
+            thisLayout->setRowStretch(0, 2);
+            thisLayout->setColumnStretch(0, 2);
+            thisLayout->setRowStretch(1, 0);
+
+            d->viewModeButton->setVisible(false);
+            d->storagePopupButton->setVisible(d->showStoragePopupBtn);
+
+            const bool splitterRestored = d->restoreSplitterState(Layout::Horizontal1Row);
+
+            d->horzSplitter->show();
+            d->left->show();
+            d->scroll_left->show();
+            d->scroll_right->show();
+            d->right->show();
+
+            d->layout = Layout::Horizontal1Row;
+
+            if (!splitterRestored) {
+                slotSaveSplitterState();
+            }
+        }
+    }
+}
+
+bool KisResourceItemChooser::Private::restoreSplitterState(Layout layout)
+{
+    KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(layout > Layout::Vertical, false);
+
+    KConfigGroup group = KSharedConfig::openConfig()->group(QString("KisResourceItemChooser_%1").arg(resourceType));
+
+    const QLatin1String key(
+        layout == Layout::Horizontal1Row ? "splitterState_1row" : "splitterState_2row");
+
+    bool splitterRestored = false;
+    QByteArray state = group.readEntry(key, QByteArray());
+    if (!state.isEmpty()) {
+        splitterRestored = horzSplitter->restoreState(state);
+        KIS_SAFE_ASSERT_RECOVER_NOOP(splitterRestored);
+    }
+
+    return splitterRestored;
+}
+
+void KisResourceItemChooser::slotSaveSplitterState()
+{
+    if (d->layout > Layout::Vertical) {
+        KConfigGroup group = KSharedConfig::openConfig()->group(QString("KisResourceItemChooser_%1").arg(d->resourceType));
+        const QByteArray state = d->horzSplitter->saveState();
+
+        const QLatin1String key(
+            d->layout == Layout::Horizontal1Row ? "splitterState_1row" : "splitterState_2row");
+
+        group.writeEntry(key, state);
+    }
+}
+
+void KisResourceItemChooser::updateView()
+{
     /// helps to set icons here in case the theme is changed
     d->viewModeButton->setIcon(KisIconUtils::loadIcon("view-choose"));
     d->importButton->setIcon(koIcon("document-import-16"));
