@@ -30,7 +30,7 @@
 /// In case of tagging the user can choose to create a new tag or select one of the existing ones.
 /// In case of untagging the user can untag from the current selected tag (in the combobox) or from some other tags.
 /// This class needs to provide correct lists of tags and take into account that "All" and "All Untagged" (and possibly other
-/// generated tags) are special and the user cannot untage the resource from it.
+/// generated tags) are special and the user cannot untag the resource from it.
 ///
 class KisResourceItemChooserContextMenu :  public QMenu
 {
@@ -83,4 +83,4 @@ private:
 
 };
 
-#endif // KORESOURCEITEMCHOOSERCONTEXTMENU_H
+#endif // KISRESOURCEITEMCHOOSERCONTEXTMENU_H
