@@ -24,7 +24,7 @@
 #include <KisTagModel.h>
 
 #include "KisResourceItemChooserContextMenu.h"
-
+#include "KisMenuStyleDontCloseOnAlt.h"
 
 class KisTagToolButton::Private
 {
@@ -59,6 +59,10 @@ KisTagToolButton::KisTagToolButton(QWidget* parent)
 
     QMenu* popup = new QMenu(this);
 
+    KisMenuStyleDontCloseOnAlt *menuStyle = new KisMenuStyleDontCloseOnAlt(popup->style());
+    menuStyle->setParent(popup);
+    popup->setStyle(menuStyle);
+
     d->addTagAction = new UserInputTagAction(popup);
     d->addTagAction->setPlaceholderText(i18n("New tag"));
     d->addTagAction->setIcon(koIcon("document-new"));
