@@ -9,6 +9,7 @@
 #include <QTimer>
 #include <QThread>
 
+#include "KisMpl.h"
 #include "kis_image.h"
 #include "kis_image_animation_interface.h"
 #include "kis_signal_auto_connection.h"
@@ -51,7 +52,7 @@ KisAsyncAnimationRendererBase::~KisAsyncAnimationRendererBase()
 
 }
 
-void KisAsyncAnimationRendererBase::startFrameRegeneration(KisImageSP image, int frame, const KisRegion &regionOfInterest, Flags flags)
+void KisAsyncAnimationRendererBase::startFrameRegeneration(KisImageSP image, int frame, const KisRegion &regionOfInterest, Flags flags, KisLockFrameGenerationLock &&frameGenerationLock)
 {
     KIS_SAFE_ASSERT_RECOVER_NOOP(QThread::currentThread() == this->thread());
 
@@ -74,12 +75,12 @@ void KisAsyncAnimationRendererBase::startFrameRegeneration(KisImageSP image, int
                 Qt::AutoConnection);
 
     m_d->regenerationTimeout.start();
-    animation->requestFrameRegeneration(m_d->requestedFrame, m_d->requestedRegion, flags & Cancellable);
+    animation->requestFrameRegeneration(m_d->requestedFrame, m_d->requestedRegion, flags & Cancellable, std::move(frameGenerationLock));
 }
 
-void KisAsyncAnimationRendererBase::startFrameRegeneration(KisImageSP image, int frame, Flags flags)
+void KisAsyncAnimationRendererBase::startFrameRegeneration(KisImageSP image, int frame, Flags flags, KisLockFrameGenerationLock &&frameGenerationLock)
 {
-    startFrameRegeneration(image, frame, KisRegion(), flags);
+    startFrameRegeneration(image, frame, KisRegion(), flags, std::move(frameGenerationLock));
 }
 
 bool KisAsyncAnimationRendererBase::isActive() const
@@ -129,10 +130,17 @@ void KisAsyncAnimationRendererBase::notifyFrameCompleted(int frame)
     // the processing was cancelled
     if (m_d->isCancelled) return;
 
-    KIS_SAFE_ASSERT_RECOVER_RETURN(m_d->requestedImage);
-    KIS_SAFE_ASSERT_RECOVER_RETURN(m_d->requestedFrame == frame);
+    {
+        // clean state right after the assert checks
+        auto cleanup = kismpl::finally([&] () {
+            clearFrameRegenerationState(false);
+        });
+
+        KIS_SAFE_ASSERT_RECOVER_RETURN(m_d->requestedImage);
+        KIS_SAFE_ASSERT_RECOVER_RETURN(m_d->requestedFrame == frame);
+    }
+
 
-    clearFrameRegenerationState(false);
     emit sigFrameCompleted(frame);
 }
 
@@ -144,10 +152,16 @@ void KisAsyncAnimationRendererBase::notifyFrameCancelled(int frame, CancelReason
     // the processing was cancelled
     if (m_d->isCancelled) return;
 
-    KIS_SAFE_ASSERT_RECOVER_RETURN(m_d->requestedImage);
-    KIS_SAFE_ASSERT_RECOVER_RETURN(m_d->requestedFrame == frame);
+    {
+        // clean state right after the assert checks
+        auto cleanup = kismpl::finally([&] () {
+            clearFrameRegenerationState(true);
+        });
+
+        KIS_SAFE_ASSERT_RECOVER_RETURN(m_d->requestedImage);
+        KIS_SAFE_ASSERT_RECOVER_RETURN(m_d->requestedFrame == frame);
+    }
 
-    clearFrameRegenerationState(true);
     emit sigFrameCancelled(frame, cancelReason);
 }
 
