@@ -1,20 +1,7 @@
 /* This file is part of the KDE project
-   Copyright (C) 2005 Peter Simonsson <psn@linux.se>
+   SPDX-FileCopyrightText: 2005 Peter Simonsson <psn@linux.se>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+   SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "KisDetailsPane.h"
@@ -40,7 +27,6 @@ KisDetailsPane::KisDetailsPane(QWidget* parent, const QString& header)
 
     setupUi(this);
 
-    m_previewLabel->installEventFilter(this);
     m_documentList->installEventFilter(this);
     m_documentList->setIconSize(QSize(IconExtent, IconExtent));
     m_documentList->setModel(&d->m_model);
@@ -48,10 +34,10 @@ KisDetailsPane::KisDetailsPane(QWidget* parent, const QString& header)
 
     changePalette();
 
-    connect(m_documentList->selectionModel(), SIGNAL(currentChanged(const QModelIndex&, const QModelIndex&)),
-            this, SLOT(selectionChanged(const QModelIndex&)));
-    connect(m_documentList, SIGNAL(doubleClicked(const QModelIndex&)),
-            this, SLOT(openFile(const QModelIndex&)));
+    connect(m_documentList->selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)),
+            this, SLOT(selectionChanged(QModelIndex)));
+    connect(m_documentList, SIGNAL(doubleClicked(QModelIndex)),
+            this, SLOT(openFile(QModelIndex)));
     connect(m_openButton, SIGNAL(clicked()), this, SLOT(openFile()));
 }
 
@@ -62,12 +48,6 @@ KisDetailsPane::~KisDetailsPane()
 
 bool KisDetailsPane::eventFilter(QObject* watched, QEvent* e)
 {
-    if (watched == m_previewLabel) {
-        if (e->type() == QEvent::MouseButtonDblClick) {
-            openFile();
-        }
-    }
-
     if (watched == m_documentList) {
         if ((e->type() == QEvent::Resize) && isVisible()) {
             emit splitterResized(this, m_splitter->sizes());
@@ -103,7 +83,7 @@ void KisDetailsPane::changePalette()
 {
     QPalette p = palette();
     p.setBrush(QPalette::Base, QColor(Qt::transparent));
-    p.setColor(QPalette::Text, p.color(QPalette::Normal, QPalette::Foreground));
+    p.setColor(QPalette::Text, p.color(QPalette::Normal, QPalette::WindowText));
     m_detailsLabel->setPalette(p);
 }
 
