@@ -6,7 +6,7 @@
  */
 
 #include "KisMainWindow.h" // XXX: remove
-#include <QMessageBox> // XXX: remove
+#include <QMessageBox>
 
 #include <KisMimeDatabase.h>
 
@@ -111,11 +111,10 @@
 #include "KisView.h"
 #include "kis_grid_config.h"
 #include "kis_guides_config.h"
-#include "kis_image_barrier_lock_adapter.h"
+#include "KisImageBarrierLock.h"
 #include "KisReferenceImagesLayer.h"
 #include "dialogs/KisRecoverNamedAutosaveDialog.h"
 
-
 #include <mutex>
 #include "kis_config_notifier.h"
 #include "kis_async_action_feedback.h"
@@ -125,6 +124,7 @@
 #include <KisMirrorAxisConfig.h>
 #include <KisDecorationsWrapperLayer.h>
 #include "kis_simple_stroke_strategy.h"
+#include <KisCursorOverrideLock.h>
 
 // Define the protocol used here for embedded documents' URL
 // This used to "store" but QUrl didn't like it,
@@ -137,6 +137,10 @@
 
 #include <unistd.h>
 
+#ifdef Q_OS_MACOS
+#include "KisMacosSecurityBookmarkManager.h"
+#endif
+
 using namespace std;
 
 namespace {
@@ -385,6 +389,9 @@ public:
     StoryboardItemList m_storyboardItemList;
     QVector<StoryboardComment> m_storyboardCommentList;
 
+    QVector<QFileInfo> audioTracks;
+    qreal audioLevel = 1.0;
+
     QColor globalAssistantsColor;
 
     KisGridConfig gridConfig;
@@ -499,6 +506,8 @@ void KisDocument::Private::copyFromImpl(const Private &rhs, KisDocument *q, KisD
         q->setAssistants(KisPaintingAssistant::cloneAssistantList(rhs.assistants));
         q->setStoryboardItemList(StoryboardItem::cloneStoryboardItemList(rhs.m_storyboardItemList));
         q->setStoryboardCommentList(rhs.m_storyboardCommentList);
+        q->setAudioTracks(rhs.audioTracks);
+        q->setAudioVolume(rhs.audioLevel);
         q->setGridConfig(rhs.gridConfig);
     } else {
         // in CONSTRUCT mode, we cannot use the functions of KisDocument
@@ -509,6 +518,8 @@ void KisDocument::Private::copyFromImpl(const Private &rhs, KisDocument *q, KisD
         assistants = KisPaintingAssistant::cloneAssistantList(rhs.assistants);
         m_storyboardItemList = StoryboardItem::cloneStoryboardItemList(rhs.m_storyboardItemList);
         m_storyboardCommentList = rhs.m_storyboardCommentList;
+        audioTracks = rhs.audioTracks;
+        audioLevel = rhs.audioLevel;
         gridConfig = rhs.gridConfig;
     }
     imageModifiedWithoutUndo = rhs.imageModifiedWithoutUndo;
@@ -539,7 +550,7 @@ public:
         : m_locked(false)
         , m_image(image)
         , m_savingLock(savingMutex)
-        , m_imageLock(image, true)
+        , m_imageLock(image)
 
     {
         /**
@@ -551,21 +562,21 @@ public:
          * Since we are trying to lock multiple objects, so we should
          * do it in a safe manner.
          */
-        m_locked = std::try_lock(m_imageLock, m_savingLock) < 0;
+        m_locked = std::try_lock(m_imageLock, *m_savingLock) < 0;
 
         if (!m_locked) {
             m_image->requestStrokeEnd();
             QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
 
             // one more try...
-            m_locked = std::try_lock(m_imageLock, m_savingLock) < 0;
+            m_locked = std::try_lock(m_imageLock, *m_savingLock) < 0;
         }
     }
 
     ~StrippedSafeSavingLocker() {
         if (m_locked) {
             m_imageLock.unlock();
-            m_savingLock.unlock();
+            m_savingLock->unlock();
         }
     }
 
@@ -574,10 +585,12 @@ public:
     }
 
 private:
+    Q_DISABLE_COPY_MOVE(StrippedSafeSavingLocker)
+
     bool m_locked;
     KisImageSP m_image;
-    StdLockableWrapper<QMutex> m_savingLock;
-    KisImageBarrierLockAdapter m_imageLock;
+    QMutex *m_savingLock;
+    KisImageReadOnlyBarrierLockAdapter m_imageLock;
 };
 
 KisDocument::KisDocument(bool addStorage)
@@ -588,6 +601,13 @@ KisDocument::KisDocument(bool addStorage)
     connect(d->autoSaveTimer, SIGNAL(timeout()), this, SLOT(slotAutoSave()));
     setObjectName(newObjectName());
 
+#ifdef Q_OS_MACOS
+    KisMacosSecurityBookmarkManager *bookmarkmngr = KisMacosSecurityBookmarkManager::instance();
+    if (bookmarkmngr->isSandboxed()) {
+        connect(this, SIGNAL(sigSavingFinished(const QString&)), bookmarkmngr, SLOT(slotCreateBookmark(const QString&)));
+    }
+#endif
+
 
     if (addStorage) {
         d->linkedResourcesStorageID = QUuid::createUuid().toString();
@@ -734,6 +754,18 @@ bool KisDocument::exportDocumentImpl(const KritaUtils::ExportFileJob &job, KisPr
             QDir().mkpath(backupDir);
 #endif
 
+#ifdef Q_OS_MACOS
+            KisMacosSecurityBookmarkManager *bookmarkmngr = KisMacosSecurityBookmarkManager::instance();
+            if (bookmarkmngr->isSandboxed()) {
+                // If the user does not have directory permission force backup
+                // files to be inside Container tmp
+                QUrl fileUrl = QUrl::fromLocalFile(job.filePath);
+                if( !bookmarkmngr->parentDirHasPermissions(fileUrl.path()) ) {
+                    backupDir = QDir::tempPath();
+                }
+            }
+#endif
+
             // Do nothing: the empty string is user file location
             break;
         }
@@ -748,6 +780,7 @@ bool KisDocument::exportDocumentImpl(const KritaUtils::ExportFileJob &job, KisPr
                                         .arg(job.filePath, backupDir.isEmpty()
                                                                ? "the same location as the file"
                                                                : backupDir));
+                slotCompleteSavingDocument(job, ImportExportCodes::ErrorWhileWriting, i18nc("Saving error message", "Failed to create a backup file"), "");
                 return false;
             }
             else {
@@ -764,6 +797,7 @@ bool KisDocument::exportDocumentImpl(const KritaUtils::ExportFileJob &job, KisPr
                                         .arg(job.filePath, backupDir.isEmpty()
                                                                ? "the same location as the file"
                                                                : backupDir));
+                slotCompleteSavingDocument(job, ImportExportCodes::ErrorWhileWriting, i18nc("Saving error message", "Failed to create a numbered backup file"), "");
                 return false;
             }
             else {
@@ -878,39 +912,41 @@ QByteArray KisDocument::serializeToNativeByteArray()
     return buffer.data();
 }
 
-class DlgLoadMessages : public KoDialog
+class DlgLoadMessages : public QMessageBox
 {
 public:
     DlgLoadMessages(const QString &title,
                     const QString &message,
-                    const QStringList &warnings)
-        : KoDialog(qApp->activeWindow())
+                    const QStringList &warnings = {},
+                    const QString &details = {})
+        : QMessageBox(QMessageBox::Warning, title, message, QMessageBox::Ok, qApp->activeWindow())
     {
-        setWindowTitle(title);
-        QWidget *page = new QWidget();
-        QVBoxLayout *layout = new QVBoxLayout(page);
-        QHBoxLayout *hlayout = new QHBoxLayout();
-        QLabel *labelWarning= new QLabel();
-        labelWarning->setPixmap(KisIconUtils::loadIcon("warning").pixmap(32, 32));
-        labelWarning->setAlignment(Qt::AlignTop);
-        hlayout->addWidget(labelWarning);
-        hlayout->addWidget(new QLabel(message), 1);
-        layout->addLayout(hlayout);
+        if (!details.isEmpty()) {
+            setInformativeText(details);
+        }
         if (!warnings.isEmpty()) {
-            QTextBrowser *browser = new QTextBrowser();
+            setDetailedText(warnings);
+        }
+    }
+
+private:
+    void setDetailedText(const QStringList &text)
+    {
+        QMessageBox::setDetailedText(text.first());
+
+        QTextEdit *messageBox = findChild<QTextEdit *>();
+
+        if (messageBox) {
+            messageBox->setAcceptRichText(true);
+
             QString warning = "<html><body><ul>";
-            Q_FOREACH (const QString &w, warnings) {
-                warning += "\n<li>" + w + "</li>";
+            Q_FOREACH (const QString &i, text) {
+                warning += "\n<li>" + i + "</li>";
             }
-            warning += "</ul>";
-            browser->setHtml(warning);
-            browser->setMinimumHeight(200);
-            browser->setMinimumWidth(400);
-            layout->addWidget(browser);
+            warning += "</ul></body></html>";
+
+            messageBox->setText(warning);
         }
-        setMainWidget(page);
-        page->setParent(this);
-        setButtons(KoDialog::Ok);
     }
 };
 
@@ -927,19 +963,46 @@ void KisDocument::slotCompleteSavingDocument(const KritaUtils::ExportFileJob &jo
                                     fileName,
                                     errorMessage), errorMessageTimeout);
 
+
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
         if (!fileBatchMode()) {
             DlgLoadMessages dlg(i18nc("@title:window", "Krita"),
-                                i18n("Could not save %1.\nReason: %2", job.filePath, status.errorMessage()),
-                                errorMessage.split("\n") + warningMessage.split("\n"));
+                                i18n("Could not save %1.", job.filePath),
+                                errorMessage.split("\n", Qt::SkipEmptyParts)
+                                    + warningMessage.split("\n", Qt::SkipEmptyParts),
+                                status.errorMessage());
+#else
+        if (!fileBatchMode()) {
+            DlgLoadMessages dlg(i18nc("@title:window", "Krita"),
+                                i18n("Could not save %1.", job.filePath),
+                                errorMessage.split("\n", QString::SkipEmptyParts)
+                                    + warningMessage.split("\n", QString::SkipEmptyParts),
+                                status.errorMessage());
+#endif
+
             dlg.exec();
         }
     }
     else {
         if (!fileBatchMode() && !warningMessage.isEmpty()) {
-            DlgLoadMessages dlg(i18nc("@title:window", "Krita"),
-                                i18nc("dialog box shown to the user if there were warnings while saving the document, %1 is the file path",
-                                      "%1 has been saved but is incomplete.\nThe following problems were encountered when saving:", job.filePath),
-                                warningMessage.split("\n"));
+
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
+            QStringList reasons = warningMessage.split("\n", Qt::SkipEmptyParts);
+#else
+            QStringList reasons = warningMessage.split("\n", QString::SkipEmptyParts);
+#endif
+
+            DlgLoadMessages dlg(
+                i18nc("@title:window", "Krita"),
+                i18nc("dialog box shown to the user if there were warnings while saving the document, "
+                      "%1 is the file path",
+                      "%1 has been saved but is incomplete.",
+                      job.filePath),
+                reasons,
+                reasons.isEmpty()
+                    ? ""
+                    : i18nc("dialog box shown to the user if there were warnings while saving the document",
+                            "Some problems were encountered when saving."));
             dlg.exec();
         }
 
@@ -1431,7 +1494,7 @@ bool KisDocument::resourceSavingFilter(const QString &path, const QByteArray &mi
                         }
 
                         if (exportConfiguration) {
-                            // make sure the the name of the resource doesn't change
+                            // make sure the name of the resource doesn't change
                             exportConfiguration->setProperty("name", res->name());
                         }
 
@@ -1553,7 +1616,7 @@ bool KisDocument::startExportInBackground(const QString &actionName,
 
     KisImportExportErrorCode initializationStatus(ImportExportCodes::OK);
     d->childSavingFuture =
-            d->importExportManager->exportDocumentAsyc(location,
+            d->importExportManager->exportDocumentAsync(location,
                                                        realLocation,
                                                        mimeType,
                                                        initializationStatus,
@@ -1613,12 +1676,18 @@ void KisDocument::finishExportInBackground()
 
 void KisDocument::setReadWrite(bool readwrite)
 {
+    const bool changed = readwrite != d->readwrite;
+
     d->readwrite = readwrite;
     setNormalAutoSaveInterval();
 
     Q_FOREACH (KisMainWindow *mainWindow, KisPart::instance()->mainWindows()) {
         mainWindow->setReadWrite(readwrite);
     }
+
+    if (changed) {
+        Q_EMIT sigReadWriteChanged(readwrite);
+    }
 }
 
 void KisDocument::setAutoSaveDelay(int delay)
@@ -1758,7 +1827,6 @@ bool KisDocument::importDocument(const QString &_path)
     if (ret) {
         dbgUI << "success, resetting url";
         resetPath();
-        setTitleModified();
     }
 
     return ret;
@@ -1881,17 +1949,35 @@ bool KisDocument::openFile()
         KisUsageLogger::log(QString("Loading %1 failed: %2").arg(prettyPath(), msg));
 
         if (!msg.isEmpty() && !fileBatchMode()) {
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
             DlgLoadMessages dlg(i18nc("@title:window", "Krita"),
-                                i18n("Could not open %2.\nReason: %1", msg, prettyPath()),
-                                errorMessage().split("\n") + warningMessage().split("\n"));
+                                i18n("Could not open %1.", prettyPath()),
+                                errorMessage().split("\n", Qt::SkipEmptyParts)
+                                    + warningMessage().split("\n", Qt::SkipEmptyParts),
+                                msg);
+#else
+            DlgLoadMessages dlg(i18nc("@title:window", "Krita"),
+                                i18n("Could not open %1.", prettyPath()),
+                                errorMessage().split("\n", QString::SkipEmptyParts)
+                                    + warningMessage().split("\n", QString::SkipEmptyParts),
+                                msg);
+#endif
+
             dlg.exec();
         }
         return false;
     }
     else if (!warningMessage().isEmpty() && !fileBatchMode()) {
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
         DlgLoadMessages dlg(i18nc("@title:window", "Krita"),
-                            i18n("There were problems opening %1", prettyPath()),
-                            warningMessage().split("\n"));
+                            i18n("There were problems opening %1.", prettyPath()),
+                            warningMessage().split("\n", Qt::SkipEmptyParts));
+#else
+        DlgLoadMessages dlg(i18nc("@title:window", "Krita"),
+                            i18n("There were problems opening %1.", prettyPath()),
+                            warningMessage().split("\n", QString::SkipEmptyParts));
+#endif
+
         dlg.exec();
         setPath(QString());
     }
@@ -1970,14 +2056,18 @@ void KisDocument::setModified(bool mod)
         documentInfo()->updateParameters();
     }
 
-    // This influences the title
-    setTitleModified();
     emit modified(mod);
 }
 
 void KisDocument::setRecovered(bool value)
 {
+    const bool changed = value != d->isRecovered;
+
     d->isRecovered = value;
+
+    if (changed) {
+        Q_EMIT sigRecoveredChanged(value);
+    }
 }
 
 bool KisDocument::isRecovered() const
@@ -2027,11 +2117,6 @@ QString KisDocument::caption() const
     return c;
 }
 
-void KisDocument::setTitleModified()
-{
-    emit titleModified(caption(), isModified());
-}
-
 QDomDocument KisDocument::createDomDocument(const QString& tagName, const QString& version) const
 {
     return createDomDocument("krita", tagName, version);
@@ -2153,6 +2238,8 @@ void KisDocument::slotConfigChanged()
         }
         d->undoStack->setUndoLimit(cfg.undoStackLimit());
     }
+    d->undoStack->setUseCumulativeUndoRedo(cfg.useCumulativeUndoRedo());
+    d->undoStack->setCumulativeUndoData(cfg.cumulativeUndoData());
 
     d->autoSaveDelay = cfg.autoSaveInterval();
     setNormalAutoSaveInterval();
@@ -2196,11 +2283,11 @@ QList<KoResourceLoadResult> KisDocument::linkedDocumentResources()
 
             QBuffer buf;
             buf.open(QBuffer::WriteOnly);
-            bool exportSuccessfull =
+            bool exportSuccessful =
                 d->linkedResourceStorage->exportResource(iter->url(), &buf);
 
             KoResourceSP resource = d->linkedResourceStorage->resource(iter->url());
-            exportSuccessfull &= bool(resource);
+            exportSuccessful &= bool(resource);
 
             const QString name = resource ? resource->name() : QString();
             const QString fileName = QFileInfo(iter->url()).fileName();
@@ -2208,7 +2295,7 @@ QList<KoResourceLoadResult> KisDocument::linkedDocumentResources()
                                                 KoMD5Generator::generateHash(buf.data()),
                                                 fileName, name);
 
-            if (exportSuccessfull) {
+            if (exportSuccessful) {
                 result << KoEmbeddedResource(signature, buf.data());
             } else {
                 result << signature;
@@ -2280,6 +2367,27 @@ void KisDocument::setStoryboardCommentList(const QVector<StoryboardComment> &sto
     }
 }
 
+QVector<QFileInfo> KisDocument::getAudioTracks() const {
+    return d->audioTracks;
+}
+
+void KisDocument::setAudioTracks(QVector<QFileInfo> f)
+{
+    d->audioTracks = f;
+    emit sigAudioTracksChanged();
+}
+
+void KisDocument::setAudioVolume(qreal level)
+{
+    d->audioLevel = level;
+    emit sigAudioLevelChanged(level);
+}
+
+qreal KisDocument::getAudioLevel()
+{
+    return d->audioLevel;
+}
+
 const KisGuidesConfig& KisDocument::guidesConfig() const
 {
     return d->guidesConfig;
@@ -2362,7 +2470,13 @@ bool KisDocument::closePath(bool promptToSave)
 
 void KisDocument::setPath(const QString &path)
 {
+    const bool changed = path != d->m_path;
+
     d->m_path = path;
+
+    if (changed) {
+        Q_EMIT sigPathChanged(path);
+    }
 }
 
 QString KisDocument::localFilePath() const
@@ -2435,7 +2549,7 @@ bool KisDocument::newImage(const QString& name,
 
     if (!cs) return false;
 
-    QApplication::setOverrideCursor(Qt::BusyCursor);
+    KisCursorOverrideLock cursorLock(Qt::BusyCursor);
 
     image = new KisImage(createUndoStore(), width, height, cs, name);
 
@@ -2470,7 +2584,7 @@ bool KisDocument::newImage(const QString& name,
         strippedAlpha.setOpacity(OPACITY_OPAQUE_U8);
 
         if (bgStyle == KisConfig::RASTER_LAYER) {
-            bgLayer = new KisPaintLayer(image.data(), i18n("Background"), OPACITY_OPAQUE_U8, cs);
+            bgLayer = new KisPaintLayer(image.data(), i18nc("Name for the bottom-most layer in the layerstack", "Background"), OPACITY_OPAQUE_U8, cs);
             bgLayer->paintDevice()->setDefaultPixel(strippedAlpha);
             bgLayer->setPinnedToTimeline(autopin);
         } else if (bgStyle == KisConfig::FILL_LAYER) {
@@ -2514,8 +2628,6 @@ bool KisDocument::newImage(const QString& name,
                  image->colorSpace()->colorDepthId().name(), image->colorSpace()->profile()->name(),
                  QString::number(numberOfLayers)));
 
-    QApplication::restoreOverrideCursor();
-
     return true;
 }
 
@@ -2532,7 +2644,7 @@ void KisDocument::waitForSavingToComplete()
 {
     if (isSaving()) {
         KisAsyncActionFeedback f(i18nc("progress dialog message when the user closes the document that is being saved", "Waiting for saving to complete..."), 0);
-        f.waitForMutex(&d->savingMutex);
+        f.waitForMutex(d->savingMutex);
     }
 }
 
