@@ -50,6 +50,7 @@
 #include <kis_painter.h>
 
 #include "KisDocument.h"
+#include "KisImportExportErrorCode.h"
 #include "KisImportExportFilter.h"
 #include "KisMainWindow.h"
 #include "KisReferenceImagesLayer.h"
@@ -59,10 +60,36 @@
 #include "kis_grid_config.h"
 #include "kis_guides_config.h"
 
+#include <KisImportUserFeedbackInterface.h>
+
 // static cache for import and export mimetypes
 QStringList KisImportExportManager::m_importMimeTypes;
 QStringList KisImportExportManager::m_exportMimeTypes;
 
+namespace {
+struct SynchronousUserFeedbackInterface : KisImportUserFeedbackInterface
+{
+    SynchronousUserFeedbackInterface(QWidget *parent, bool batchMode)
+        : m_parent(parent)
+        , m_batchMode(batchMode)
+    {
+    }
+
+    Result askUser(AskCallback callback) override
+    {
+        if (m_batchMode) return SuppressedByBatchMode;
+        KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(m_parent, SuppressedByBatchMode);
+
+        return callback(m_parent) ? Success : UserCancelled;
+    }
+
+    QWidget *m_parent {nullptr};
+    bool m_batchMode {false};
+};
+
+}
+
+
 class Q_DECL_HIDDEN KisImportExportManager::Private
 {
 public:
@@ -142,7 +169,7 @@ KisImportExportErrorCode KisImportExportManager::exportDocument(const QString& l
     return result.status();
 }
 
-QFuture<KisImportExportErrorCode> KisImportExportManager::exportDocumentAsyc(const QString &location, const QString &realLocation, const QByteArray &mimeType,
+QFuture<KisImportExportErrorCode> KisImportExportManager::exportDocumentAsync(const QString &location, const QString &realLocation, const QByteArray &mimeType,
                                                                             KisImportExportErrorCode &status, bool showWarnings, KisPropertiesConfigurationSP exportConfiguration, bool isAdvancedExporting)
 {
     ConversionResult result = convert(Export, location, realLocation, mimeType, showWarnings, exportConfiguration, true, isAdvancedExporting);
@@ -166,7 +193,12 @@ QStringList KisImportExportManager::supportedMimeTypes(Direction direction)
             QList<KoJsonTrader::Plugin> list = KoJsonTrader::instance()->query("Krita/FileFilter", "");
             Q_FOREACH(const KoJsonTrader::Plugin &loader, list) {
                 QJsonObject json = loader.metaData().value("MetaData").toObject();
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
+                Q_FOREACH(const QString &mimetype, json.value("X-KDE-Import").toString().split(",", Qt::SkipEmptyParts)) {
+#else
                 Q_FOREACH(const QString &mimetype, json.value("X-KDE-Import").toString().split(",", QString::SkipEmptyParts)) {
+#endif
+
                     //qDebug() << "Adding  import mimetype" << mimetype << KisMimeDatabase::descriptionForMimeType(mimetype) << "from plugin" << loader;
                     mimeTypes << mimetype;
                 }
@@ -184,7 +216,12 @@ QStringList KisImportExportManager::supportedMimeTypes(Direction direction)
             QList<KoJsonTrader::Plugin> list = KoJsonTrader::instance()->query("Krita/FileFilter", "");
             Q_FOREACH(const KoJsonTrader::Plugin &loader, list) {
                 QJsonObject json = loader.metaData().value("MetaData").toObject();
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
+                Q_FOREACH(const QString &mimetype, json.value("X-KDE-Export").toString().split(",", Qt::SkipEmptyParts)) {
+#else
                 Q_FOREACH(const QString &mimetype, json.value("X-KDE-Export").toString().split(",", QString::SkipEmptyParts)) {
+#endif
+
                     //qDebug() << "Adding  export mimetype" << mimetype << KisMimeDatabase::descriptionForMimeType(mimetype) << "from plugin" << loader;
                     mimeTypes << mimetype;
                 }
@@ -211,7 +248,12 @@ KisImportExportFilter *KisImportExportManager::filterForMimeType(const QString &
         QJsonObject json = loader.metaData().value("MetaData").toObject();
         QString directionKey = direction == Export ? "X-KDE-Export" : "X-KDE-Import";
 
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
+        if (json.value(directionKey).toString().split(",", Qt::SkipEmptyParts).contains(mimetype)) {
+#else
         if (json.value(directionKey).toString().split(",", QString::SkipEmptyParts).contains(mimetype)) {
+#endif
+
             KLibFactory *factory = qobject_cast<KLibFactory *>(loader.instance());
 
             if (!factory) {
@@ -333,6 +375,11 @@ KisImportExportManager::ConversionResult KisImportExportManager::convert(KisImpo
     filter->setBatchMode(batchMode());
     filter->setMimeType(typeName);
 
+    if (direction == Import) {
+        KisMainWindow *kisMain = KisPart::instance()->currentMainwindow();
+        filter->setImportUserFeedBackInterface(new SynchronousUserFeedbackInterface(kisMain, batchMode()));
+    }
+
     if (!d->updater.isNull()) {
         // WARNING: The updater is not guaranteed to be persistent! If you ever want
         // to add progress reporting to "Save also as .kra", make sure you create
@@ -362,13 +409,8 @@ KisImportExportManager::ConversionResult KisImportExportManager::convert(KisImpo
     if (direction == Import) {
 
         KisUsageLogger::log(QString("Importing %1 to %2. Location: %3. Real location: %4. Batchmode: %5")
-                            .arg(QString::fromLatin1(from))
-                            .arg(QString::fromLatin1(to))
-                            .arg(location)
-                            .arg(realLocation)
-                            .arg(batchMode()));
-
-
+                                .arg(QString::fromLatin1(from), QString::fromLatin1(to), location,
+                                     realLocation, QString::number(batchMode())));
 
         // async importing is not yet supported!
         KIS_SAFE_ASSERT_RECOVER_NOOP(!isAsync);
@@ -383,17 +425,21 @@ KisImportExportManager::ConversionResult KisImportExportManager::convert(KisImpo
         if (result.status().isOk()) {
             KisImageSP image = m_document->image().toStrongRef();
             if (image) {
-                KisUsageLogger::log(QString("Loaded image from %1. Size: %2 * %3 pixels, %4 dpi. Color model: %6 %5 (%7). Layers: %8")
-                                    .arg(QString::fromLatin1(from))
-                                    .arg(image->width())
-                                    .arg(image->height())
-                                    .arg(image->xRes())
-                                    .arg(image->colorSpace()->colorModelId().name())
-                                    .arg(image->colorSpace()->colorDepthId().name())
-                                    .arg(image->colorSpace()->profile()->name())
-                                    .arg(image->nlayers()));
-            }
-            else {
+                KIS_SAFE_ASSERT_RECOVER(image->colorSpace() != nullptr && image->colorSpace()->profile() != nullptr)
+                {
+                    qWarning() << "Loaded a profile-less file without a fallback. Rejecting image "
+                                  "opening";
+                    return KisImportExportErrorCode(ImportExportCodes::InternalError);
+                }
+                KisUsageLogger::log(QString("Loaded image from %1. Size: %2 * %3 pixels, %4 dpi. Color "
+                                            "model: %6 %5 (%7). Layers: %8")
+                                        .arg(QString::fromLatin1(from), QString::number(image->width()),
+                                             QString::number(image->height()), QString::number(image->xRes()),
+                                             image->colorSpace()->colorModelId().name(),
+                                             image->colorSpace()->colorDepthId().name(),
+                                             image->colorSpace()->profile()->name(),
+                                             QString::number(image->nlayers())));
+            } else {
                 qWarning() << "The filter returned OK, but there is no image";
             }
 
@@ -423,13 +469,12 @@ KisImportExportManager::ConversionResult KisImportExportManager::convert(KisImpo
             return KisImportExportErrorCode(ImportExportCodes::Cancelled);
         }
 
-        KisUsageLogger::log(QString("Converting from %1 to %2. Location: %3. Real location: %4. Batchmode: %5. Configuration: %6")
-                            .arg(QString::fromLatin1(from))
-                            .arg(QString::fromLatin1(to))
-                            .arg(location)
-                            .arg(realLocation)
-                            .arg(batchMode())
-                            .arg(exportConfiguration ? exportConfiguration->toXML() : "none"));
+        KisUsageLogger::log(
+            QString(
+                "Converting from %1 to %2. Location: %3. Real location: %4. Batchmode: %5. Configuration: %6")
+                .arg(QString::fromLatin1(from), QString::fromLatin1(to), location, realLocation,
+                     QString::number(batchMode()),
+                     (exportConfiguration ? exportConfiguration->toXML() : "none")));
 
         const QString alsoAsKraLocation = alsoAsKra ? getAlsoAsKraLocation(location) : QString();
         if (isAsync) {
@@ -716,7 +761,10 @@ KisImportExportErrorCode KisImportExportManager::doExport(const QString &locatio
 //                    and without using QSaveFile the issue can still occur
 //                    when QFile::copy fails because Dropbox/Google/OneDrive
 //                    locks the target file.
-#ifndef Q_OS_WIN
+// 02-24-2022 update: Added macOS since QSaveFile does not work on sandboxed krita
+//                    It can work if user gives access to the container dir, but
+//                    we cannot guarantee the user gave us permission.
+#if !(defined(Q_OS_WIN) || defined(Q_OS_MACOS))
 #define USE_QSAVEFILE
 #endif
 
