@@ -10,7 +10,7 @@
 #include <QMap>
 
 #include <QElapsedTimer>
-#include <QWaitCondition>>
+#include <QWaitCondition>
 
 #include "kis_canvas2.h"
 #include "KisCanvasAnimationState.h"
@@ -95,7 +95,7 @@ static void mltOnConsumerFrameShow(mlt_consumer c, void* p_self, mlt_frame p_fra
     KIS_SAFE_ASSERT_RECOVER_RETURN(!iface->waitingForFrame);
     iface->waitingForFrame = true;
 
-    self->sigChangeActiveCanvasFrame(position);
+    emit self->sigChangeActiveCanvasFrame(position);
 
     while (iface->renderingAllowed && iface->waitingForFrame) {
         iface->renderingWaitCondition.wait(&iface->renderingControlMutex);
