@@ -142,21 +142,25 @@ QString KoDocumentInfo::aboutInfo(const QString &info) const
 }
 
 
-bool KoDocumentInfo::loadAuthorInfo(const QDomElement &e)
+bool KoDocumentInfo::loadAuthorInfo(const QDomElement &root)
 {
     m_contact.clear();
-    QDomNode n = e.namedItem("author").firstChild();
-    for (; !n.isNull(); n = n.nextSibling()) {
-        QDomElement e = n.toElement();
-        if (e.isNull())
-            continue;
 
-        if (e.tagName() == "full-name") {
-            setActiveAuthorInfo("creator", e.text().trimmed());
-        } else if (e.tagName() == "contact") {
-            m_contact.insert(e.text(), e.attribute("type"));
+    QDomElement e = root.firstChildElement("author");
+    if(e.isNull()) {
+        return false;
+    }
+
+    for (e = e.firstChildElement(); !e.isNull(); e = e.nextSiblingElement()) {
+        QString field = e.tagName();
+        QString value = e.text();
+
+        if (field == "full-name") {
+            setActiveAuthorInfo("creator", value.trimmed());
+        } else if (field == "contact") {
+            m_contact.insert(value, e.attribute("type"));
         } else {
-            setActiveAuthorInfo(e.tagName(), e.text().trimmed());
+            setActiveAuthorInfo(field, value.trimmed());
         }
     }
 
@@ -189,19 +193,15 @@ QDomElement KoDocumentInfo::saveAuthorInfo(QDomDocument &doc)
 }
 
 
-bool KoDocumentInfo::loadAboutInfo(const QDomElement &e)
+bool KoDocumentInfo::loadAboutInfo(const QDomElement &root)
 {
-    QDomNode n = e.namedItem("about").firstChild();
-    QDomElement tmp;
-    for (; !n.isNull(); n = n.nextSibling()) {
-        tmp = n.toElement();
-        if (tmp.isNull())
-            continue;
+    QDomElement e = root.firstChildElement("about");
+    if(e.isNull()) {
+        return false;
+    }
 
-        if (tmp.tagName() == "abstract")
-            setAboutInfo("abstract", tmp.text());
-
-        setAboutInfo(tmp.tagName(), tmp.text());
+    for (e = e.firstChildElement(); !e.isNull(); e = e.nextSiblingElement()) {
+        setAboutInfo(e.tagName(), e.text());
     }
 
     return true;
