@@ -31,7 +31,7 @@
 #include "kis_pixel_selection.h"
 #include "kis_paint_layer.h"
 #include "kis_image.h"
-#include "kis_image_barrier_locker.h"
+#include "KisImageBarrierLock.h"
 #include "kis_fill_painter.h"
 #include "kis_transaction.h"
 #include "kis_iterator_ng.h"
@@ -83,7 +83,7 @@ namespace ActionHelper {
         // We need to allow for trimming from non-transparent defaultPixel layers.
         // Default color should be phased out of use when the area in question is not aligned with image bounds.
         // Otherwise, we can maintain default pixel.
-        const bool hasNonTransparentDefaultPixel = device->defaultPixel() != KoColor(Qt::transparent, device->colorSpace());
+        const bool hasNonTransparentDefaultPixel = device->defaultPixel() != KoColor::createTransparent(device->colorSpace());
         const bool needsTransparentPixel = selection && rc != image->bounds() && hasNonTransparentDefaultPixel;
 
         if (selection) {
@@ -117,7 +117,7 @@ namespace ActionHelper {
 
 
         if ( needsTransparentPixel ) {
-            device->setDefaultPixel(KoColor(Qt::transparent, device->colorSpace()));
+            device->setDefaultPixel(KoColor::createTransparent(device->colorSpace()));
             device->purgeDefaultPixels();
         }
 
@@ -287,7 +287,7 @@ void KisCutCopyActionFactory::run(bool willCut, bool makeSharpClip, KisViewManag
     if (!makeSharpClip && (haveShapesSelected || currentToolHasSelection)) {
         // XXX: "Add saving of XML data for Cut/Copy of shapes"
 
-        KisImageBarrierLocker locker(image);
+        KisImageBarrierLock lock(image);
         if (willCut) {
             view->canvasBase()->toolProxy()->cut();
         } else {
@@ -303,7 +303,7 @@ void KisCutCopyActionFactory::run(bool willCut, bool makeSharpClip, KisViewManag
             }
         }
 
-        selectedNodes = KisLayerUtils::sortAndFilterMergableInternalNodes(selectedNodes);
+        selectedNodes = KisLayerUtils::sortAndFilterMergeableInternalNodes(selectedNodes);
 
         KisNodeList nodes;
         Q_FOREACH (KisNodeSP node, selectedNodes) {
