@@ -226,6 +226,33 @@ void KisMirrorAxis::drawDecoration(QPainter& gc, const QRectF& updateArea, const
 
 }
 
+static KoPointerEvent *getKoPointerEvent(QEvent *event)
+{
+    switch (event->type()) {
+    case QEvent::MouseButtonPress:
+    case QEvent::MouseMove:
+    case QEvent::MouseButtonRelease: {
+        QMouseEvent *me = static_cast<QMouseEvent *>(event);
+        return new KoPointerEvent(me, me->pos());
+    }
+    case QEvent::TabletPress:
+    case QEvent::TabletMove:
+    case QEvent::TabletRelease: {
+        QTabletEvent *te = static_cast<QTabletEvent *>(event);
+        return new KoPointerEvent(te, te->pos());
+    }
+    case QEvent::TouchBegin:
+    case QEvent::TouchUpdate:
+    case QEvent::TouchEnd:
+    case QEvent::TouchCancel: {
+        QTouchEvent *te = static_cast<QTouchEvent *>(event);
+        return new KoPointerEvent(te, te->touchPoints().at(0).pos());
+    }
+    default:
+        return nullptr;
+    }
+}
+
 bool KisMirrorAxis::eventFilter(QObject* target, QEvent* event)
 {
     if (!visible()) return false;
@@ -235,10 +262,10 @@ bool KisMirrorAxis::eventFilter(QObject* target, QEvent* event)
 
     if (!expectedCanvasWidget || target != expectedCanvasWidget) return false;
 
-    if(event->type() == QEvent::MouseButtonPress || event->type() == QEvent::TabletPress) {
-        QMouseEvent *me = dynamic_cast<QMouseEvent*>(event);
-        QTabletEvent *te = dynamic_cast<QTabletEvent*>(event);
-        QPoint pos = me ? me->pos() : (te ? te->pos() : QPoint(77,77));
+    if (event->type() == QEvent::MouseButtonPress || event->type() == QEvent::TabletPress
+        || event->type() == QEvent::TouchBegin) {
+        const QScopedPointer<KoPointerEvent> pointerEvent(getKoPointerEvent(event));
+        const QPoint pos = !pointerEvent.isNull() ? pointerEvent->pos() : QPoint(77,77);
 
         if(d->config.mirrorHorizontal() && d->horizontalHandle.contains(pos) && !d->config.lockHorizontal() && !d->config.hideHorizontalDecoration() ) {
             d->xActive = true;
@@ -254,11 +281,10 @@ bool KisMirrorAxis::eventFilter(QObject* target, QEvent* event)
             return true;
         }
     }
-    if(event->type() == QEvent::MouseMove || event->type() == QEvent::TabletMove) {
-        QMouseEvent *me = dynamic_cast<QMouseEvent*>(event);
-        QTabletEvent *te = dynamic_cast<QTabletEvent*>(event);
-
-        QPoint pos = me ? me->pos() : (te ? te->pos() : QPoint(77,77));
+    if (event->type() == QEvent::MouseMove || event->type() == QEvent::TabletMove
+        || event->type() == QEvent::TouchUpdate) {
+        const QScopedPointer<KoPointerEvent> pointerEvent(getKoPointerEvent(event));
+        const QPoint pos = !pointerEvent.isNull() ? pointerEvent->pos() : QPoint(77,77);
 
         if(d->xActive) {
             float axisX = view()->viewConverter()->widgetToImage<QPoint>(pos).x();
@@ -316,16 +342,21 @@ bool KisMirrorAxis::eventFilter(QObject* target, QEvent* event)
             }
         }
     }
-    if(event->type() == QEvent::MouseButtonRelease || event->type() == QEvent::TabletRelease) {
+    if (event->type() == QEvent::MouseButtonRelease || event->type() == QEvent::TabletRelease
+        || event->type() == QEvent::TouchEnd || event->type() == QEvent::TouchCancel) {
 
         if(d->xActive) {
-            QApplication::restoreOverrideCursor();
+            while (QApplication::overrideCursor()) {
+                QApplication::restoreOverrideCursor();
+            }
             d->xActive = false;
             event->accept();
             return true;
         }
         if(d->yActive) {
-            QApplication::restoreOverrideCursor();
+            while (QApplication::overrideCursor()) {
+                QApplication::restoreOverrideCursor();
+            }
             d->yActive = false;
             event->accept();
             return true;
@@ -377,19 +408,18 @@ void KisMirrorAxis::setMirrorAxisConfig(const KisMirrorAxisConfig &config)
 
         d->config = config;
 
-        d->setAxisPosition(d->config.axisPosition().x(), d->config.axisPosition().y());
-
         d->resourceProvider->setMirrorHorizontal(d->config.mirrorHorizontal());
         d->resourceProvider->setMirrorVertical(d->config.mirrorVertical());
 
         d->resourceProvider->setMirrorHorizontalLock(d->config.lockHorizontal());
         d->resourceProvider->setMirrorVerticalLock(d->config.lockVertical());
 
-        d->resourceProvider->setMirrorHorizontal(d->config.mirrorHorizontal());
-        d->resourceProvider->setMirrorVertical(d->config.mirrorVertical());
-
         d->resourceProvider->setMirrorHorizontalHideDecorations(d->config.hideHorizontalDecoration());
         d->resourceProvider->setMirrorVerticalHideDecorations(d->config.hideVerticalDecoration());
+
+        if (view()) {
+            view()->canvasBase()->updateCanvas();
+        }
     }
 
     toggleMirrorActions();
@@ -452,9 +482,6 @@ void KisMirrorAxis::Private::setAxisPosition(float x, float y)
 
     config.setAxisPosition(newPosition);
 
-    const QPointF relativePosition = KisAlgebra2D::absoluteToRelative(newPosition, image->bounds());
-    image->setMirrorAxesCenter(relativePosition);
-
     q->view()->canvasBase()->updateCanvas();
 }
 
@@ -463,8 +490,6 @@ void KisMirrorAxis::Private::recomputeVisibleAxes(QRect viewport)
 {
     KisCoordinatesConverter *converter = q->view()->viewConverter();
 
-    config.setAxisPosition(KisAlgebra2D::relativeToAbsolute(image->mirrorAxesCenter(), image->bounds()));
-
     QPointF samplePt1 = converter->imageToWidget<QPointF>(QPointF(config.axisPosition().x(), 0));
     QPointF samplePt2 = converter->imageToWidget<QPointF>(QPointF(config.axisPosition().x(), 100));
 
