@@ -11,6 +11,7 @@
 
 #include "input/kis_abstract_input_action.h"
 #include "input/kis_input_profile_manager.h"
+#include "input/kis_shortcut_configuration.h"
 #include "kis_action_shortcuts_model.h"
 #include "kis_input_type_delegate.h"
 #include "kis_input_mode_delegate.h"
@@ -19,9 +20,11 @@
 
 #include <QAction>
 #include <QMessageBox>
+#include <QToolTip>
 
 KisInputConfigurationPageItem::KisInputConfigurationPageItem(QWidget *parent, Qt::WindowFlags f)
     : QWidget(parent, f)
+    , m_defaultToolTipText(i18n("Conflict found for a Shortcut in this Action."))
 {
     ui = new Ui::KisInputConfigurationPageItem;
     this->setContentsMargins(0,0,0,0);
@@ -35,6 +38,17 @@ KisInputConfigurationPageItem::KisInputConfigurationPageItem(QWidget *parent, Qt
     ui->shortcutsView->header()->setSectionResizeMode(QHeaderView::Stretch);
     setExpanded(false);
 
+    ui->warningConflictButton->setIcon(KisIconUtils::loadIcon("warning"));
+    ui->warningConflictButton->setFlat(true);
+    ui->warningConflictButton->hide();
+    connect(ui->warningConflictButton, &QPushButton::clicked, [&]() {
+        QToolTip::showText(QCursor::pos(), ui->warningConflictButton->toolTip());
+    });
+
+    connect(m_shortcutsModel, &KisActionShortcutsModel::dataChanged, this, [&]() {
+        emit inputConfigurationChanged();
+    });
+
     QAction *deleteAction = new QAction(KisIconUtils::loadIcon("edit-delete"), i18n("Delete Shortcut"), ui->shortcutsView);
     connect(deleteAction, SIGNAL(triggered(bool)), SLOT(deleteShortcut()));
     ui->shortcutsView->addAction(deleteAction);
@@ -58,6 +72,20 @@ void KisInputConfigurationPageItem::setAction(KisAbstractInputAction *action)
     qobject_cast<KisInputModeDelegate *>(ui->shortcutsView->itemDelegateForColumn(2))->setAction(action);
 }
 
+void KisInputConfigurationPageItem::setWarningEnabled(bool enabled, QString additionalToolTipText)
+{
+    ui->warningConflictButton->setVisible(enabled);
+    if (!enabled) {
+        ui->warningConflictButton->setToolTip(m_defaultToolTipText);
+    } else if (!additionalToolTipText.isEmpty()) {
+        const QString toolTipText =
+            "<html>" +
+            ui->warningConflictButton->toolTip() + "<br>" + additionalToolTipText +
+            "</html>";
+        ui->warningConflictButton->setToolTip(toolTipText);
+    }
+}
+
 void KisInputConfigurationPageItem::setExpanded(bool expand)
 {
     if (expand) {
@@ -78,6 +106,7 @@ void KisInputConfigurationPageItem::deleteShortcut()
 
     if (m_shortcutsModel->canRemoveRow(row)) {
         m_shortcutsModel->removeRow(row, QModelIndex());
+        emit inputConfigurationChanged();
     } else {
         QMessageBox shortcutMessage;
         shortcutMessage.setText(i18n("Deleting last shortcut for this action!"));
