@@ -132,12 +132,12 @@ void KisInputManager::attachPriorityEventFilter(QObject *filter, int priority)
     Private::PriorityList::iterator end = d->priorityEventFilter.end();
 
     it = std::find_if(begin, end,
-                      [filter] (const Private::PriorityPair &a) { return a.second == filter; });
+                      kismpl::mem_equal_to(&Private::PriorityPair::second, filter));
 
     if (it != end) return;
 
     it = std::find_if(begin, end,
-                      [priority] (const Private::PriorityPair &a) { return a.first > priority; });
+                      kismpl::mem_greater(&Private::PriorityPair::first, priority));
 
     d->priorityEventFilter.insert(it, qMakePair(priority, filter));
     d->priorityEventFilterSeqNo++;
@@ -149,7 +149,7 @@ void KisInputManager::detachPriorityEventFilter(QObject *filter)
     Private::PriorityList::iterator end = d->priorityEventFilter.end();
 
     it = std::find_if(it, end,
-                      [filter] (const Private::PriorityPair &a) { return a.second == filter; });
+                      kismpl::mem_equal_to(&Private::PriorityPair::second, filter));
 
     if (it != end) {
         d->priorityEventFilter.erase(it);
@@ -388,7 +388,7 @@ bool KisInputManager::eventFilterImpl(QEvent * event)
                 guessedKeys.removeOne(key);
             }
 
-            d->matcher.recoveryModifiersWithoutFocus(guessedKeys);
+            d->fixShortcutMatcherModifiersState(guessedKeys, modifiers);
             d->shouldSynchronizeOnNextKeyPress = false;
         }
 
@@ -398,7 +398,7 @@ bool KisInputManager::eventFilterImpl(QEvent * event)
             retval = d->matcher.autoRepeatedKeyPressed(key);
         }
 
-        // In case we matched ashortcut we should accept the event to
+        // In case we matched a shortcut we should accept the event to
         // notify Qt that it shouldn't try to trigger its partially matched
         // shortcuts.
         if (retval) {
@@ -558,18 +558,8 @@ bool KisInputManager::eventFilterImpl(QEvent * event)
         d->debugEvent<QEvent, false>(event);
         KisAbstractInputAction::setInputManager(this);
 
-        //Clear all state so we don't have half-matched shortcuts dangling around.
-        d->matcher.reinitialize();
-
-    { // Emulate pressing of the key that are already pressed
-        KisExtendedModifiersMapper mapper;
-
-        Qt::KeyboardModifiers modifiers = mapper.queryStandardModifiers();
-        Q_FOREACH (Qt::Key key, mapper.queryExtendedModifiers()) {
-            QKeyEvent kevent(QEvent::ShortcutOverride, key, modifiers);
-            eventFilterImpl(&kevent);
-        }
-    }
+        d->fixShortcutMatcherModifiersState();
+        d->matcher.reinitializeButtons();
 
         d->allowMouseEvents();
         break;
@@ -716,7 +706,7 @@ bool KisInputManager::eventFilterImpl(QEvent * event)
                 && !d->touchHasBlockedPressEvents
                 && touchEvent->touchPoints().count() == 1
                 && touchEvent->touchPointStates() != Qt::TouchPointStationary
-                && (qAbs(currentPos.x() - d->previousPos.x()) > 1		// stop wobbiliness which Qt sends us
+                && (qAbs(currentPos.x() - d->previousPos.x()) > 1		// stop wobbliness which Qt sends us
                 ||  qAbs(currentPos.y() - d->previousPos.y()) > 1)))
             {
                 d->previousPos = currentPos;
