@@ -309,6 +309,9 @@ void KisCanvasResourceProvider::slotCanvasResourceChanged(int key, const QVarian
     case(KoCanvasResource::CurrentKritaNode) :
         emit sigNodeChanged(currentNode());
         break;
+    case(KoCanvasResource::CurrentEffectiveCompositeOp) :
+        emit sigEffectiveCompositeOpChanged();
+        break;
     case (KoCanvasResource::Opacity):
     {
         emit sigOpacityChanged(res.toDouble());
@@ -339,7 +342,6 @@ void KisCanvasResourceProvider::setEraserMode(bool value)
 {
     m_resourceManager->setResource(KoCanvasResource::EraserMode,
                                    QVariant::fromValue(value));
-    emit sigEraserModeToggled(value);
 }
 
 void KisCanvasResourceProvider::slotPainting()
@@ -485,6 +487,16 @@ qreal KisCanvasResourceProvider::size() const
     return m_resourceManager->resource(KoCanvasResource::Size).toReal();
 }
 
+void KisCanvasResourceProvider::setBrushRotation(qreal rotation)
+{
+    m_resourceManager->setResource(KoCanvasResource::BrushRotation, rotation);
+}
+
+qreal KisCanvasResourceProvider::brushRotation() const
+{
+    return m_resourceManager->resource(KoCanvasResource::BrushRotation).toReal();
+}
+
 void KisCanvasResourceProvider::setPatternSize(qreal size)
 {
     m_resourceManager->setResource(KoCanvasResource::PatternSize, size);
