@@ -681,6 +681,14 @@ public:
     bool useLayerSelectionCheckbox(bool defaultValue = false) const;
     void setUseLayerSelectionCheckbox(bool value);
 
+    enum AssistantsDrawMode {
+        ASSISTANTS_DRAW_MODE_DIRECT = 0,             // no caching, draw directly on canvas
+        ASSISTANTS_DRAW_MODE_PIXMAP_CACHE = 1,
+        ASSISTANTS_DRAW_MODE_LARGE_PIXMAP_CACHE = 2,
+    };
+    AssistantsDrawMode assistantsDrawMode(bool defaultValue = false) const;
+    void setAssistantsDrawMode(AssistantsDrawMode value);
+
     template<class T>
     void writeEntry(const QString& name, const T& value) {
         m_cfg.writeEntry(name, value);
