@@ -21,6 +21,7 @@
 #include "kis_icon_utils.h"
 #include "KisViewManager.h"
 #include <KoCompositeOpRegistry.h>
+#include "kis_tool_proxy.h"
 
 #include <QPainter>
 #include <QPainterPath>
@@ -32,6 +33,7 @@ struct KisPaintingAssistantsDecoration::Private {
         , outlineVisible(false)
         , snapOnlyOneAssistant(true)
         , snapEraser(false)
+        , useCache(false)
         , firstAssistant(0)
         , aFirstStroke(false)
         , m_handleSize(14)
@@ -41,22 +43,27 @@ struct KisPaintingAssistantsDecoration::Private {
     bool outlineVisible;
     bool snapOnlyOneAssistant;
     bool snapEraser;
+    bool useCache;
     KisPaintingAssistantSP firstAssistant;
     KisPaintingAssistantSP selectedAssistant;
     bool aFirstStroke;
     bool m_isEditingAssistants = false;
-    bool m_outlineVisible = false;
     int m_handleSize; // size of editor handles on assistants
 
     // move, visibility, delete icons for each assistant. These only display while the assistant tool is active
-    // these icons will be covered by the kis_paintint_assistant_decoration with things like the perspective assistant
+    // these icons will be covered by the kis_painting_assistant_decoration with things like the perspective assistant
 
     AssistantEditorData toolData;
 
-    QPixmap m_iconDelete = KisIconUtils::loadIcon("dialog-cancel").pixmap(toolData.deleteIconSize, toolData.deleteIconSize);
+    QPixmap m_iconDelete = KisIconUtils::loadIcon("deletelayer").pixmap(toolData.deleteIconSize, toolData.deleteIconSize);
     QPixmap m_iconSnapOn = KisIconUtils::loadIcon("visible").pixmap(toolData.snapIconSize, toolData.snapIconSize);
     QPixmap m_iconSnapOff = KisIconUtils::loadIcon("novisible").pixmap(toolData.snapIconSize, toolData.snapIconSize);
     QPixmap m_iconMove = KisIconUtils::loadIcon("transform-move").pixmap(toolData.moveIconSize, toolData.moveIconSize);
+    QPixmap m_iconLockOn = KisIconUtils::loadIcon("layer-locked").pixmap(toolData.lockedIconSize, toolData.lockedIconSize);
+    QPixmap m_iconLockOff = KisIconUtils::loadIcon("layer-unlocked").pixmap(toolData.lockedIconSize, toolData.lockedIconSize);
+    QPixmap m_iconDragEditorWidget = KisIconUtils::loadIcon("gridbrush").pixmap(toolData.dragEditorWidgetIconSize, toolData.dragEditorWidgetIconSize);
+
+
 
     KisCanvas2 * m_canvas = 0;
 };
@@ -72,6 +79,8 @@ KisPaintingAssistantsDecoration::KisPaintingAssistantsDecoration(QPointer<KisVie
     setPriority(95);
     d->snapOnlyOneAssistant = true; //turn on by default.
     d->snapEraser = false;
+
+    slotConfigChanged(); // load the initial config
 }
 
 KisPaintingAssistantsDecoration::~KisPaintingAssistantsDecoration()
@@ -88,6 +97,16 @@ void KisPaintingAssistantsDecoration::slotUpdateDecorationVisibility()
     }
 }
 
+void KisPaintingAssistantsDecoration::slotConfigChanged()
+{
+    KisConfig cfg(true);
+    const KisConfig::AssistantsDrawMode drawMode = cfg.assistantsDrawMode();
+
+    d->useCache =
+        (drawMode == KisConfig::ASSISTANTS_DRAW_MODE_PIXMAP_CACHE) ||
+        (drawMode == KisConfig::ASSISTANTS_DRAW_MODE_LARGE_PIXMAP_CACHE);
+}
+
 void KisPaintingAssistantsDecoration::addAssistant(KisPaintingAssistantSP assistant)
 {
     QList<KisPaintingAssistantSP> assistants = view()->document()->assistants();
@@ -148,7 +167,7 @@ QPointF KisPaintingAssistantsDecoration::adjustPosition(const QPointF& point, co
 {
 
     if (assistants().empty()) {
-        // No assisants, so no adjustment
+        // No assistants, so no adjustment
         return point;
     }
 
@@ -158,12 +177,19 @@ QPointF KisPaintingAssistantsDecoration::adjustPosition(const QPointF& point, co
         return point;
     }
 
+    KisImageSP image = d->m_canvas->image();
+    KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(image, point);
+
+    const KisCoordinatesConverter *converter = d->m_canvas->coordinatesConverter();
+    const qreal moveThresholdPt = 2.0 * converter->effectiveZoom() / qMax(image->xRes(), image->yRes());
+
+
     // There is at least 1 assistant
     if (assistants().count() == 1) {
         // Things are easy when there is only one assistant
         if(assistants().first()->isSnappingActive() == true){
             bool snapSingle = d->snapOnlyOneAssistant && d->aFirstStroke;
-            QPointF newpoint = assistants().first()->adjustPosition(point, strokeBegin, !snapSingle);
+            QPointF newpoint = assistants().first()->adjustPosition(point, strokeBegin, !snapSingle, moveThresholdPt);
             // check for NaN
             if (newpoint.x() != newpoint.x()) return point;
             // Tell the assistant that its guidelines should
@@ -181,7 +207,7 @@ QPointF KisPaintingAssistantsDecoration::adjustPosition(const QPointF& point, co
             }
             return newpoint;
         } else {
-            // One assisant, but it is not active, so no adjustment
+            // One assistant, but it is not active, so no adjustment
             return point;
         }
     }
@@ -197,7 +223,7 @@ QPointF KisPaintingAssistantsDecoration::adjustPosition(const QPointF& point, co
         // In this mode the best assistant is constantly computed anew.
         Q_FOREACH (KisPaintingAssistantSP assistant, assistants()) {
             if (assistant->isSnappingActive() == true){//this checks if the assistant in question has it's snapping boolean turned on//
-                QPointF pt = assistant->adjustPosition(point, strokeBegin, true);
+                QPointF pt = assistant->adjustPosition(point, strokeBegin, true, moveThresholdPt);
                 // check for NaN
                 if (pt.x() != pt.x()) continue;
                 double dist = qAbs(pt.x() - point.x()) + qAbs(pt.y() - point.y());
@@ -215,7 +241,7 @@ QPointF KisPaintingAssistantsDecoration::adjustPosition(const QPointF& point, co
         bool foundAGoodAssistant = false;
         Q_FOREACH (KisPaintingAssistantSP assistant, assistants()) {
             if(assistant->isSnappingActive() == true){//this checks if the assistant in question has it's snapping boolean turned on//
-                QPointF pt = assistant->adjustPosition(point, strokeBegin, true);
+                QPointF pt = assistant->adjustPosition(point, strokeBegin, true, moveThresholdPt);
                 if (pt.x() != pt.x()) continue;
                 double dist = qAbs(pt.x() - point.x()) + qAbs(pt.y() - point.y());
                 if (dist < distance) {
@@ -235,7 +261,7 @@ QPointF KisPaintingAssistantsDecoration::adjustPosition(const QPointF& point, co
         }
     } else if (d->firstAssistant) {
         //make sure there's a first assistant to begin with.//
-        QPointF newpoint = d->firstAssistant->adjustPosition(point, strokeBegin, false);
+        QPointF newpoint = d->firstAssistant->adjustPosition(point, strokeBegin, false, moveThresholdPt);
         // BUGFIX: 402535
         // assistants might return (NaN,NaN), must always check for that
         if (newpoint.x() == newpoint.x()) {
@@ -257,6 +283,56 @@ QPointF KisPaintingAssistantsDecoration::adjustPosition(const QPointF& point, co
     return best;
 }
 
+void KisPaintingAssistantsDecoration::adjustLine(QPointF &point, QPointF &strokeBegin)
+{
+    if (assistants().empty()) {
+        // No assistants, so no adjustment
+        return;
+    }
+
+    // TODO: figure it out
+    if  (!d->snapEraser
+        && (d->m_canvas->resourceManager()->resource(KoCanvasResource::CurrentEffectiveCompositeOp).toString() == COMPOSITE_ERASE)) {
+        // No snapping if eraser snapping is disabled and brush is an eraser
+        return;
+    }
+
+    QPointF originalPoint = point;
+    QPointF originalStrokeBegin = strokeBegin;
+
+    qreal minDistance = 10000.0;
+    bool minDistValid = false;
+    QPointF finalPoint = originalPoint;
+    QPointF finalStrokeBegin = originalStrokeBegin;
+    int id = 0;
+    KisPaintingAssistantSP bestAssistant;
+    Q_FOREACH (KisPaintingAssistantSP assistant, assistants()) {
+        if(assistant->isSnappingActive() == true){//this checks if the assistant in question has it's snapping boolean turned on//
+            //QPointF pt = assistant->adjustPosition(point, strokeBegin, true);
+            QPointF p1 = originalPoint;
+            QPointF p2 = originalStrokeBegin;
+            assistant->adjustLine(p1, p2);
+            if (p1.isNull() || p2.isNull()) {
+                // possibly lines cannot snap to this assistant, or this line cannot, at least
+                continue;
+            }
+            qreal distance = kisSquareDistance(p1, originalPoint) + kisSquareDistance(p2, originalStrokeBegin);
+            if (distance < minDistance || !minDistValid) {
+                finalPoint = p1;
+                finalStrokeBegin = p2;
+                minDistValid = true;
+                bestAssistant = assistant;
+            }
+        }
+        id ++;
+    }
+    if (bestAssistant) {
+        bestAssistant->setFollowBrushPosition(true);
+    }
+    point = finalPoint;
+    strokeBegin = finalStrokeBegin;
+}
+
 void KisPaintingAssistantsDecoration::endStroke()
 {
     d->aFirstStroke = false;
@@ -266,7 +342,7 @@ void KisPaintingAssistantsDecoration::endStroke()
     }
 }
 
-void KisPaintingAssistantsDecoration::drawDecoration(QPainter& gc, const QRectF& updateRect, const KisCoordinatesConverter *converter,KisCanvas2* canvas)
+void KisPaintingAssistantsDecoration::drawDecoration(QPainter& gc, const QRectF& updateRect, const KisCoordinatesConverter *converter, KisCanvas2* canvas)
 {
     if(assistants().isEmpty()) {
         return; // no assistants to worry about, ok to exit
@@ -279,15 +355,19 @@ void KisPaintingAssistantsDecoration::drawDecoration(QPainter& gc, const QRectF&
     }
 
     // the preview functionality for assistants. do not show while editing
-    if (d->m_isEditingAssistants) {
-        d->m_outlineVisible = false;
-    }
-    else {
-        d->m_outlineVisible = outlineVisibility();
-    }
+
+    KoToolProxy *proxy = view()->canvasBase()->toolProxy();
+    KIS_SAFE_ASSERT_RECOVER_RETURN(proxy);
+    KisToolProxy *kritaProxy = dynamic_cast<KisToolProxy*>(proxy);
+    KIS_SAFE_ASSERT_RECOVER_RETURN(kritaProxy);
+
+    const bool outlineVisible =
+        outlineVisibility() &&
+        !d->m_isEditingAssistants &&
+        kritaProxy->supportsPaintingAssistants();
 
     Q_FOREACH (KisPaintingAssistantSP assistant, assistants()) {
-        assistant->drawAssistant(gc, updateRect, converter, true, canvas, assistantVisibility(), d->m_outlineVisible);
+        assistant->drawAssistant(gc, updateRect, converter, d->useCache, canvas, assistantVisibility(), outlineVisible);
 
         if (isEditingAssistants()) {
             drawHandles(assistant, gc, converter);
@@ -532,7 +612,9 @@ void KisPaintingAssistantsDecoration::drawEditorWidget(KisPaintingAssistantSP as
 
     QPointF iconMovePosition(actionsPosition + toolData.moveIconPosition);
     QPointF iconSnapPosition(actionsPosition + toolData.snapIconPosition);
+    QPointF iconLockedPosition(actionsPosition + toolData.lockedIconPosition);
     QPointF iconDeletePosition(actionsPosition + toolData.deleteIconPosition);
+    QPointF iconDragEditorWidgetPosition(actionsPosition + toolData.dragEditorWidgetIconPosition);
 
     // Background container for helpers
     QBrush backgroundColor = d->m_canvas->viewManager()->mainWindowAsQWidget()->palette().window();
@@ -570,7 +652,18 @@ void KisPaintingAssistantsDecoration::drawEditorWidget(KisPaintingAssistantSP as
         gc.drawPixmap(iconSnapPosition, d->m_iconSnapOff);
     }
 
+    if (assistant->isLocked()) {
+        gc.drawPixmap(iconLockedPosition, d->m_iconLockOn);
+    } else {
+        qreal oldOpacity = gc.opacity();
+        gc.setOpacity(0.35);
+        gc.drawPixmap(iconLockedPosition, d->m_iconLockOff);
+        gc.setOpacity(oldOpacity);
+    }
+
+
     gc.drawPixmap(iconDeletePosition, d->m_iconDelete);
+    gc.drawPixmap(iconDragEditorWidgetPosition, d->m_iconDragEditorWidget);
 
 
 }
