@@ -13,11 +13,12 @@
 KisPaintOpOptionListModel::KisPaintOpOptionListModel(QObject *parent)
     : BaseOptionCategorizedListModel(parent)
 {
+    connect(&m_stateSignalsMapper, qOverload<int>(&QSignalMapper::mapped),
+            this, &KisPaintOpOptionListModel::slotCheckedEnabledStateChanged);
 }
 
-void KisPaintOpOptionListModel::addPaintOpOption(KisPaintOpOption *option, int widgetIndex, const QString &label, KisPaintOpOption::PaintopCategory categoryType)
+QString KisPaintOpOptionListModel::categoryName(KisPaintOpOption::PaintopCategory categoryType)
 {
-
     QString category;
     switch(categoryType) {
     case KisPaintOpOption::GENERAL:
@@ -36,8 +37,7 @@ void KisPaintOpOptionListModel::addPaintOpOption(KisPaintOpOption *option, int w
         category = i18nc("option category", "Masked Brush");
         break;
     };
-
-    addPaintOpOption(option, widgetIndex, label, category);
+    return category;
 }
 
 void KisPaintOpOptionListModel::addPaintOpOption(KisPaintOpOption *option, int widgetIndex, const QString &label, const QString &category) {
@@ -47,29 +47,20 @@ void KisPaintOpOptionListModel::addPaintOpOption(KisPaintOpOption *option, int w
     if (option->isCheckable()) {
         item->setCheckable(true);
         item->setChecked(option->isChecked());
+        connect(option, &KisPaintOpOption::sigCheckedChanged,
+                &m_stateSignalsMapper, qOverload<>(&QSignalMapper::map));
     }
 
+    item->setEnabled(option->isEnabled());
+    connect(option, &KisPaintOpOption::sigEnabledChanged,
+            &m_stateSignalsMapper, qOverload<>(&QSignalMapper::map));
+    m_stateSignalsMapper.setMapping(option, categoriesMapper()->rowFromItem(item));
+
     categoriesMapper()->expandAllCategories();
 }
 
 QVariant KisPaintOpOptionListModel::data(const QModelIndex& idx, int role) const
 {
-    if (!idx.isValid()) return false;
-
-    DataItem *item = categoriesMapper()->itemFromRow(idx.row());
-    Q_ASSERT(item);
-
-    // Lazy fetching of the real checked value (there are no notifications
-    // when changing the pointop preset)
-
-    if (role == Qt::CheckStateRole && item->isCheckable()) {
-        bool realChecked = item->data()->option->isChecked();
-
-        if (realChecked != item->isChecked()) {
-            item->setChecked(realChecked);
-        }
-    }
-
     return BaseOptionCategorizedListModel::data(idx, role);
 }
 
@@ -100,3 +91,21 @@ void KisPaintOpOptionListModel::signalDataChanged(const QModelIndex& index)
 {
     emit dataChanged(index,index);
 }
+
+void KisPaintOpOptionListModel::slotCheckedEnabledStateChanged(int row)
+{
+    QModelIndex idx(index(row));
+
+    DataItem *item = categoriesMapper()->itemFromRow(row);
+    KIS_SAFE_ASSERT_RECOVER_RETURN(item);
+
+    if (item->data()->option->isEnabled() != item->isEnabled()) {
+        item->setEnabled(item->data()->option->isEnabled());
+    }
+
+    if (item->data()->option->isChecked() != item->isChecked()) {
+        item->setChecked(item->data()->option->isChecked());
+    }
+
+    emit dataChanged(idx, idx);
+}
