@@ -92,6 +92,7 @@ void KisZoomManager::updateScreenResolution(QWidget *parentWidget)
 
     KisCoordinatesConverter *converter =
         dynamic_cast<KisCoordinatesConverter*>(m_zoomHandler);
+    KIS_ASSERT_RECOVER_RETURN(converter);
 
     converter->setDevicePixelRatio(m_devicePixelRatio);
 
@@ -332,9 +333,7 @@ void KisZoomManager::setMinMaxZoom()
     qreal minDimension = qMin(imageSize.width(), imageSize.height());
     qreal minZoom = qMin(100.0 / minDimension, 0.1);
 
-    m_zoomAction->setMinimumZoom(minZoom);
-    m_zoomAction->setMaximumZoom(90.0);
-
+    m_zoomAction->setMinMaxZoom(minZoom, 90.0);
 }
 
 void KisZoomManager::updateGuiAfterDocumentSize()
@@ -387,6 +386,8 @@ void KisZoomManager::changeCanvasMappingMode(bool canvasMappingMode)
     m_canvasMappingMode = canvasMappingMode;
     m_zoomController->setZoom(newMode, newZoom, resolutionX(), resolutionY());
     m_view->canvasBase()->notifyZoomChanged();
+
+    m_view->viewManager()->updatePrintSizeAction(canvasMappingMode);
 }
 
 void KisZoomManager::pageOffsetChanged()
@@ -404,6 +405,23 @@ void KisZoomManager::zoomTo100()
     m_view->canvasBase()->notifyZoomChanged();
 }
 
+void KisZoomManager::slotZoomToFit()
+{
+    m_zoomController->setZoom(KoZoomMode::ZOOM_PAGE, 0);
+    m_view->canvasBase()->notifyZoomChanged();
+}
+
+void KisZoomManager::slotZoomToFitWidth()
+{
+    m_zoomController->setZoom(KoZoomMode::ZOOM_WIDTH, 0);
+    m_view->canvasBase()->notifyZoomChanged();
+}
+void KisZoomManager::slotZoomToFitHeight()
+{
+    m_zoomController->setZoom(KoZoomMode::ZOOM_HEIGHT, 0);
+    m_view->canvasBase()->notifyZoomChanged();
+}
+
 void KisZoomManager::slotToggleZoomToFit()
 {
     KoZoomMode::Mode currentZoomMode = m_zoomController->zoomMode();
