@@ -348,11 +348,10 @@ void KisOpenGLImageTextures::recalculateCache(KisUpdateInfoSP info, bool blockMi
             qDebug() << "Still unsignalled after processed" << numProcessedTiles << "tiles";
 #endif
 
-            const uint nextSize = qNextPowerOfTwo(m_bufferStorage.size());
-            m_bufferStorage.allocateMoreBuffers(nextSize);
+            m_bufferStorage.allocateMoreBuffers();
 
 #ifdef DEBUG_BUFFER_REALLOCATION
-            qDebug() << "    increased number of buffers to" << nextSize;
+            qDebug() << "    increased number of buffers to" << m_bufferStorage.size();
 #endif
         }
 
@@ -415,6 +414,10 @@ void KisOpenGLImageTextures::generateCheckerTexture(const QImage &checkImage)
 
         f->glTexImage2D(GL_TEXTURE_2D, 0, internalFormat, BACKGROUND_TEXTURE_SIZE, BACKGROUND_TEXTURE_SIZE,
                         0, format, type, checkers.data());
+
+        // QPainter::drawText relies on this.
+        // Ref: https://bugreports.qt.io/browse/QTBUG-65496
+        f->glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
     }
     else {
         dbgUI << "OpenGL: Tried to generate checker texture before OpenGL was initialized.";
@@ -505,7 +508,7 @@ void KisOpenGLImageTextures::setChannelFlags(const QBitArray &channelFlags)
     QBitArray targetChannelFlags = channelFlags;
     int selectedChannels = 0;
     const KoColorSpace *projectionCs = m_image->projection()->colorSpace();
-    QList<KoChannelInfo*> channelInfo = projectionCs->channels();
+    const QList<KoChannelInfo*> channelInfo = projectionCs->channels();
 
     if (targetChannelFlags.size() != channelInfo.size()) {
         targetChannelFlags = QBitArray();
@@ -577,7 +580,7 @@ bool KisOpenGLImageTextures::setInternalColorManagementActive(bool value)
         recreateImageTextureTiles();
 
         // at this point the value of m_internalColorManagementActive might
-        // have been forcely reverted to 'false' in case of some problems
+        // have been forcefully reverted to 'false' in case of some problems
     }
 
     return needsFinalRegeneration;
@@ -783,6 +786,7 @@ void KisOpenGLImageTextures::updateTextureFormat()
                 if (ctx->hasExtension("GL_EXT_texture_norm16")) {
                     m_texturesInfo.internalFormat = GL_RGBA16_EXT;
                     m_texturesInfo.type = GL_UNSIGNED_SHORT;
+                    m_texturesInfo.format = GL_RGBA;
                     destinationColorDepthId = Integer16BitsColorDepthID;
                     dbgUI << "Using 16 bits rgba (GLES v2)";
                 }
@@ -822,6 +826,7 @@ void KisOpenGLImageTextures::updateTextureFormat()
                 if (ctx->hasExtension("GL_EXT_texture_norm16")) {
                     m_texturesInfo.internalFormat = GL_RGBA16_EXT;
                     m_texturesInfo.type = GL_UNSIGNED_SHORT;
+                    m_texturesInfo.format = GL_RGBA;
                     destinationColorDepthId = Integer16BitsColorDepthID;
                     dbgUI << "Using conversion to 16 bits rgba (GLES v2)";
                 }
