@@ -21,28 +21,20 @@ class QOpenGLBuffer;
 
 struct KisGLTexturesInfo {
 
-    KisGLTexturesInfo()
-        : width(0)
-        , height(0)
-        , effectiveWidth(1)
-        , effectiveHeight(1)
-        , border(0)
-    {}
-
     // real width and height
-    int width;
-    int height;
+    int width {0};
+    int height {0};
 
     // width and height minus border padding?
-    int effectiveWidth;
-    int effectiveHeight;
+    int effectiveWidth {1};
+    int effectiveHeight {1};
 
     // size of the border padding
-    int border;
+    int border {0};
 
-    GLint internalFormat;
-    GLint format;
-    GLint type;
+    GLint internalFormat {0};
+    GLint format {0};
+    GLint type {0};
 };
 
 class KisTextureTile
@@ -78,7 +70,7 @@ public:
     QRectF imageRectInTexturePixels(const QRect &imageRect) const;
 
     /**
-     * Binds the tile's testure to the current GL_TEXTURE_2D binding point,
+     * Binds the tile's texture to the current GL_TEXTURE_2D binding point,
      * regenerates the mipmap if needed and returns the levelOfDetail that
      * should be used for painting
      */
@@ -86,6 +78,7 @@ public:
 
 private:
     inline void setTextureParameters();
+    inline void restoreTextureParameters();
 
     void setNeedsMipmapRegeneration();
     void setPreparedLodPlane(int lod);
