@@ -1,24 +1,12 @@
 /*
- *  Copyright (c) 2015 Dmitry Kazakov <dimula73@gmail.com>
+ *  SPDX-FileCopyrightText: 2015 Dmitry Kazakov <dimula73@gmail.com>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "kis_file_layer_test.h"
 
-#include <QTest>
+#include <simpletest.h>
 
 #include <KoColor.h>
 
@@ -27,6 +15,7 @@
 #include <kis_transform_mask_params_interface.h>
 
 #include <testutil.h>
+#include <testui.h>
 
 #include "config-limit-long-tests.h"
 
@@ -48,7 +37,7 @@ void KisFileLayerTest::testFileLayerPlusTransformMaskOffImage()
     TestUtil::MaskParent p(refRect);
 
     QString refName(TestUtil::fetchDataFileLazy("hakonepa.png"));
-    KisLayerSP flayer = new KisFileLayer(p.image, "", refName, KisFileLayer::None, "flayer", OPACITY_OPAQUE_U8);
+    KisLayerSP flayer = new KisFileLayer(p.image, "", refName, KisFileLayer::None, "Bicubic", "flayer", OPACITY_OPAQUE_U8);
 
     p.image->addNode(flayer, p.image->root(), KisNodeSP());
 
@@ -110,7 +99,7 @@ void KisFileLayerTest::testFileLayerPlusTransformMaskSmallFileBigOffset()
     TestUtil::MaskParent p(refRect);
 
     QString refName(TestUtil::fetchDataFileLazy("file_layer_source.png"));
-    KisLayerSP flayer = new KisFileLayer(p.image, "", refName, KisFileLayer::None, "flayer", OPACITY_OPAQUE_U8);
+    KisLayerSP flayer = new KisFileLayer(p.image, "", refName, KisFileLayer::None, "Bicubic", "flayer", OPACITY_OPAQUE_U8);
 
     p.image->addNode(flayer, p.image->root(), KisNodeSP());
 
