@@ -1,28 +1,16 @@
 /*
- *  Copyright (c) 2012 Dmitry Kazakov <dimula73@gmail.com>
+ *  SPDX-FileCopyrightText: 2012 Dmitry Kazakov <dimula73@gmail.com>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "kis_zoom_and_pan_test.h"
 
 #include <cmath>
-#include <QTest>
+#include <simpletest.h>
 
 #include <kis_filter_configuration.h>
-#include "testutil.h"
+#include <testutil.h>
 #include "qimage_based_test.h"
 
 #include <kactioncollection.h>
@@ -40,7 +28,7 @@
 #include "kis_coordinates_converter.h"
 #include "kis_filter_strategy.h"
 
-#include "kistest.h"
+#include "testui.h"
 
 class ZoomAndPanTester : public TestUtil::QImageBasedTest
 {
@@ -674,7 +662,7 @@ void KisZoomAndPanTest::testRotation(qreal vastScrolling, qreal zoom)
          * vastScrolling value is 0.2. The problem is that the centering
          * correction applied  to the offset in
          * KisCanvasController::rotateCanvas pollutes the preferredCenter
-         * value, because KoCnvasControllerWidget has no access to this
+         * value, because KoCanvasControllerWidget has no access to this
          * correction and cannot calculate the real value of the center of
          * the image. To fix this bug the calculation of correction
          * (aka "origin") should be moved to the KoCanvasControllerWidget
