@@ -44,8 +44,7 @@ KisPaintingInformationBuilder::~KisPaintingInformationBuilder()
 void KisPaintingInformationBuilder::updateSettings()
 {
     KisConfig cfg(true);
-    KisCubicCurve curve;
-    curve.fromString(cfg.pressureTabletCurve());
+    const KisCubicCurve curve(cfg.pressureTabletCurve());
     m_pressureSamples = curve.floatTransfer(LEVEL_OF_PRESSURE_RESOLUTION + 1);
     m_maxAllowedSpeedValue = cfg.readEntry("maxAllowedSpeedValue", 30);
     m_speedSmoother->updateSettings();
@@ -242,17 +241,20 @@ qreal KisToolFreehandPaintingInformationBuilder::calculatePerspective(const QPoi
 qreal KisToolFreehandPaintingInformationBuilder::canvasRotation() const
 {
     KisCanvas2 *canvas = dynamic_cast<KisCanvas2*>(m_tool->canvas());
+    KIS_ASSERT_RECOVER_RETURN_VALUE(canvas, 0.0);
     return canvas->coordinatesConverter()->rotationAngle();
 }
 
 bool KisToolFreehandPaintingInformationBuilder::canvasMirroredX() const
 {
     KisCanvas2 *canvas = dynamic_cast<KisCanvas2*>(m_tool->canvas());
+    KIS_ASSERT_RECOVER_RETURN_VALUE(canvas, false);
     return canvas->coordinatesConverter()->xAxisMirrored();
 }
 
 bool KisToolFreehandPaintingInformationBuilder::canvasMirroredY() const
 {
     KisCanvas2 *canvas = dynamic_cast<KisCanvas2*>(m_tool->canvas());
+    KIS_ASSERT_RECOVER_RETURN_VALUE(canvas, false);
     return canvas->coordinatesConverter()->yAxisMirrored();
 }
