@@ -40,6 +40,10 @@ void KisSelectionToolConfigWidgetHelper::createOptionWidget(
             &KisSelectionOptions::growSelectionChanged,
             this,
             &KisSelectionToolConfigWidgetHelper::slotWidgetGrowChanged);
+    connect(m_optionsWidget,
+            &KisSelectionOptions::stopGrowingAtDarkestPixelChanged,
+            this,
+            &KisSelectionToolConfigWidgetHelper::slotWidgetStopGrowingAtDarkestPixelChanged);
     connect(m_optionsWidget,
             &KisSelectionOptions::featherSelectionChanged,
             this,
@@ -89,6 +93,14 @@ int KisSelectionToolConfigWidgetHelper::growSelection() const
     return m_optionsWidget->growSelection();
 }
 
+bool KisSelectionToolConfigWidgetHelper::stopGrowingAtDarkestPixel() const
+{
+    if (!m_optionsWidget) {
+        return false;
+    }
+    return m_optionsWidget->stopGrowingAtDarkestPixel();
+}
+
 int KisSelectionToolConfigWidgetHelper::featherSelection() const
 {
     if (!m_optionsWidget) {
@@ -148,6 +160,12 @@ void KisSelectionToolConfigWidgetHelper::slotWidgetGrowChanged(int value)
     cfg.writeEntry("growSelection", value);
 }
 
+void KisSelectionToolConfigWidgetHelper::slotWidgetStopGrowingAtDarkestPixelChanged(bool value)
+{
+    KConfigGroup cfg = KSharedConfig::openConfig()->group(m_configGroupForTool);
+    cfg.writeEntry("stopGrowingAtDarkestPixel", value);
+}
+
 void KisSelectionToolConfigWidgetHelper::slotWidgetFeatherChanged(int value)
 {
     KConfigGroup cfg = KSharedConfig::openConfig()->group(m_configGroupForTool);
@@ -245,13 +263,22 @@ void KisSelectionToolConfigWidgetHelper::reloadExactToolConfig()
     const bool antiAliasSelection =
         cfgToolSpecific.readEntry("antiAliasSelection", true);
     const int growSelection = cfgToolSpecific.readEntry("growSelection", 0);
+    const bool stopGrowingAtDarkestPixel =
+        cfgToolSpecific.readEntry("stopGrowingAtDarkestPixel", false);
     const int featherSelection =
         cfgToolSpecific.readEntry("featherSelection", 0);
     const QString referenceLayersStr =
         cfgToolSpecific.readEntry("sampleLayersMode", "sampleCurrentLayer");
+
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
+    const QStringList colorLabelsStr =
+        cfgToolSpecific.readEntry<QString>("colorLabels", "")
+            .split(',', Qt::SkipEmptyParts);
+#else
     const QStringList colorLabelsStr =
         cfgToolSpecific.readEntry<QString>("colorLabels", "")
             .split(',', QString::SkipEmptyParts);
+#endif
 
     const KisSelectionOptions::ReferenceLayers referenceLayers =
         referenceLayersStr == "sampleAllLayers"
@@ -271,6 +298,7 @@ void KisSelectionToolConfigWidgetHelper::reloadExactToolConfig()
     KisSignalsBlocker b(m_optionsWidget);
     m_optionsWidget->setAntiAliasSelection(antiAliasSelection);
     m_optionsWidget->setGrowSelection(growSelection);
+    m_optionsWidget->setStopGrowingAtDarkestPixel(stopGrowingAtDarkestPixel);
     m_optionsWidget->setFeatherSelection(featherSelection);
     m_optionsWidget->setReferenceLayers(referenceLayers);
     m_optionsWidget->setSelectedColorLabels(colorLabels);
