@@ -33,7 +33,7 @@
 
 #include "strokes/freehand_stroke.h"
 #include "strokes/KisFreehandStrokeInfo.h"
-#include "KisAsyncronousStrokeUpdateHelper.h"
+#include "KisAsynchronousStrokeUpdateHelper.h"
 #include "kis_canvas_resource_provider.h"
 #include <KisOptimizedBrushOutline.h>
 
@@ -59,7 +59,7 @@ struct KisToolFreehandHelper::Private
     KoCanvasResourceProvider *resourceManager;
     KisPaintingInformationBuilder *infoBuilder;
     KisStrokesFacade *strokesFacade;
-    KisAsyncronousStrokeUpdateHelper asyncUpdateHelper;
+    KisAsynchronousStrokeUpdateHelper asyncUpdateHelper;
 
     KUndo2MagicString transactionText;
 
@@ -466,7 +466,7 @@ void KisToolFreehandHelper::paint(KisPaintInformation &info)
      * 3) 'Tail Aggressiveness' is used for controlling the end of the
      *    stroke
      *
-     * 4) The formila is a little bit different: 'Distance' parameter
+     * 4) The formula is a little bit different: 'Distance' parameter
      *    stands for $3 \Sigma$
      */
     if (m_d->smoothingOptions->smoothingType() == KisSmoothingOptions::WEIGHTED_SMOOTHING
@@ -514,10 +514,10 @@ void KisToolFreehandHelper::paint(KisPaintInformation &info)
                 if (i < m_d->history.size() - 1) {
                     pressureGrad = nextInfo.pressure() - m_d->history.at(i + 1).pressure();
 
-                    const qreal tailAgressiveness = 40.0 * m_d->smoothingOptions->tailAggressiveness();
+                    const qreal tailAggressiveness = 40.0 * m_d->smoothingOptions->tailAggressiveness();
 
                     if (pressureGrad > 0.0 ) {
-                        pressureGrad *= tailAgressiveness * (1.0 - nextInfo.pressure());
+                        pressureGrad *= tailAggressiveness * (1.0 - nextInfo.pressure());
                         distance += pressureGrad * 3.0 * sigma; // (3 * sigma) --- holds > 90% of the region
                     }
                 }
@@ -640,6 +640,10 @@ void KisToolFreehandHelper::endPaint()
      */
     m_d->strokeInfos.clear();
 
+    // last update to complete rendering if there is still something pending
+    m_d->strokesFacade->addJob(m_d->strokeId,
+       new KisAsynchronousStrokeUpdateHelper::UpdateData(true));
+
     m_d->strokesFacade->endStroke(m_d->strokeId);
     m_d->strokeId.clear();
     m_d->infoBuilder->reset();
@@ -888,6 +892,9 @@ void KisToolFreehandHelper::doAirbrushing()
                                       prevPaint.perspective(),
                                       elapsedStrokeTime(),
                                       0.0);
+        nextPaint.setCanvasRotation(prevPaint.canvasRotation());
+        nextPaint.setCanvasMirroredH(prevPaint.canvasMirroredH());
+        nextPaint.setCanvasMirroredV(prevPaint.canvasMirroredV());
         paint(nextPaint);
     }
 }
