@@ -199,6 +199,11 @@ void KisToolPaint::activate(const QSet<KoShape*> &shapes)
 
     }
 
+    connect(action("rotate_brush_tip_clockwise"), SIGNAL(triggered()), SLOT(rotateBrushTipClockwise()), Qt::UniqueConnection);
+    connect(action("rotate_brush_tip_clockwise_precise"), SIGNAL(triggered()), SLOT(rotateBrushTipClockwisePrecise()), Qt::UniqueConnection);
+    connect(action("rotate_brush_tip_counter_clockwise"), SIGNAL(triggered()), SLOT(rotateBrushTipCounterClockwise()), Qt::UniqueConnection);
+    connect(action("rotate_brush_tip_counter_clockwise_precise"), SIGNAL(triggered()), SLOT(rotateBrushTipCounterClockwisePrecise()), Qt::UniqueConnection);
+
     tryRestoreOpacitySnapshot();
 }
 
@@ -209,6 +214,11 @@ void KisToolPaint::deactivate()
         disconnect(action("decrease_brush_size"), 0, this, 0);
     }
 
+    disconnect(action("rotate_brush_tip_clockwise"), 0, this, 0);
+    disconnect(action("rotate_brush_tip_clockwise_precise"), 0, this, 0);
+    disconnect(action("rotate_brush_tip_counter_clockwise"), 0, this, 0);
+    disconnect(action("rotate_brush_tip_counter_clockwise_precise"), 0, this, 0);
+
     tryRestoreOpacitySnapshot();
     emit statusTextChanged(QString());
 
@@ -565,7 +575,7 @@ void KisToolPaint::decreaseBrushSize()
     std::vector<int>::reverse_iterator result =
         std::upper_bound(m_standardBrushSizes.rbegin(),
                          m_standardBrushSizes.rend(),
-                         (int)paintopSize,
+                         qRound(paintopSize),
                          std::greater<int>());
 
     int newValue = result != m_standardBrushSizes.rend() ? *result : m_standardBrushSizes.front();
@@ -582,6 +592,34 @@ void KisToolPaint::showBrushSize()
                                                    , QIcon(), 1000, KisFloatingMessage::High,  Qt::AlignLeft | Qt::TextWordWrap | Qt::AlignVCenter);
 }
 
+void KisToolPaint::rotateBrushTipClockwise()
+{
+    const qreal angle = currentPaintOpPreset()->settings()->paintOpAngle();
+    currentPaintOpPreset()->settings()->setPaintOpAngle(angle - 15);
+    requestUpdateOutline(m_outlineDocPoint, 0);
+}
+
+void KisToolPaint::rotateBrushTipClockwisePrecise()
+{
+    const qreal angle = currentPaintOpPreset()->settings()->paintOpAngle();
+    currentPaintOpPreset()->settings()->setPaintOpAngle(angle - 1);
+    requestUpdateOutline(m_outlineDocPoint, 0);
+}
+
+void KisToolPaint::rotateBrushTipCounterClockwise()
+{
+    const qreal angle = currentPaintOpPreset()->settings()->paintOpAngle();
+    currentPaintOpPreset()->settings()->setPaintOpAngle(angle + 15);
+    requestUpdateOutline(m_outlineDocPoint, 0);
+}
+
+void KisToolPaint::rotateBrushTipCounterClockwisePrecise()
+{
+    const qreal angle = currentPaintOpPreset()->settings()->paintOpAngle();
+    currentPaintOpPreset()->settings()->setPaintOpAngle(angle + 1);
+    requestUpdateOutline(m_outlineDocPoint, 0);
+}
+
 void KisToolPaint::requestUpdateOutline(const QPointF &outlineDocPoint, const KoPointerEvent *event)
 {
     QRectF outlinePixelRect;
