@@ -29,6 +29,7 @@
 #include "kis_scalar_keyframe_channel.h"
 #include "kis_image_animation_interface.h"
 #include "commands_new/KisSimpleModifyTransformMaskCommand.h"
+#include "KisAnimAutoKey.h"
 
 /* MoveNodeStrategyBase and descendants
  *
@@ -206,7 +207,7 @@ template <typename Functor>
 void MoveStrokeStrategy::recursiveApplyNodes(KisNodeList nodes, Functor &&func) {
     Q_FOREACH(KisNodeSP subtree, nodes) {
         KisLayerUtils::recursiveApplyNodes(subtree,
-            [this, func = std::forward<Functor>(func)] (KisNodeSP node) {
+            [&] (KisNodeSP node) {
                 if (!m_blacklistedNodes.contains(node)) {
                     func(node);
                 }
@@ -259,7 +260,7 @@ MoveStrokeStrategy::MoveStrokeStrategy(const MoveStrokeStrategy &rhs, int lod)
 void MoveStrokeStrategy::initStrokeCallback()
 {
     /**
-     * Our LodN moght have already prepared the list of nodes for us,
+     * Our LodN might have already prepared the list of nodes for us,
      * so we should reuse it to avoid different nodes to be moved in
      * LodN and Lod0 modes.
      */
@@ -267,7 +268,7 @@ void MoveStrokeStrategy::initStrokeCallback()
         m_nodes = m_requestedNodeSelection.selectNodesToProcess();
 
         if (!m_nodes.isEmpty()) {
-            m_nodes = KisLayerUtils::sortAndFilterMergableInternalNodes(m_nodes, true);
+            m_nodes = KisLayerUtils::sortAndFilterMergeableInternalNodes(m_nodes, true);
         }
 
         KritaUtils::filterContainer<KisNodeList>(m_nodes,
@@ -306,25 +307,6 @@ void MoveStrokeStrategy::initStrokeCallback()
 
     QVector<KisRunnableStrokeJobData*> jobs;
 
-    if (KisAutoKey::activeMode() > KisAutoKey::NONE) {
-        KritaUtils::addJobBarrier(jobs, [this]() {
-            Q_FOREACH(KisNodeSP node, m_nodes) {
-                if (node->hasEditablePaintDevice()) {
-                        // Try to create a copy keyframe if available.
-                        KisPaintDeviceSP device = node->paintDevice();
-                        KIS_ASSERT(device);
-                        if (device->keyframeChannel()) {
-                            KUndo2CommandSP undo(new KUndo2Command);
-                            const int activeKeyframe = device->keyframeChannel()->activeKeyframeTime();
-                            const int targetKeyframe = node->image()->animationInterface()->currentTime();
-                            device->keyframeChannel()->copyKeyframe(activeKeyframe, targetKeyframe, undo.data());
-                            runAndSaveCommand(undo, KisStrokeJobData::BARRIER, KisStrokeJobData::NORMAL);
-                        }
-                    }
-                }
-        });
-    }
-
     KritaUtils::addJobBarrier(jobs, [this]() {
         Q_FOREACH(KisNodeSP node, m_nodes) {
             KisLayerUtils::forceAllHiddenOriginalsUpdate(node);
@@ -348,6 +330,19 @@ void MoveStrokeStrategy::initStrokeCallback()
                 handlesRect |= node->projectionPlane()->tightUserVisibleBounds();
             });
 
+        KisStrokeStrategyUndoCommandBased::initStrokeCallback();
+
+        Q_FOREACH(KisNodeSP node, m_nodes) {
+            KIS_SAFE_ASSERT_RECOVER(node->hasEditablePaintDevice()) { continue; }
+
+            KUndo2Command *autoKeyframeCommand =
+                KisAutoKey::tryAutoCreateDuplicatedFrame(node->paintDevice(),
+                                                         KisAutoKey::SupportsLod);
+            if (autoKeyframeCommand) {
+                runAndSaveCommand(toQShared(autoKeyframeCommand), KisStrokeJobData::BARRIER, KisStrokeJobData::NORMAL);
+            }
+        }
+
         /**
          * Create strategies and start the transactions when necessary
          */
@@ -362,8 +357,6 @@ void MoveStrokeStrategy::initStrokeCallback()
                 }
             });
 
-        KisStrokeStrategyUndoCommandBased::initStrokeCallback();
-
         if (m_updatesEnabled) {
             KisLodTransform t(m_nodes.first()->image()->currentLevelOfDetail());
             handlesRect = t.mapInverted(handlesRect);
@@ -429,7 +422,7 @@ void MoveStrokeStrategy::cancelStrokeCallback()
                 m_dirtyRects[node] |= dirtyRect;
 
                 /// emit updates not looking onto the
-                /// epdatesEnabled switch, since that is
+                /// updatesEnabled switch, since that is
                 /// the end of the stroke
                 m_updatesFacade->refreshGraphAsync(node, dirtyRect);
             }
@@ -478,8 +471,8 @@ void MoveStrokeStrategy::doStrokeCallback(KisStrokeJobData *data)
 
         doCanvasUpdate(barrierData->forceUpdate);
 
-    } else if (KisAsyncronousStrokeUpdateHelper::UpdateData *updateData =
-               dynamic_cast<KisAsyncronousStrokeUpdateHelper::UpdateData*>(data)) {
+    } else if (KisAsynchronousStrokeUpdateHelper::UpdateData *updateData =
+               dynamic_cast<KisAsynchronousStrokeUpdateHelper::UpdateData*>(data)) {
 
         tryPostUpdateJob(updateData->forceUpdate);
 
@@ -606,7 +599,7 @@ MoveStrokeStrategy::PickLayerData::PickLayerData(const MoveStrokeStrategy::PickL
 }
 
 MoveStrokeStrategy::BarrierUpdateData::BarrierUpdateData(bool _forceUpdate)
-    : KisAsyncronousStrokeUpdateHelper::UpdateData(_forceUpdate, BARRIER, EXCLUSIVE)
+    : KisAsynchronousStrokeUpdateHelper::UpdateData(_forceUpdate, BARRIER, EXCLUSIVE)
 {}
 
 KisStrokeJobData *MoveStrokeStrategy::BarrierUpdateData::createLodClone(int levelOfDetail) {
@@ -614,6 +607,6 @@ KisStrokeJobData *MoveStrokeStrategy::BarrierUpdateData::createLodClone(int leve
 }
 
 MoveStrokeStrategy::BarrierUpdateData::BarrierUpdateData(const MoveStrokeStrategy::BarrierUpdateData &rhs, int levelOfDetail)
-    : KisAsyncronousStrokeUpdateHelper::UpdateData(rhs, levelOfDetail)
+    : KisAsynchronousStrokeUpdateHelper::UpdateData(rhs, levelOfDetail)
 {
 }
