@@ -30,6 +30,7 @@
 #include <KoColorSlider.h>
 #include <KoColorSpace.h>
 #include <KisHsvColorSlider.h>
+#include <KoColorConversions.h>
 
 #include "kis_double_parse_spin_box.h"
 #include "kis_int_parse_spin_box.h"
@@ -442,13 +443,14 @@ KisHsvColorInput::KisHsvColorInput(QWidget *parent, KoColor *color)
     , m_color(color)
     , m_hSlider(nullptr)
     , m_sSlider(nullptr)
-    , m_vSlider(nullptr)
+    , m_xSlider(nullptr)
     , m_hInput(nullptr)
     , m_sInput(nullptr)
-    , m_vInput(nullptr)
+    , m_xInput(nullptr)
     , m_h(0)
     , m_s(0)
-    , m_v(0)
+    , m_x(0)
+    , m_mixMode(KisHsvColorSlider::MIX_MODE::HSV)
 {
 
     QLabel *labels[3];
@@ -473,6 +475,7 @@ KisHsvColorInput::KisHsvColorInput(QWidget *parent, KoColor *color)
 
         // Slider itself
         KisHsvColorSlider *slider = new KisHsvColorSlider(Qt::Horizontal, this);
+        slider->setMixMode(m_mixMode);
         slider->setMinimum(0);
         slider->setMaximum(maxValues[i]);
         slider->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
@@ -514,19 +517,21 @@ KisHsvColorInput::KisHsvColorInput(QWidget *parent, KoColor *color)
     connect(inputs[2], SIGNAL(valueChanged(double)), this, SLOT(setValue(double)));
 
     m_hSlider = sliders[0];
+    m_hSlider->setMixMode(KisHsvColorSlider::MIX_MODE::HSV);
     m_sSlider = sliders[1];
-    m_vSlider = sliders[2];
+    m_xSlider = sliders[2];
 
     m_hInput = inputs[0];
     m_sInput = inputs[1];
-    m_vInput = inputs[2];
+    m_xInput = inputs[2];
 
     // Set initial values
     QColor c = m_color->toQColor();
-    c.getHsvF(&m_h, &m_s, &m_v);
+    getHsxF(c, &m_h, &m_s, &m_x);
+
     m_hInput->setValue(m_h);
     m_sInput->setValue(m_s);
-    m_vInput->setValue(m_v);
+    m_xInput->setValue(m_x);
 
     // Update sliders
     QColor minC, maxC;
@@ -538,21 +543,37 @@ KisHsvColorInput::KisHsvColorInput(QWidget *parent, KoColor *color)
     recolorSliders();
 }
 
+void KisHsvColorInput::setMixMode(KisHsvColorSlider::MIX_MODE mixMode) {
+	QColor c = m_color->toQColor();
+	m_mixMode = mixMode;
+	getHsxF(c, &m_h, &m_s, &m_x);
+
+	m_sSlider->setMixMode(m_mixMode);
+	m_xSlider->setMixMode(m_mixMode);
+
+    sendUpdate();
+}
+
 void KisHsvColorInput::sendUpdate()
 {
     {
         KisSignalsBlocker blocker(
-            m_hSlider, m_sSlider, m_vSlider
+            m_hSlider, m_sSlider, m_xSlider,
+            m_hInput, m_sInput, m_xInput
         );
         m_hSlider->setValue(m_h * 360);
         m_sSlider->setValue(m_s * 100);
-        m_vSlider->setValue(m_v * 100);
+        m_xSlider->setValue(m_x * 100);
+
+		m_hInput->setValue(m_h * 360);
+		m_sInput->setValue(m_s * 100);
+		m_xInput->setValue(m_x * 100);
     }
 
     recolorSliders();
 
-    QColor c;
-    c.setHsvF(m_h, m_s, m_v);
+	QColor c;
+    fillColor(c);
 
     m_color->fromQColor(c);
     emit(updated());
@@ -596,7 +617,7 @@ void KisHsvColorInput::setValue(double x)
         x = 100;
     }
 
-    m_v = x / 100;
+    m_x = x / 100;
     sendUpdate();
 }
 
@@ -612,38 +633,41 @@ void KisHsvColorInput::saturationSliderChanged(int i)
 
 void KisHsvColorInput::valueSliderChanged(int i)
 {
-    m_vInput->setValue(i);
+    m_xInput->setValue(i);
 }
 
 void KisHsvColorInput::recolorSliders() {
     // Update sliders
     QColor minC, maxC;
-    minC.setHsvF(m_h, 0, m_v);
-    maxC.setHsvF(m_h, 1, m_v);
+
+    minC.setHsvF(m_h, 0, m_x);
+    maxC.setHsvF(m_h, 1, m_x);
     m_sSlider->setColors(minC, maxC);
 
     minC.setHsvF(m_h, m_s, 0);
     maxC.setHsvF(m_h, m_s, 1);
-    m_vSlider->setColors(minC, maxC);
+    m_xSlider->setColors(minC, maxC);
 }
 
 void KisHsvColorInput::update()
 {
     KisSignalsBlocker blocker(
-        m_hInput, m_sInput, m_vInput,
-        m_hSlider, m_sSlider, m_vSlider
+        m_hInput, m_sInput, m_xInput,
+        m_hSlider, m_sSlider, m_xSlider
     );
 
     // Check if it is the same color we have
     QColor current;
-    current.setHsvF(m_h, m_s, m_v);
+
+    fillColor(current);
+
     QColor theirs = m_color->toQColor();
 
     // Truncate to integer for this check
     if (!(current.red() == theirs.red() && current.green() == theirs.green() && current.blue() == theirs.blue())) {
         // Apply the update
         qreal theirH;
-        theirs.getHsvF(&theirH, &m_s, &m_v);
+        getHsxF(theirs, &theirH, &m_s, &m_x);
 
         // Don't jump the Hue slider around to 0 if it is currently on 360
         const qreal EPSILON = 1e-6;
@@ -653,13 +677,80 @@ void KisHsvColorInput::update()
 
         m_hInput->setValue(m_h * 360);
         m_sInput->setValue(m_s * 100);
-        m_vInput->setValue(m_v * 100);
+        m_xInput->setValue(m_x * 100);
 
         recolorSliders();
 
         // Update slider positions
         m_hSlider->setValue(m_h * 360);
         m_sSlider->setValue(m_s * 100);
-        m_vSlider->setValue(m_v * 100);
+        m_xSlider->setValue(m_x * 100);
+    }
+}
+
+void KisHsvColorInput::fillColor(QColor& c) {
+    fillColor(c, m_h, m_s, m_x);
+}
+
+void KisHsvColorInput::fillColor(QColor& c, const qreal& h, const qreal& s, const qreal& x)
+{
+	switch (m_mixMode) {
+	case KisHsvColorSlider::MIX_MODE::HSL:
+		c.setHslF(h, s, x);
+		break;
+
+	case KisHsvColorSlider::MIX_MODE::HSY: {
+		qreal r, g, b;
+		HSYToRGB(h, s, x, &r, &g, &b);
+
+		// Clamp
+		r = qBound(0.0, r, 1.0);
+		g = qBound(0.0, g, 1.0);
+		b = qBound(0.0, b, 1.0);
+
+		c.setRgbF(r, g, b);
+		break;
+	}
+
+	case KisHsvColorSlider::MIX_MODE::HSI: {
+		qreal r, g, b;
+		HSIToRGB(h, s, x, &r, &g, &b);
+		c.setRgbF(r, g, b);
+		break;
+	}
+
+	default: // fallthrough
+	case KisHsvColorSlider::MIX_MODE::HSV:
+		c.setHsvF(h, s, x);
+		break;
+	}
+}
+
+void KisHsvColorInput::getHsxF(const QColor& color, qreal* h, qreal* s, qreal* x)
+{
+    qreal tempH;
+	switch (m_mixMode) {
+	case KisHsvColorSlider::MIX_MODE::HSL:
+        color.getHslF(&tempH, s, x);
+		break;
+
+	case KisHsvColorSlider::MIX_MODE::HSY: {
+		RGBToHSY(color.redF(), color.greenF(), color.blueF(), &tempH, s, x);
+		break;
+	}
+
+	case KisHsvColorSlider::MIX_MODE::HSI: {
+		RGBToHSI(color.redF(), color.greenF(), color.blueF(), &tempH, s, x);
+		break;
+	}
+
+	default: // fallthrough
+	case KisHsvColorSlider::MIX_MODE::HSV:
+		color.getHsvF(&tempH, s, x);
+		break;
+	}
+
+    if (tempH >= 0.0 && tempH <= 1.0) {
+        *h = tempH;
     }
 }
