@@ -99,7 +99,7 @@ public:
 private:
     QStringList m_pinnedFonts;
     QStringList m_blacklistedFonts;
-    bool m_initilized {false};
+    bool m_initialized {false};
     bool m_initializeFromConfig;
     int m_separatorIndex;
     PinnedFontsSeparator *m_fontSeparator;
