@@ -189,6 +189,9 @@ public:
      */
     void setSoftMaximum(int newSoftMaximum);
 
+Q_SIGNALS:
+    void draggingFinished();
+
 protected:
     virtual void setInternalValue(int value, bool blockUpdateSignal);
     void setPrivateValue(int value);
@@ -247,10 +250,13 @@ protected:
     virtual void setInternalValue(qreal newValue, bool newBlockUpdateSignal);
     void setPrivateValue(qreal newValue);
 
+Q_SIGNALS:
+    void draggingFinished();
+
 private:
     template <typename SpinBoxTypeTP, typename BaseSpinBoxTypeTP>
     friend class KisSliderSpinBoxPrivate;
     QScopedPointer<KisSliderSpinBoxPrivate<KisDoubleSliderSpinBox, KisDoubleParseSpinBox>> d;
 };
 
-#endif //kISSLIDERSPINBOX_H
+#endif // KISSLIDERSPINBOX_H
