@@ -1,22 +1,10 @@
 /*
- * Copyright (c) 2008 Cyrille Berger <cberger@cberger.net>
- * Copyright (c) 2010 Geoffry Song <goffrie@gmail.com>
- * Copyright (c) 2014 Wolthera van Hövell tot Westerflier <griffinvalley@gmail.com>
- * Copyright (c) 2017 Scott Petrovic <scottpetrovic@gmail.com>
+ * SPDX-FileCopyrightText: 2008 Cyrille Berger <cberger@cberger.net>
+ * SPDX-FileCopyrightText: 2010 Geoffry Song <goffrie@gmail.com>
+ * SPDX-FileCopyrightText: 2014 Wolthera van Hövell tot Westerflier <griffinvalley@gmail.com>
+ * SPDX-FileCopyrightText: 2017 Scott Petrovic <scottpetrovic@gmail.com>
  *
- *  This library is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License as published by
- *  the Free Software Foundation; version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Lesser General Public License for more details.
- *
- *  You should have received a copy of the GNU Lesser General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
 #include "ParallelRulerAssistant.h"
@@ -32,13 +20,12 @@
 #include <kis_canvas2.h>
 #include <kis_coordinates_converter.h>
 #include <kis_algebra_2d.h>
+#include <kis_dom_utils.h>
 
 #include <math.h>
 
 ParallelRulerAssistant::ParallelRulerAssistant()
     : KisPaintingAssistant("parallel ruler", i18n("Parallel Ruler assistant"))
-    , m_followBrushPosition(false)
-    , m_adjustedPositionValid(false)
 {
 }
 
@@ -49,32 +36,10 @@ KisPaintingAssistantSP ParallelRulerAssistant::clone(QMap<KisPaintingAssistantHa
 
 ParallelRulerAssistant::ParallelRulerAssistant(const ParallelRulerAssistant &rhs, QMap<KisPaintingAssistantHandleSP, KisPaintingAssistantHandleSP> &handleMap)
     : KisPaintingAssistant(rhs, handleMap)
-    , m_followBrushPosition(rhs.m_followBrushPosition)
-    , m_adjustedPositionValid(rhs.m_adjustedPositionValid)
-    , m_adjustedBrushPosition(rhs.m_adjustedBrushPosition)
 {
 }
 
-void ParallelRulerAssistant::setAdjustedBrushPosition(const QPointF position)
-{
-    m_adjustedBrushPosition = position;
-    m_adjustedPositionValid = true;
-}
-
-void ParallelRulerAssistant::endStroke()
-{
-    // Brush stroke ended, guides should follow the brush position again.
-    m_followBrushPosition = false;
-    m_adjustedPositionValid = false;
-}
-
-
-void ParallelRulerAssistant::setFollowBrushPosition(bool follow)
-{
-    m_followBrushPosition = follow;
-}
-
-QPointF ParallelRulerAssistant::project(const QPointF& pt, const QPointF& strokeBegin)
+QPointF ParallelRulerAssistant::project(const QPointF& pt, const QPointF& strokeBegin, qreal moveThresholdPt)
 {
     Q_ASSERT(isAssistantComplete());
 
@@ -82,10 +47,18 @@ QPointF ParallelRulerAssistant::project(const QPointF& pt, const QPointF& stroke
     qreal dx = pt.x() - strokeBegin.x();
     qreal dy = pt.y() - strokeBegin.y();
 
-    if (dx * dx + dy * dy < 4.0) {
+    if (KisAlgebra2D::norm(QPointF(dx, dy)) < moveThresholdPt) {
         return strokeBegin; // allow some movement before snapping
     }
 
+    if (isLocal() && isAssistantComplete()) {
+        if (getLocalRect().contains(pt)) {
+            m_hasBeenInsideLocalRect = true;
+        } else if (isLocal() && !m_hasBeenInsideLocalRect) {
+            return QPointF(qQNaN(), qQNaN());
+        }
+    }
+
     //dbgKrita<<strokeBegin<< ", " <<*handles()[0];
     QLineF snapLine = QLineF(*handles()[0], *handles()[1]);
     QPointF translation = (*handles()[0]-strokeBegin)*-1.0;
@@ -105,9 +78,14 @@ QPointF ParallelRulerAssistant::project(const QPointF& pt, const QPointF& stroke
     //return pt;
 }
 
-QPointF ParallelRulerAssistant::adjustPosition(const QPointF& pt, const QPointF& strokeBegin)
+QPointF ParallelRulerAssistant::adjustPosition(const QPointF& pt, const QPointF& strokeBegin, const bool /*snapToAny*/, qreal moveThresholdPt)
 {
-    return project(pt, strokeBegin);
+    return project(pt, strokeBegin, moveThresholdPt);
+}
+
+void ParallelRulerAssistant::adjustLine(QPointF &point, QPointF &strokeBegin)
+{
+    point = project(point, strokeBegin, 0.0);
 }
 
 void ParallelRulerAssistant::drawAssistant(QPainter& gc, const QRectF& updateRect, const KisCoordinatesConverter* converter, bool cached, KisCanvas2* canvas, bool assistantVisible, bool previewVisible)
@@ -126,9 +104,30 @@ void ParallelRulerAssistant::drawAssistant(QPainter& gc, const QRectF& updateRec
         dbgFile<<"canvas does not exist in ruler, you may have passed arguments incorrectly:"<<canvas;
     }
 
+
+    QTransform initialTransform = converter->documentToWidgetTransform();
+    QRectF local = getLocalRect();
+    QRectF localTransformed = initialTransform.mapRect(local);
+    const QRect viewport= gc.viewport();
+    QPolygonF viewportAndLocal = !localTransformed.isEmpty() ? QPolygonF(QRectF(viewport)).intersected(localTransformed) : QRectF(viewport);
+
+
+
+    if (assistantVisible && isLocal() && isAssistantComplete()) {
+        QPainterPath path;
+        // note: be careful; bottom and right only work with RectF, not Rect
+        path.moveTo(initialTransform.map(local.topLeft()));
+
+        path.lineTo(initialTransform.map(local.topRight()));
+        path.lineTo(initialTransform.map(local.bottomRight()));
+        path.lineTo(initialTransform.map(local.bottomLeft()));
+        path.lineTo(initialTransform.map(local.topLeft()));
+        drawPath(gc, path, isSnappingActive());//and we draw the preview.
+    }
+
+
     if (isAssistantComplete() && isSnappingActive() && previewVisible==true) {
         //don't draw if invalid.
-        QTransform initialTransform = converter->documentToWidgetTransform();
         QLineF snapLine= QLineF(initialTransform.map(*handles()[0]), initialTransform.map(*handles()[1]));
 
         if (m_followBrushPosition && m_adjustedPositionValid) {
@@ -138,8 +137,7 @@ void ParallelRulerAssistant::drawAssistant(QPainter& gc, const QRectF& updateRec
         QPointF translation = (initialTransform.map(*handles()[0])-mousePos)*-1.0;
         snapLine= snapLine.translated(translation);
 
-        QRect viewport= gc.viewport();
-        KisAlgebra2D::intersectLineRect(snapLine, viewport);
+        KisAlgebra2D::cropLineToConvexPolygon(snapLine, viewportAndLocal, true, true);
 
 
         QPainterPath path;
@@ -156,7 +154,7 @@ void ParallelRulerAssistant::drawAssistant(QPainter& gc, const QRectF& updateRec
 
 void ParallelRulerAssistant::drawCache(QPainter& gc, const KisCoordinatesConverter *converter, bool assistantVisible)
 {
-    if (assistantVisible == false || !isAssistantComplete()){
+    if (assistantVisible == false || handles().size() < 2) {
         return;
     }
 
@@ -174,14 +172,52 @@ void ParallelRulerAssistant::drawCache(QPainter& gc, const KisCoordinatesConvert
 
 }
 
-QPointF ParallelRulerAssistant::getEditorPosition() const
+KisPaintingAssistantHandleSP ParallelRulerAssistant::firstLocalHandle() const
 {
-    return (*handles()[0] + *handles()[1]) * 0.5;
+    return handles().size() > 2 ? handles()[2] : 0;
+}
+
+KisPaintingAssistantHandleSP ParallelRulerAssistant::secondLocalHandle() const
+{
+    return handles().size() > 3 ? handles()[3] : 0;
+}
+
+QPointF ParallelRulerAssistant::getDefaultEditorPosition() const
+{
+    if (handles().size() > 1) {
+        return (*handles()[0] + *handles()[1]) * 0.5;
+    } else if (handles().size() > 0) {
+        KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(false, *handles()[0]);
+        return *handles()[0];
+    } else {
+        KIS_SAFE_ASSERT_RECOVER_RETURN_VALUE(false, QPointF(0, 0));
+        return QPointF(0, 0);
+    }
 }
 
 bool ParallelRulerAssistant::isAssistantComplete() const
 {
-    return handles().size() >= 2;
+    return handles().size() >= numHandles();
+}
+
+bool ParallelRulerAssistant::canBeLocal() const
+{
+    return true;
+}
+
+void ParallelRulerAssistant::saveCustomXml(QXmlStreamWriter *xml)
+{
+    xml->writeStartElement("isLocal");
+    xml->writeAttribute("value", KisDomUtils::toString( (int)this->isLocal()));
+    xml->writeEndElement();
+}
+
+bool ParallelRulerAssistant::loadCustomXml(QXmlStreamReader *xml)
+{
+    if (xml && xml->name() == "isLocal") {
+        this->setLocal((bool)KisDomUtils::toInt(xml->attributes().value("value").toString()));
+    }
+    return true;
 }
 
 ParallelRulerAssistantFactory::ParallelRulerAssistantFactory()
