@@ -16,6 +16,8 @@
 #include <QLineF>
 #include <QTransform>
 
+#include <PerspectiveBasedAssistantHelper.h>
+
 class PerspectiveAssistant : public KisAbstractPerspectiveGrid, public KisPaintingAssistant
 {
     Q_OBJECT
@@ -23,17 +25,17 @@ public:
     PerspectiveAssistant(QObject * parent = 0);
     KisPaintingAssistantSP clone(QMap<KisPaintingAssistantHandleSP, KisPaintingAssistantHandleSP> &handleMap) const override;
 
-    QPointF adjustPosition(const QPointF& point, const QPointF& strokeBegin, const bool snapToAny) override;
-    void setAdjustedBrushPosition(const QPointF position) override;
-    void setFollowBrushPosition(bool follow) override;
+    QPointF adjustPosition(const QPointF& point, const QPointF& strokeBegin, const bool snapToAny, qreal moveThresholdPt) override;
+    void adjustLine(QPointF &point, QPointF& strokeBegin) override;
     void endStroke() override;
 
-    QPointF getEditorPosition() const override;
+    QPointF getDefaultEditorPosition() const override;
     int numHandles() const override { return 4; }
     void drawAssistant(QPainter& gc, const QRectF& updateRect, const KisCoordinatesConverter* converter, bool cached = true,KisCanvas2* canvas=0, bool assistantVisible=true, bool previewVisible=true) override;
 
     bool contains(const QPointF& point) const override;
     qreal distance(const QPointF& point) const override;
+    bool isActive() const override;
 
     int subdivisions() const;
     void setSubdivisions(int subdivisions);
@@ -46,9 +48,8 @@ public:
 protected:
     void drawCache(QPainter& gc, const KisCoordinatesConverter *converter,  bool assistantVisible=true) override;
 private:
-    QPointF project(const QPointF& pt, const QPointF& strokeBegin);
+    QPointF project(const QPointF& pt, const QPointF& strokeBegin, const bool alwaysStartAnew, qreal moveThresholdPt);
     // creates the convex hull, returns false if it's not a quadrilateral
-    bool quad(QPolygonF& out) const;
     // finds the transform from perspective coordinates (a unit square) to the document
     bool getTransform(QPolygonF& polyOut, QTransform& transformOut) const;
     explicit PerspectiveAssistant(const PerspectiveAssistant &rhs, QMap<KisPaintingAssistantHandleSP, KisPaintingAssistantHandleSP> &handleMap);
@@ -62,11 +63,9 @@ private:
     mutable QPolygonF m_cachedPolygon;
     mutable QPointF m_cachedPoints[4];
     mutable bool m_cacheValid {false};
-    // Needed to make sure that when we are in the middle of a brush stroke, the
-    // guides follow the brush position, not the cursor position.
-    bool m_followBrushPosition;
-    bool m_adjustedPositionValid;
-    QPointF m_adjustedBrushPosition;
+
+    mutable PerspectiveBasedAssistantHelper::CacheData m_cache;
+
 };
 
 class PerspectiveAssistantFactory : public KisPaintingAssistantFactory
